@@ -2,7 +2,11 @@ from arcgis.apps.dashboard.dashboard import Dashboard
 from arcgis.apps.dashboard._serialchart._serial_chart import SerialChart
 from arcgis.apps.dashboard._header._header import Header
 from arcgis.apps.dashboard._sidepanel.side_panel import SidePanel
-from arcgis.apps.dashboard._side_widgets._addon import CategorySelector, NumberSelector, DatePicker
+from arcgis.apps.dashboard._side_widgets._addon import (
+    CategorySelector,
+    NumberSelector,
+    DatePicker,
+)
 from arcgis.apps.dashboard._embcontent._embcontent import EmbeddedContent
 from arcgis.apps.dashboard._richtext._richtext import RichText
 from arcgis.apps.dashboard._details._details import Details
@@ -11,4 +15,4 @@ from arcgis.apps.dashboard._piechart._pie_chart import PieChart
 from arcgis.apps.dashboard._map_legend._map_legend import MapLegend
 from arcgis.apps.dashboard._gauge._gauge import Gauge
 from arcgis.apps.dashboard._list._list import List
-from arcgis.apps.dashboard._utils._basewidget import add_row, add_column
\ No newline at end of file
+from arcgis.apps.dashboard._utils._basewidget import add_row, add_column
