@@ -30,10 +30,19 @@ class PieChart(_BaseWidget):
     description                 Optional string. Description for the widget.
     =========================   ===========================================
     """
-    def __init__(self, item, name='PieChart', layer=0, categories_from="groupByValues", title='', description=''):
+
+    def __init__(
+        self,
+        item,
+        name="PieChart",
+        layer=0,
+        categories_from="groupByValues",
+        title="",
+        description="",
+    ):
         super().__init__(name, title, description)
 
-        if item.type not in ['Feature Service', 'mapWidget']:
+        if item.type not in ["Feature Service", "mapWidget"]:
             raise Exception("Please specify an item")
 
         self.item = item
@@ -49,7 +58,7 @@ class PieChart(_BaseWidget):
         if categories_from not in ["groupByValues", "features", "fields"]:
             raise Exception("category_from can be groupByValues or features or fields")
 
-        self._data = PieChartData._create_data(categories_from, data_item = item)
+        self._data = PieChartData._create_data(categories_from, data_item=item)
         self._pie = PieChartProperties._create_chart()
         self._slices = SliceProperties._slice_properties()
         self._nodata = NoDataProperties._nodata_init()
@@ -82,21 +91,21 @@ class PieChart(_BaseWidget):
         :return: Pie Chart Data object. Set data properties, categories and values.
         """
         return self._data
-    
+
     @property
     def legend(self):
         """
         :return: Legend Object, set Visibility and placement
         """
         return self._legend
-    
+
     @property
     def pie(self):
         """
         :return: Pie Object, set various pie properties
         """
         return self._pie
-    
+
     @property
     def outline(self):
         """
@@ -162,10 +171,10 @@ class PieChart(_BaseWidget):
 
     def _color_picker(self):
         import random
-        
-        random_number = random.randint(0,16777215)
+
+        random_number = random.randint(0, 16777215)
         hex_number = str(hex(random_number))
-        hex_number ='#'+ hex_number[2:]
+        hex_number = "#" + hex_number[2:]
 
         return hex_number
 
@@ -181,16 +190,46 @@ class PieChart(_BaseWidget):
             self._category = None
             self._field_name = "category"
             for category_field in self._data.category_field:
-                self._fields_slices.append({"key":str(category_field), "label":str(category_field), "color":self._color_picker()})
-                self._statistic_fields.append({"onStatisticField":str(category_field), "outStatisticFieldName":str(category_field), "statisticType":self.data.statistic})
+                self._fields_slices.append(
+                    {
+                        "key": str(category_field),
+                        "label": str(category_field),
+                        "color": self._color_picker(),
+                    }
+                )
+                self._statistic_fields.append(
+                    {
+                        "onStatisticField": str(category_field),
+                        "outStatisticFieldName": str(category_field),
+                        "statisticType": self.data.statistic,
+                    }
+                )
         elif self._data.categories_from == "groupByValues":
             for slices in self.data._slice_fields:
-                self._fields_slices.append({"key":str(slices), "label":str(slices), "color":self._color_picker()})
-            self._statistic_fields.append({"onStatisticField":str(self.data.statistics_field), "outStatisticFieldName":"value", "statisticType":self.data.statistic})
+                self._fields_slices.append(
+                    {
+                        "key": str(slices),
+                        "label": str(slices),
+                        "color": self._color_picker(),
+                    }
+                )
+            self._statistic_fields.append(
+                {
+                    "onStatisticField": str(self.data.statistics_field),
+                    "outStatisticFieldName": "value",
+                    "statisticType": self.data.statistic,
+                }
+            )
             self._category = self._data.category_field
         else:
             for slices in self.data._slice_fields:
-                self._fields_slices.append({"key":str(slices), "label":str(slices), "color":self._color_picker()})
+                self._fields_slices.append(
+                    {
+                        "key": str(slices),
+                        "label": str(slices),
+                        "color": self._color_picker(),
+                    }
+                )
             self._category = self._data.category_field
 
         if self.labels == True:
@@ -198,96 +237,96 @@ class PieChart(_BaseWidget):
         else:
             self._labels_format = "hide"
 
-        if self.item.type == 'mapWidget':
+        if self.item.type == "mapWidget":
             wlayer = self.item.layers[self.layer]
             widget_id = self.item._id
             layer_id = wlayer["id"]
-            self._datasource = {"id":str(widget_id)+'#'+str(layer_id)}
+            self._datasource = {"id": str(widget_id) + "#" + str(layer_id)}
             self._targetid = self._datasource
         else:
             self._datasource = {
-                        "type": "featureServiceDataSource",
-                        "itemId": self.item.itemid,
-                        "layerId": 0,
-                        "table": True
-                    }
+                "type": "featureServiceDataSource",
+                "itemId": self.item.itemid,
+                "layerId": 0,
+                "table": True,
+            }
 
         json_data = {
             "type": "pieChartWidget",
             "category": {
                 "sliceProperties": self._fields_slices,
                 "fieldName": self._category if self._category else self._field_name,
-                "nullLabel":self._slices.null_label,
-                "blankLabel":self._slices.blank_label,
-                "defaultColor":self._slices.default_color,
-                "nullColor":self._slices.null_color,
-                "blankColor":self._slices.blank_color
+                "nullLabel": self._slices.null_label,
+                "blankLabel": self._slices.blank_label,
+                "defaultColor": self._slices.default_color,
+                "nullColor": self._slices.null_color,
+                "blankColor": self._slices.blank_color,
             },
             "pie": {
-                "type":"pie",
-                "color":self._pie.text_color,
-                "fontSize":self._pie.font_size,
-                "titleField":"category",
-                "valueField":"absoluteValue",
-                "alpha":self._slices.opacity,
-                "outlineAlpha":self._outline.opacity,
-                "outlineColor":self._outline.color,
-                "outlineThickness":self._outline.thickness,
-                "innerRadius":self._pie.inner_radius,
-                "labelsEnabled":self.labels,
-                "labelsFormat":self._labels_format,
-                "labelTickAlpha":0.5,
-                "labelTickColor":"#fab123",
-                "maxLabelWidth":100,
-                "startAngle":self._pie.start_angle,
-                "autoMargins":False,
-                "marginTop":0,
-                "marginBottom":0,
-                "marginLeft":0,
-                "marginRight":0,
-                "groupPercent":self._slices.grouping_percent,
-                "groupedColor":self._slices.grouping_color
+                "type": "pie",
+                "color": self._pie.text_color,
+                "fontSize": self._pie.font_size,
+                "titleField": "category",
+                "valueField": "absoluteValue",
+                "alpha": self._slices.opacity,
+                "outlineAlpha": self._outline.opacity,
+                "outlineColor": self._outline.color,
+                "outlineThickness": self._outline.thickness,
+                "innerRadius": self._pie.inner_radius,
+                "labelsEnabled": self.labels,
+                "labelsFormat": self._labels_format,
+                "labelTickAlpha": 0.5,
+                "labelTickColor": "#fab123",
+                "maxLabelWidth": 100,
+                "startAngle": self._pie.start_angle,
+                "autoMargins": False,
+                "marginTop": 0,
+                "marginBottom": 0,
+                "marginLeft": 0,
+                "marginRight": 0,
+                "groupPercent": self._slices.grouping_percent,
+                "groupedColor": self._slices.grouping_color,
             },
-            #"valueField":self._data.value_field,
-            "legend":{
-                "enabled":self._legend.visibility,
-                "format":"value",
-                "position":self._legend.placement,
-                "markerSize":15,
-                "markerType":"circle",
-                "align":"center",
-                "labelWidth":100,
-                "valueWidth":50
+            # "valueField":self._data.value_field,
+            "legend": {
+                "enabled": self._legend.visibility,
+                "format": "value",
+                "position": self._legend.placement,
+                "markerSize": 15,
+                "markerType": "circle",
+                "align": "center",
+                "labelWidth": 100,
+                "valueWidth": 50,
             },
-            "showBalloon":True,
-            "valueFormat":{
-                "name":"value",
-                "type":"decimal",
-                "prefix":True,
-                "pattern":"#,###.#"
+            "showBalloon": True,
+            "valueFormat": {
+                "name": "value",
+                "type": "decimal",
+                "prefix": True,
+                "pattern": "#,###.#",
             },
-            "percentageFormat":{
-                "name":"percentage",
-                "type":"decimal",
-                "prefix":False,
-                "pattern":"#.##"
+            "percentageFormat": {
+                "name": "percentage",
+                "type": "decimal",
+                "prefix": False,
+                "pattern": "#.##",
             },
-            "events":[],
-            "selectionMode":"single",
-            "categoryType":self._data.categories_from,
-            "datasets":[
-                    {
-                    "type":"serviceDataset",
-                    "dataSource":self._datasource,
-                    "outFields":["*"],
-                    "groupByFields":[self._category] if self._category else [],
-                    "orderByFields":[],
-                    "statisticDefinitions":self._statistic_fields,
+            "events": [],
+            "selectionMode": "single",
+            "categoryType": self._data.categories_from,
+            "datasets": [
+                {
+                    "type": "serviceDataset",
+                    "dataSource": self._datasource,
+                    "outFields": ["*"],
+                    "groupByFields": [self._category] if self._category else [],
+                    "orderByFields": [],
+                    "statisticDefinitions": self._statistic_fields,
                     "maxFeatures": self.max_features,
-                    "querySpatialRelationship":"esriSpatialRelIntersects",
-                    "returnGeometry":False,
-                    "clientSideStatistics":False,
-                    "name":"main"
+                    "querySpatialRelationship": "esriSpatialRelIntersects",
+                    "returnGeometry": False,
+                    "clientSideStatistics": False,
+                    "name": "main",
                 }
             ],
             "id": self._id,
@@ -295,34 +334,37 @@ class PieChart(_BaseWidget):
             "caption": self.title,
             "description": self.description,
             "showLastUpdate": self.show_last_update,
-            "noDataText":self._nodata.text,
-            "noDataVerticalAlignment":self._nodata.alignment,
-            "showCaptionWhenNoData":self._nodata.show_title,
-            "showDescriptionWhenNoData":self._nodata.show_description
+            "noDataText": self._nodata.text,
+            "noDataVerticalAlignment": self._nodata.alignment,
+            "showCaptionWhenNoData": self._nodata.show_title,
+            "showDescriptionWhenNoData": self._nodata.show_description,
         }
 
         if self.events.enable:
-            json_data["events"].append({"type":self.events.type, "actions":self.events.synced_widgets})
+            json_data["events"].append(
+                {"type": self.events.type, "actions": self.events.synced_widgets}
+            )
             json_data["selectionMode"] = self.events.selection_mode
 
         if self.background_color:
-            json_data["backgroundColor"] = self.background_color,
+            json_data["backgroundColor"] = (self.background_color,)
 
         if self.text_color:
             json_data["textColor"] = self.text_color
 
         if self.events.enable:
-            json_data["events"].append({"type": self.events.type, "actions": self.events.synced_widgets})
+            json_data["events"].append(
+                {"type": self.events.type, "actions": self.events.synced_widgets}
+            )
             json_data["selectionMode"] = self.events.selection_mode
 
-        if hasattr(self._data, '_value_field'):
+        if hasattr(self._data, "_value_field"):
             json_data["valueField"] = self._data._value_field
 
         return json_data
 
 
 class PieChartData(object):
-
     @classmethod
     def _create_data(cls, categories_from, data_item=None):
         piechart_data = PieChartData()
@@ -338,7 +380,7 @@ class PieChartData(object):
         piechart_data._parse_dates = True
         piechart_data._statistic = "count"
         piechart_data._filters = []
-        
+
         piechart_data._slice_fields = []
 
         piechart_data._categories_from = categories_from
@@ -365,14 +407,14 @@ class PieChartData(object):
         Set True to parse input category fields of type date. For groupByValues and features.
         """
         self._parse_dates = bool(value)
-        
+
     @property
     def objectid_field(self):
         """
         :return: object ID field name.
         """
         return self._objectid_field
-    
+
     @property
     def statistics_field(self):
         """
@@ -393,7 +435,7 @@ class PieChartData(object):
         Set category field from dataset. For groupByValues or features pass a single value. For fields pass a list
         """
         self._category_field = value
-            
+
     @property
     def value_field(self):
         """
@@ -407,7 +449,9 @@ class PieChartData(object):
         Set value field from dataset. For features and groupByValues only.
         """
         if self._categories_from in ["fields"]:
-            raise Exception("Can set this attribute for 'features' or 'groupByValues' category only.")
+            raise Exception(
+                "Can set this attribute for 'features' or 'groupByValues' category only."
+            )
 
         self._value_field = value
 
@@ -426,16 +470,16 @@ class PieChartData(object):
         if self._categories_from in ["features"]:
             raise Exception("Can't set this attribute for 'features' category.")
 
-        if value in ['count', 'avg', 'min', 'max', 'stddev', 'sum']:
+        if value in ["count", "avg", "min", "max", "stddev", "sum"]:
             self._statistic = value
-    
+
     @property
     def filters(self):
         """
         :return: filters associated with widget
         """
         return self._filters
-    
+
     def add_filter(self, field, join, condition, **kwargs):
         """
         Add filters associated with widget.
@@ -445,21 +489,54 @@ class PieChartData(object):
             self._filter_join = join
         else:
             raise Exception("Please select from 'AND', 'OR'")
-        if condition in ["between", "not between", "equal", "not equal", "greater than", "greater than or equal", "less than", "less than or equal", "is null", "is not null"]:
+        if condition in [
+            "between",
+            "not between",
+            "equal",
+            "not equal",
+            "greater than",
+            "greater than or equal",
+            "less than",
+            "less than or equal",
+            "is null",
+            "is not null",
+        ]:
             self._filter_condition = condition
         else:
             raise Exception("Please select the right condition")
 
         if condition in ["between", "not between"]:
-            self._val1 = kwargs.get('start')
-            self,_val2 = kwargs.get('end')
-            self._filters.append({"filtertype":self._filter_join, "field":self._filter_field, "operator":self._filter_condition, "start":self._val1, "end":self._val2})
+            self._val1 = kwargs.get("start")
+            self, _val2 = kwargs.get("end")
+            self._filters.append(
+                {
+                    "filtertype": self._filter_join,
+                    "field": self._filter_field,
+                    "operator": self._filter_condition,
+                    "start": self._val1,
+                    "end": self._val2,
+                }
+            )
         else:
             raise Exception("Please provide 'start' and 'end' values as parameters")
 
-        if condition in ["equal", "not equal", "greater than", "greater than or equal", "less than", "less than or equal"]:
-            self._val = kwargs.get('value')
-            self._filters.append({"filtertype":self._filter_join, "field":self._filter_field, "operator":self._filter_condition, "value":self._val})
+        if condition in [
+            "equal",
+            "not equal",
+            "greater than",
+            "greater than or equal",
+            "less than",
+            "less than or equal",
+        ]:
+            self._val = kwargs.get("value")
+            self._filters.append(
+                {
+                    "filtertype": self._filter_join,
+                    "field": self._filter_field,
+                    "operator": self._filter_condition,
+                    "value": self._val,
+                }
+            )
         else:
             raise Exception("Please provide a 'value' parameter for comparison")
 
@@ -468,17 +545,34 @@ class PieChartData(object):
         c_field = self.category_field
         v_field = self.value_field
 
-        x = self._item.tables[0].query(group_by_fields_for_statistics=c_field ,out_statistics=[{"statisticType":statistics,"onStatisticField":v_field,"outStatisticFieldName":"value"}], as_df=True)
+        x = self._item.tables[0].query(
+            group_by_fields_for_statistics=c_field,
+            out_statistics=[
+                {
+                    "statisticType": statistics,
+                    "onStatisticField": v_field,
+                    "outStatisticFieldName": "value",
+                }
+            ],
+            as_df=True,
+        )
         self._slice_fields = x[c_field]
 
-class PieChartProperties(object):
 
+class PieChartProperties(object):
     @classmethod
-    def _create_chart(cls, text_color="#000000", font_size=12, start_angle=90, inner_radius=0, hover_text=True):
+    def _create_chart(
+        cls,
+        text_color="#000000",
+        font_size=12,
+        start_angle=90,
+        inner_radius=0,
+        hover_text=True,
+    ):
 
         chart = PieChartProperties()
 
-        chart._text_color= text_color
+        chart._text_color = text_color
         chart._font_size = font_size
         chart._start_angle = start_angle
         chart._inner_radius = inner_radius
@@ -562,20 +656,30 @@ class PieChartProperties(object):
         """
         self._hover_text = value
 
-class SliceProperties(object):
 
+class SliceProperties(object):
     @classmethod
-    def _slice_properties(cls, opacity=1.0, default_color="#ffffff", null_color="#ffffff", blank_color="#000000", grouping_color="#000000", grouping_percent=0, null_label='Null', blank_label='Blank'):
+    def _slice_properties(
+        cls,
+        opacity=1.0,
+        default_color="#ffffff",
+        null_color="#ffffff",
+        blank_color="#000000",
+        grouping_color="#000000",
+        grouping_percent=0,
+        null_label="Null",
+        blank_label="Blank",
+    ):
         slices = SliceProperties()
 
         slices._opacity = 1.0
         slices._default_color = "#ffffff"
 
         slices._null_color = "#ffffff"
-        slices._null_label = 'Null'
+        slices._null_label = "Null"
 
         slices._blank_color = "#000000"
-        slices._blank_label = 'Blank'
+        slices._blank_label = "Blank"
 
         slices._grouping_percent = 0
         slices._grouping_color = "#000000"
@@ -600,7 +704,7 @@ class SliceProperties(object):
         :return: slices opacity value.
         """
         return self._opacity
-    
+
     @opacity.setter
     def opacity(self, value):
         """
@@ -706,8 +810,8 @@ class SliceProperties(object):
         """
         self._grouping_percent = value
 
-class OutlineProperties(object):
 
+class OutlineProperties(object):
     @classmethod
     def _outline_init(cls, opacity=0.4, thickness=0.2, color="#000000"):
         outline = OutlineProperties()
@@ -763,9 +867,9 @@ class OutlineProperties(object):
         Set outline color.
         """
         self._color = str(value)
- 
-class Events(object):
 
+
+class Events(object):
     @classmethod
     def _create_events(cls, enable=False):
         events = Events()
@@ -831,7 +935,7 @@ class Events(object):
         =========================   ===========================================
         **Argument**                **Description**
         -------------------------   -------------------------------------------
-        action_type                 Required string. Actions can be one of 
+        action_type                 Required string. Actions can be one of
                                     "zoom", "flash", "show_popup", "pan".
         -------------------------   -------------------------------------------
         widget                      Required MapWidget item. Name of the map
@@ -846,11 +950,21 @@ class Events(object):
                     self._actions.append({"type": action_type, "targetId": widget._id})
                 elif action_type == "filter":
                     if self._targetid is not None:
-                        self._actions.append({"type":action_type, "by":"whereClause", "targetId":self._targetid})
+                        self._actions.append(
+                            {
+                                "type": action_type,
+                                "by": "whereClause",
+                                "targetId": self._targetid,
+                            }
+                        )
                     else:
-                        raise Exception("This operation is not suitable for given dataSource.")
+                        raise Exception(
+                            "This operation is not suitable for given dataSource."
+                        )
                 else:
-                    raise Exception("Please select action_type from 'zoom', 'flash', 'show_popup' and 'pan'")
+                    raise Exception(
+                        "Please select action_type from 'zoom', 'flash', 'show_popup' and 'pan'"
+                    )
             else:
                 raise Exception("Please select a map widget")
 
@@ -872,15 +986,31 @@ class Events(object):
             if isinstance(widgets, list):
                 for widget in widgets:
                     if widget.type == "mapWidget":
-                        raise Exception("Use sync_map method to add actions for map widgets") ##duplicate or erase
+                        raise Exception(
+                            "Use sync_map method to add actions for map widgets"
+                        )  ##duplicate or erase
                     else:
                         action_type = "filter"
-                        widget_id = str(widget._id)+'#main'
-                        self._actions.append({"type":action_type, "by":"whereClause", "targetId":widget_id})
+                        widget_id = str(widget._id) + "#main"
+                        self._actions.append(
+                            {
+                                "type": action_type,
+                                "by": "whereClause",
+                                "targetId": widget_id,
+                            }
+                        )
             else:
                 if widgets.type == "mapWidget":
-                    raise Exception("Use sync_map method to add actions for map widgets") ##duplicate or erase
+                    raise Exception(
+                        "Use sync_map method to add actions for map widgets"
+                    )  ##duplicate or erase
                 else:
                     action_type = "filter"
-                    widget_id = str(widgets._id)+'#main'
-                    self._actions.append({"type":action_type, "by":"whereClause", "targetId":widget_id})
+                    widget_id = str(widgets._id) + "#main"
+                    self._actions.append(
+                        {
+                            "type": action_type,
+                            "by": "whereClause",
+                            "targetId": widget_id,
+                        }
+                    )
