@@ -6,7 +6,10 @@ from arcgis import env
 from arcgis.gis import GIS
 from arcgis.gis import Item
 from ._ref import reference
+from arcgis._impl.common._deprecate import deprecated
 
+
+@deprecated(deprecated_in="2.0.0", removed_in=None, current_version="2.0.0")
 class JournalStoryMap(object):
     """
     Represents a Journal Story Map
@@ -20,6 +23,7 @@ class JournalStoryMap(object):
     ===============     ====================================================================
 
     """
+
     _properties = None
     _gis = None
     _itemid = None
@@ -35,37 +39,46 @@ class JournalStoryMap(object):
             self._item = gis.content.get(item)
             self._itemid = self._item.itemid
             self._properties = self._item.get_data()
-        elif item and isinstance(item, Item) and \
-             'MapJournal' in item.typeKeywords:
+        elif item and isinstance(item, Item) and "MapJournal" in item.typeKeywords:
             self._item = item
             self._itemid = self._item.itemid
             self._properties = self._item.get_data()
-        elif item and isinstance(item, Item) and \
-             'MapJournal' not in item.typeKeywords:
+        elif item and isinstance(item, Item) and "MapJournal" not in item.typeKeywords:
             raise ValueError("Item is not a Journal Story Map")
         else:
-            self._properties = reference['journal']
-    #----------------------------------------------------------------------
+            self._properties = reference["journal"]
+
+    # ----------------------------------------------------------------------
     def __str__(self):
         return json.dumps(self._properties)
-    #----------------------------------------------------------------------
+
+    # ----------------------------------------------------------------------
     def __repr__(self):
         return self.__str__()
-    #----------------------------------------------------------------------
+
+    # ----------------------------------------------------------------------
     def _refresh(self):
         if self._item:
             self._properties = json.loads(self._item.get_data())
-    #----------------------------------------------------------------------
+
+    # ----------------------------------------------------------------------
     @property
     def properties(self):
         """returns the storymap's JSON"""
         return self._properties
-    #----------------------------------------------------------------------
-    def add(self, title,
-            url_or_item, content=None,
-            actions=None, visible=True,
-            alt_text="", display='stretch',
-            **kwargs):
+
+    # ----------------------------------------------------------------------
+    def add(
+        self,
+        title,
+        url_or_item,
+        content=None,
+        actions=None,
+        visible=True,
+        alt_text="",
+        display="stretch",
+        **kwargs,
+    ):
         """
         Adds a new section to the StoryMap
 
@@ -143,42 +156,65 @@ class JournalStoryMap(object):
             popup = kwargs.pop("popup", None)
             if layer_visibility:
                 layer_visibility = json.dumps(layer_visibility)
-            return self._add_webmap(item=url_or_item, title=title, content=content,
-                                    actions=actions, visible=visible, alt_text=alt_text,
-                                    display=display,
-                                    show_legend=show_legend,
-                                    show_default_legend=show_default_legend,
-                                    extent=extent,
-                                    layer_visibility=layer_visibility,
-                                    popup=popup)
+            return self._add_webmap(
+                item=url_or_item,
+                title=title,
+                content=content,
+                actions=actions,
+                visible=visible,
+                alt_text=alt_text,
+                display=display,
+                show_legend=show_legend,
+                show_default_legend=show_default_legend,
+                extent=extent,
+                layer_visibility=layer_visibility,
+                popup=popup,
+            )
         elif isinstance(url_or_item, str):
             mt = mimetypes.guess_type(url=url_or_item)
-            if mt[0].lower().find('video') > -1:
-                return self._add_video(url=url_or_item,
-                                      title=title,
-                                      content=content,
-                                      actions=actions,
-                                      visible=visible,
-                                      alt_text=alt_text,
-                                      display=display)
-            elif mt[0].lower().find('image') > -1:
-                return self._add_image(title=title, image=url_or_item,
-                                       content=content, actions=actions, visible=visible,
-                                       alt_text=alt_text, display=display)
+            if mt[0].lower().find("video") > -1:
+                return self._add_video(
+                    url=url_or_item,
+                    title=title,
+                    content=content,
+                    actions=actions,
+                    visible=visible,
+                    alt_text=alt_text,
+                    display=display,
+                )
+            elif mt[0].lower().find("image") > -1:
+                return self._add_image(
+                    title=title,
+                    image=url_or_item,
+                    content=content,
+                    actions=actions,
+                    visible=visible,
+                    alt_text=alt_text,
+                    display=display,
+                )
             else:
-                return self._add_webpage(title=title, url=url_or_item,
-                                         content=content, actions=actions, visible=visible,
-                                         alt_text=alt_text, display=display)
+                return self._add_webpage(
+                    title=title,
+                    url=url_or_item,
+                    content=content,
+                    actions=actions,
+                    visible=visible,
+                    alt_text=alt_text,
+                    display=display,
+                )
         return False
-        #----------------------------------------------------------------------
-    def _add_webpage(self,
-                    title,
-                    url,
-                    content=None,
-                    actions=None,
-                    visible=True,
-                    alt_text="",
-                    display='stretch'):
+        # ----------------------------------------------------------------------
+
+    def _add_webpage(
+        self,
+        title,
+        url,
+        content=None,
+        actions=None,
+        visible=True,
+        alt_text="",
+        display="stretch",
+    ):
         """
         Adds a webpage to the storymap
 
@@ -211,7 +247,7 @@ class JournalStoryMap(object):
             visible = "PUBLISHED"
         else:
             visible = "HIDDEN"
-        self._properties['values']['story']['sections'].append(
+        self._properties["values"]["story"]["sections"].append(
             {
                 "title": title,
                 "content": content,
@@ -227,22 +263,24 @@ class JournalStoryMap(object):
                         "altText": alt_text,
                         "display": display,
                         "unload": True,
-                        "hash": "5"
-                    }
-                }
+                        "hash": "5",
+                    },
+                },
             }
         )
         return True
-    #----------------------------------------------------------------------
-    def _add_video(self,
-                  url,
-                  title,
-                  content,
-                  actions=None,
-                  visible=True,
-                  alt_text="",
-                  display='stretch'
-                  ):
+
+    # ----------------------------------------------------------------------
+    def _add_video(
+        self,
+        url,
+        title,
+        content,
+        actions=None,
+        visible=True,
+        alt_text="",
+        display="stretch",
+    ):
         """
         Adds a video section to the StoryMap.
 
@@ -288,27 +326,29 @@ class JournalStoryMap(object):
                     "url": url,
                     "type": "video",
                     "altText": alt_text,
-                    "display": display
-                }
-            }
+                    "display": display,
+                },
+            },
         }
-        self._properties['values']['story']['sections'].append(video)
+        self._properties["values"]["story"]["sections"].append(video)
         return True
-    #----------------------------------------------------------------------
-    def _add_webmap(self,
-                   item,
-                   title,
-                   content,
-                   actions=None,
-                   visible=True,
-                   alt_text="",
-                   display='stretch',
-                   show_legend=False,
-                   show_default_legend=False,
-                   extent=None,
-                   layer_visibility=None,
-                   popup=None
-                   ):
+
+    # ----------------------------------------------------------------------
+    def _add_webmap(
+        self,
+        item,
+        title,
+        content,
+        actions=None,
+        visible=True,
+        alt_text="",
+        display="stretch",
+        show_legend=False,
+        show_default_legend=False,
+        extent=None,
+        layer_visibility=None,
+        popup=None,
+    ):
         """
         Adds a WebMap to the Section.
 
@@ -361,32 +401,30 @@ class JournalStoryMap(object):
                     "extent": extent,
                     "layers": layer_visibility,
                     "popup": popup,
-                    "overview": {
-                        "enable": False,
-                        "openByDefault": True
-                        },
+                    "overview": {"enable": False, "openByDefault": True},
                     "legend": {
                         "enable": show_legend,
-                        "openByDefault": show_default_legend
-                        },
-                    "geocoder": {
-                        "enable": False
-                        },
-                    "altText": alt_text
-                }
-            }
+                        "openByDefault": show_default_legend,
+                    },
+                    "geocoder": {"enable": False},
+                    "altText": alt_text,
+                },
+            },
         }
-        self._properties['values']['story']['sections'].append(wm)
+        self._properties["values"]["story"]["sections"].append(wm)
         return True
-    #----------------------------------------------------------------------
-    def _add_image(self,
-                  title,
-                  image,
-                  content=None,
-                  actions=None,
-                  visible=True,
-                  alt_text=None,
-                  display='fill'):
+
+    # ----------------------------------------------------------------------
+    def _add_image(
+        self,
+        title,
+        image,
+        content=None,
+        actions=None,
+        visible=True,
+        alt_text=None,
+        display="fill",
+    ):
         """
         Adds a new image section to the storymap
 
@@ -420,7 +458,7 @@ class JournalStoryMap(object):
             visible = "PUBLISHED"
         else:
             visible = "HIDDEN"
-        self._properties['values']['story']['sections'].append(
+        self._properties["values"]["story"]["sections"].append(
             {
                 "title": title,
                 "content": content,
@@ -434,13 +472,14 @@ class JournalStoryMap(object):
                         "url": image,
                         "type": "image",
                         "altText": alt_text,
-                        "display": display
-                    }
-                }
+                        "display": display,
+                    },
+                },
             }
         )
         return True
-    #----------------------------------------------------------------------
+
+    # ----------------------------------------------------------------------
     def remove(self, index):
         """
         Removes a section by index.
@@ -456,12 +495,13 @@ class JournalStoryMap(object):
 
         """
         try:
-            item = self._properties['values']['story']['sections'][index]
-            self._properties['values']['story']['sections'].remove(item)
+            item = self._properties["values"]["story"]["sections"][index]
+            self._properties["values"]["story"]["sections"].remove(item)
             return True
         except:
             return False
-    #----------------------------------------------------------------------
+
+    # ----------------------------------------------------------------------
     def save(self, title=None, tags=None, description=None):
         """
         Saves an Journal StoryMap to the GIS
@@ -484,57 +524,79 @@ class JournalStoryMap(object):
         import uuid
 
         if self._item:
-            p = {
-                    'text' : json.dumps(self._properties)
-                }
+            p = {"text": json.dumps(self._properties)}
             if title:
-                p['title'] = title
+                p["title"] = title
             if tags:
-                p['tags'] = tags
+                p["tags"] = tags
             return self._item.update(item_properties=p)
         else:
             if title is None:
                 title = "Map Journal, %s" % uuid.uuid4().hex[:10]
             if tags is None:
                 tags = "Story Map,Map Journal"
-            typeKeywords = ",".join(['JavaScript', 'layout-side', 'Map', 'MapJournal',
-                                     'Mapping Site', 'Online Map', 'Ready To Use',
-                                     'selfConfigured', 'Story Map', 'Story Maps',
-                                     'Web Map'])
-            item = self._gis.content.add(item_properties={
-                'title' : title,
-                'tags' : tags,
-                'text' : json.dumps(self._properties),
-                'typeKeywords' : typeKeywords,
-                'itemType' : 'text',
-                'type' : "Web Mapping Application",
-            })
+            typeKeywords = ",".join(
+                [
+                    "JavaScript",
+                    "layout-side",
+                    "Map",
+                    "MapJournal",
+                    "Mapping Site",
+                    "Online Map",
+                    "Ready To Use",
+                    "selfConfigured",
+                    "Story Map",
+                    "Story Maps",
+                    "Web Map",
+                ]
+            )
+            item = self._gis.content.add(
+                item_properties={
+                    "title": title,
+                    "tags": tags,
+                    "text": json.dumps(self._properties),
+                    "typeKeywords": typeKeywords,
+                    "itemType": "text",
+                    "type": "Web Mapping Application",
+                }
+            )
             parse = urlparse(self._gis._con.baseurl)
             isinstance(self._gis, GIS)
             if self._gis._portal.is_arcgisonline:
-                url = "%s://%s/apps/MapJournal/index.html?appid=%s" % (parse.scheme, parse.netloc, item.itemid)
+                url = "%s://%s/apps/MapJournal/index.html?appid=%s" % (
+                    parse.scheme,
+                    parse.netloc,
+                    item.itemid,
+                )
             else:
                 import os
+
                 wa = os.path.dirname(parse.path[1:])
-                url = "%s://%s/%s/sharing/rest/apps/MapJournal/index.html?appid=%s" % (parse.scheme, parse.netloc, wa, item.itemid)
-            return item.update(item_properties={
-                'url' : url
-            })
+                url = "%s://%s/%s/sharing/rest/apps/MapJournal/index.html?appid=%s" % (
+                    parse.scheme,
+                    parse.netloc,
+                    wa,
+                    item.itemid,
+                )
+            return item.update(item_properties={"url": url})
         return False
-    #----------------------------------------------------------------------
+
+    # ----------------------------------------------------------------------
     def delete(self):
         """Deletes the saved item on ArcGIS Online/Portal"""
         if self._item:
             return self._item.delete()
         return False
-    #----------------------------------------------------------------------
+
+    # ----------------------------------------------------------------------
     @property
     def panel(self):
         """
         Gets/Sets the panel state for the Journal Story Map
         """
         return self._properties["values"]["settings"]["layout"]["id"]
-    #----------------------------------------------------------------------
+
+    # ----------------------------------------------------------------------
     @panel.setter
     def panel(self, value):
         """
@@ -544,49 +606,44 @@ class JournalStoryMap(object):
             self._properties["values"]["settings"]["layout"]["id"] = "float"
         else:
             self._properties["values"]["settings"]["layout"]["id"] = "side"
-    #----------------------------------------------------------------------
+
+    # ----------------------------------------------------------------------
     @property
     def header(self):
         """gets/sets the headers for the Journal StoryMap"""
         default = {
-            "social": {
-                "bitly": True,
-                "twitter": True,
-                "facebook": True
-                },
+            "social": {"bitly": True, "twitter": True, "facebook": True},
             "logoURL": None,
             "linkURL": "https://storymaps.arcgis.com",
             "logoTarget": "",
-            "linkText": "A Story Map"
+            "linkText": "A Story Map",
         }
-        if 'header' in self._properties['values']['settings']:
-            return self._properties['values']['settings']['header']
+        if "header" in self._properties["values"]["settings"]:
+            return self._properties["values"]["settings"]["header"]
         else:
-            self._properties['values']['settings']['header'] = default
+            self._properties["values"]["settings"]["header"] = default
             return default
-    #----------------------------------------------------------------------
+
+    # ----------------------------------------------------------------------
     @header.setter
     def header(self, value):
-        """"""
+        """ """
         if value is None:
             default = {
-                "social": {
-                    "bitly": True,
-                    "twitter": True,
-                    "facebook": True
-                    },
+                "social": {"bitly": True, "twitter": True, "facebook": True},
                 "logoURL": None,
                 "linkURL": "https://storymaps.arcgis.com",
                 "logoTarget": "",
-                "linkText": "A Story Map"
+                "linkText": "A Story Map",
             }
-            self._properties['values']['settings']['header'] = default
+            self._properties["values"]["settings"]["header"] = default
         else:
-            self._properties['values']['settings']['header'] = value
-    #----------------------------------------------------------------------
+            self._properties["values"]["settings"]["header"] = value
+
+    # ----------------------------------------------------------------------
     @property
     def theme(self):
-        """"""
+        """ """
         default = {
             "colors": {
                 "text": "#FFFFFF",
@@ -598,29 +655,30 @@ class JournalStoryMap(object):
                 "textLink": "#DDD",
                 "esriLogo": "white",
                 "dotNav": "#000000",
-                "softBtn": "#AAA"
-                },
+                "softBtn": "#AAA",
+            },
             "fonts": {
                 "sectionTitle": {
-                    "value": "font-family:\'open_sansregular\', sans-serif;",
-                    "id": "default"
-                    },
+                    "value": "font-family:'open_sansregular', sans-serif;",
+                    "id": "default",
+                },
                 "sectionContent": {
-                    "value": "font-family:\'open_sansregular\', sans-serif;",
-                    "id": "default"
-                }
-            }
+                    "value": "font-family:'open_sansregular', sans-serif;",
+                    "id": "default",
+                },
+            },
         }
-        if 'theme' in self._properties['values']['settings']:
-            return self._properties['values']['settings']['theme']
+        if "theme" in self._properties["values"]["settings"]:
+            return self._properties["values"]["settings"]["theme"]
         else:
-            self._properties['values']['settings']['theme'] = default
-            return self._properties['values']['settings']['theme']
+            self._properties["values"]["settings"]["theme"] = default
+            return self._properties["values"]["settings"]["theme"]
         return default
-    #----------------------------------------------------------------------
+
+    # ----------------------------------------------------------------------
     @theme.setter
     def theme(self, value):
-        """"""
+        """ """
         default = {
             "colors": {
                 "text": "#FFFFFF",
@@ -632,27 +690,22 @@ class JournalStoryMap(object):
                 "textLink": "#DDD",
                 "esriLogo": "white",
                 "dotNav": "#000000",
-                "softBtn": "#AAA"
-                },
+                "softBtn": "#AAA",
+            },
             "fonts": {
                 "sectionTitle": {
-                    "value": "font-family:\'open_sansregular\', sans-serif;",
-                    "id": "default"
-                    },
+                    "value": "font-family:'open_sansregular', sans-serif;",
+                    "id": "default",
+                },
                 "sectionContent": {
-                    "value": "font-family:\'open_sansregular\', sans-serif;",
-                    "id": "default"
-                }
-            }
+                    "value": "font-family:'open_sansregular', sans-serif;",
+                    "id": "default",
+                },
+            },
         }
-        if 'theme' in self._properties['values']['settings']:
-            self._properties['values']['settings']['theme'] = value
-        elif not 'theme' in self._properties['values']['settings']:
-            self._properties['values']['settings']['theme'] = value
+        if "theme" in self._properties["values"]["settings"]:
+            self._properties["values"]["settings"]["theme"] = value
+        elif not "theme" in self._properties["values"]["settings"]:
+            self._properties["values"]["settings"]["theme"] = value
         elif value is None:
-            self._properties['values']['settings']['theme'] = default
-
-
-
-
-
+            self._properties["values"]["settings"]["theme"] = default
