@@ -83,7 +83,7 @@ class Assignment(FeatureModel):
     worker                 Optional :class:`~arcgis.apps.workforce.Worker`.
                            The worker assigned to the assignment
     ==================     ====================================================================
-    
+
     .. code-block:: python
 
         # Get an assignment and update it
@@ -99,11 +99,29 @@ class Assignment(FeatureModel):
 
     """
 
-    def __init__(self, project, feature=None, geometry=None, assignment_type=None,
-            assigned_date=None, assignment_read=None, completed_date=None, declined_comment=None,
-            declined_date=None, description=None, dispatcher=None, due_date=None, in_progress_date=None,
-            location=None, notes=None, paused_date=None, priority="none", status=None,
-            work_order_id=None, worker=None):
+    def __init__(
+        self,
+        project,
+        feature=None,
+        geometry=None,
+        assignment_type=None,
+        assigned_date=None,
+        assignment_read=None,
+        completed_date=None,
+        declined_comment=None,
+        declined_date=None,
+        description=None,
+        dispatcher=None,
+        due_date=None,
+        in_progress_date=None,
+        location=None,
+        notes=None,
+        paused_date=None,
+        priority="none",
+        status=None,
+        work_order_id=None,
+        worker=None,
+    ):
         super().__init__(project, project.assignments_layer, feature)
         self._schema = AssignmentSchema(project.assignments_layer)
         if feature:
@@ -113,27 +131,43 @@ class Assignment(FeatureModel):
             # forces GUIDs to upper case
             if feature.attributes[project._assignment_schema.worker_id]:
                 if self.project._is_v2_project:
-                    self.worker = self.project._cached_workers[feature.attributes[project._assignment_schema.worker_id].upper()]
+                    self.worker = self.project._cached_workers[
+                        feature.attributes[project._assignment_schema.worker_id].upper()
+                    ]
                 else:
-                    self.worker = self.project._cached_workers[feature.attributes[project._assignment_schema.worker_id]]
+                    self.worker = self.project._cached_workers[
+                        feature.attributes[project._assignment_schema.worker_id]
+                    ]
             else:
                 self.worker = None
             if feature.attributes[project._assignment_schema.dispatcher_id]:
                 # in case dispatcher for an existing assignment has been deleted
                 try:
                     if self.project._is_v2_project:
-                        self.dispatcher = self.project._cached_dispatchers[feature.attributes[project._assignment_schema.dispatcher_id].upper()]
+                        self.dispatcher = self.project._cached_dispatchers[
+                            feature.attributes[
+                                project._assignment_schema.dispatcher_id
+                            ].upper()
+                        ]
                     else:
-                        self.dispatcher = self.project._cached_dispatchers[feature.attributes[project._assignment_schema.dispatcher_id]]
+                        self.dispatcher = self.project._cached_dispatchers[
+                            feature.attributes[project._assignment_schema.dispatcher_id]
+                        ]
                 except KeyError:
                     self.dispatcher = None
             else:
                 self.dispatcher = None
             if feature.attributes[project._assignment_schema.assignment_type]:
                 if self.project._is_v2_project:
-                    self.assignment_type = self.project._cached_assignment_types[feature.attributes[project._assignment_schema.assignment_type].upper()]
+                    self.assignment_type = self.project._cached_assignment_types[
+                        feature.attributes[
+                            project._assignment_schema.assignment_type
+                        ].upper()
+                    ]
                 else:
-                    self.assignment_type = self.project._cached_assignment_types[feature.attributes[project._assignment_schema.assignment_type]]
+                    self.assignment_type = self.project._cached_assignment_types[
+                        feature.attributes[project._assignment_schema.assignment_type]
+                    ]
             else:
                 self.assignment_type = None
         else:
@@ -168,87 +202,121 @@ class Assignment(FeatureModel):
     def __repr__(self):
         return "<Assignment {}>".format(self.object_id)
 
-    def update(self, geometry=None, assignment_type=None,
-            assigned_date=None, assignment_read=None, completed_date=None, declined_comment=None,
-            declined_date=None, description=None, dispatcher=None, due_date=None, in_progress_date=None,
-            location=None, notes=None, paused_date=None, priority=None, status=None,
-            work_order_id=None, worker=None):
+    def update(
+        self,
+        geometry=None,
+        assignment_type=None,
+        assigned_date=None,
+        assignment_read=None,
+        completed_date=None,
+        declined_comment=None,
+        declined_date=None,
+        description=None,
+        dispatcher=None,
+        due_date=None,
+        in_progress_date=None,
+        location=None,
+        notes=None,
+        paused_date=None,
+        priority=None,
+        status=None,
+        work_order_id=None,
+        worker=None,
+    ):
         """
-            Updates the assignment on the server
+        Updates the assignment on the server
 
-            ==================     ====================================================================
-            **Argument**           **Description**
-            ------------------     --------------------------------------------------------------------
-            geometry               Optional :class:`Dict`.
-                                   A dictionary containing the assignment geometry
-            ------------------     --------------------------------------------------------------------
-            assignment_type        Optional :class:`~arcgis.apps.workforce.AssignmentType`.
-                                   The assignment type that represents this assignment.
-            ------------------     --------------------------------------------------------------------
-            assigned_date          Optional :class:`Date`
-                                   The date and time the assignment was assigned
-            ------------------     --------------------------------------------------------------------
-            assignment_read        Optional :class:`Bool`.
-                                   A flag indicating that the mobile worker has seen the assignment.
-                                   Version 1 Projects Only
-            ------------------     --------------------------------------------------------------------
-            completed_date         Optional :class:`Date`.
-                                   The date the assignment was completed
-            ------------------     --------------------------------------------------------------------
-            declined_comment       Optional :class:`String`.
-                                   The comment submitted by the mobile worker.
-            ------------------     --------------------------------------------------------------------
-            declined_date          Optional :class:`Date`.
-                                   The date the assignment was declined.
-            ------------------     --------------------------------------------------------------------
-            description            Optional :class:`Description`.
-                                   The description associated with the assignment.
-            ------------------     --------------------------------------------------------------------
-            dispatcher             Optional :class:`~arcgis.apps.workforce.Dispatcher`.
-                                   The dispatcher that assigned/created the assignment.
-            ------------------     --------------------------------------------------------------------
-            due_date               Optional :class:`Date`.
-                                   The date the assignment is due.
-            ------------------     --------------------------------------------------------------------
-            in_progress_date       Optional :class:`Date`.
-                                   The date the assignment was started.
-            ------------------     --------------------------------------------------------------------
-            location               Optional :class:`String`.
-                                   The location or address of the assignment.
-            ------------------     --------------------------------------------------------------------
-            notes                  Optional :class:`String`.
-                                   The notes associated with the assignment.
-            ------------------     --------------------------------------------------------------------
-            paused_date            Optional :class:`Date`.
-                                   The date and time the assignment was paused.
-            ------------------     --------------------------------------------------------------------
-            priority               Optional :class:`String`.
-                                   The priority of the assignment
+        ==================     ====================================================================
+        **Argument**           **Description**
+        ------------------     --------------------------------------------------------------------
+        geometry               Optional :class:`Dict`.
+                               A dictionary containing the assignment geometry
+        ------------------     --------------------------------------------------------------------
+        assignment_type        Optional :class:`~arcgis.apps.workforce.AssignmentType`.
+                               The assignment type that represents this assignment.
+        ------------------     --------------------------------------------------------------------
+        assigned_date          Optional :class:`Date`
+                               The date and time the assignment was assigned
+        ------------------     --------------------------------------------------------------------
+        assignment_read        Optional :class:`Bool`.
+                               A flag indicating that the mobile worker has seen the assignment.
+                               Version 1 Projects Only
+        ------------------     --------------------------------------------------------------------
+        completed_date         Optional :class:`Date`.
+                               The date the assignment was completed
+        ------------------     --------------------------------------------------------------------
+        declined_comment       Optional :class:`String`.
+                               The comment submitted by the mobile worker.
+        ------------------     --------------------------------------------------------------------
+        declined_date          Optional :class:`Date`.
+                               The date the assignment was declined.
+        ------------------     --------------------------------------------------------------------
+        description            Optional :class:`Description`.
+                               The description associated with the assignment.
+        ------------------     --------------------------------------------------------------------
+        dispatcher             Optional :class:`~arcgis.apps.workforce.Dispatcher`.
+                               The dispatcher that assigned/created the assignment.
+        ------------------     --------------------------------------------------------------------
+        due_date               Optional :class:`Date`.
+                               The date the assignment is due.
+        ------------------     --------------------------------------------------------------------
+        in_progress_date       Optional :class:`Date`.
+                               The date the assignment was started.
+        ------------------     --------------------------------------------------------------------
+        location               Optional :class:`String`.
+                               The location or address of the assignment.
+        ------------------     --------------------------------------------------------------------
+        notes                  Optional :class:`String`.
+                               The notes associated with the assignment.
+        ------------------     --------------------------------------------------------------------
+        paused_date            Optional :class:`Date`.
+                               The date and time the assignment was paused.
+        ------------------     --------------------------------------------------------------------
+        priority               Optional :class:`String`.
+                               The priority of the assignment
 
-                                   `none`, `low`, `medium`, `high`, `critical`
-            ------------------     --------------------------------------------------------------------
-            status                 Optional :class:`String`.
-                                   The status of the assignment.
+                               `none`, `low`, `medium`, `high`, `critical`
+        ------------------     --------------------------------------------------------------------
+        status                 Optional :class:`String`.
+                               The status of the assignment.
 
-                                   `unassigned`, `assigned`, `in_progress`, `completed`, `declined`,
-                                   `paused`, `canceled`
-            ------------------     --------------------------------------------------------------------
-            work_order_id          Optional :class:`String`.
-                                   The work order id associated with the assignment.
-            ------------------     --------------------------------------------------------------------
-            worker                 Optional :class:`~arcgis.apps.workforce.Worker`.
-                                   The worker assigned to the assignment
-            ==================     ====================================================================
+                               `unassigned`, `assigned`, `in_progress`, `completed`, `declined`,
+                               `paused`, `canceled`
+        ------------------     --------------------------------------------------------------------
+        work_order_id          Optional :class:`String`.
+                               The work order id associated with the assignment.
+        ------------------     --------------------------------------------------------------------
+        worker                 Optional :class:`~arcgis.apps.workforce.Worker`.
+                               The worker assigned to the assignment
+        ==================     ====================================================================
         """
-        update_assignment(self.project, self, geometry, assignment_type, assigned_date, assignment_read,
-                          completed_date, declined_comment, declined_date, description,
-                          dispatcher, due_date, in_progress_date, location, notes, paused_date,
-                          priority, status, work_order_id, worker)
+        update_assignment(
+            self.project,
+            self,
+            geometry,
+            assignment_type,
+            assigned_date,
+            assignment_read,
+            completed_date,
+            declined_comment,
+            declined_date,
+            description,
+            dispatcher,
+            due_date,
+            in_progress_date,
+            location,
+            notes,
+            paused_date,
+            priority,
+            status,
+            work_order_id,
+            worker,
+        )
 
     def delete(self):
         """Deletes the assignment from the server"""
         delete_assignments(self.project, [self])
-    
+
     @property
     def _supports_assignment_read_field(self):
         return bool(self._schema.assignment_read)
@@ -273,19 +341,28 @@ class Assignment(FeatureModel):
         if self._supports_assignment_read_field:
             return bool(self._feature.attributes.get(self._schema.assignment_read))
         else:
-            warn("This Workforce Project does not support the assignment_read field.", WorkforceWarning)
+            warn(
+                "This Workforce Project does not support the assignment_read field.",
+                WorkforceWarning,
+            )
 
     @assignment_read.setter
     def assignment_read(self, value):
         if self._supports_assignment_read_field:
             self._feature.attributes[self._schema.assignment_read] = 1 if value else 0
         else:
-            warn("This Workforce Project does not support the assignment_read field.", WorkforceWarning)
+            warn(
+                "This Workforce Project does not support the assignment_read field.",
+                WorkforceWarning,
+            )
 
     @property
     def assignment_type_code(self):
         if self.project._is_v2_project and self._assignment_type is not None:
-            return self._feature.attributes.get(self._schema.assignment_type).upper()
+            value = self._feature.attributes.get(self._schema.assignment_type)
+            if value:
+                return value.upper()
+            return None
         else:
             return self._feature.attributes.get(self._schema.assignment_type)
 
@@ -313,7 +390,9 @@ class Assignment(FeatureModel):
         else:
             raise ValidationError("Invalid Assignment Type", self)
         if self._assignment_type:
-            self._feature.attributes[self._schema.assignment_type] = self._assignment_type.code
+            self._feature.attributes[
+                self._schema.assignment_type
+            ] = self._assignment_type.code
         else:
             self._feature.attributes[self._schema.assignment_type] = None
 
@@ -357,7 +436,10 @@ class Assignment(FeatureModel):
     def dispatcher_id(self):
         """Gets the dispatcher id of the assignment"""
         if self.project._is_v2_project and self.dispatcher is not None:
-            return self._feature.attributes.get(self._schema.dispatcher_id).upper()
+            value = self._feature.attributes.get(self._schema.dispatcher_id)
+            if value:
+                return value.upper()
+            return None
         else:
             return self._feature.attributes.get(self._schema.dispatcher_id)
 
@@ -373,9 +455,13 @@ class Assignment(FeatureModel):
         else:
             self._dispatcher = self.project._cached_dispatcher
         if self.project._is_v2_project:
-            self._feature.attributes[self._schema.dispatcher_id] = self._dispatcher.global_id
+            self._feature.attributes[
+                self._schema.dispatcher_id
+            ] = self._dispatcher.global_id
         else:
-            self._feature.attributes[self._schema.dispatcher_id] = self._dispatcher.object_id
+            self._feature.attributes[
+                self._schema.dispatcher_id
+            ] = self._dispatcher.object_id
 
     @property
     def due_date(self):
@@ -429,17 +515,11 @@ class Assignment(FeatureModel):
     @property
     def priority(self):
         """
-            Gets/Sets the :class:`String` priority of the assignment
+        Gets/Sets the :class:`String` priority of the assignment
 
-            `none`, `low`, `medium`, `high`, `critical`
+        `none`, `low`, `medium`, `high`, `critical`
         """
-        lut = {
-            0: "none",
-            1: "low",
-            2: "medium",
-            3: "high",
-            4: "critical"
-        }
+        lut = {0: "none", 1: "low", 2: "medium", 3: "high", 4: "critical"}
         if self._feature.attributes[self._schema.priority] is not None:
             return lut[self._feature.attributes[self._schema.priority]]
         else:
@@ -447,7 +527,7 @@ class Assignment(FeatureModel):
 
     @priority.setter
     def priority(self, value):
-        if (isinstance(value, int) and value >=0 and value <= 4) or value is None:
+        if (isinstance(value, int) and value >= 0 and value <= 4) or value is None:
             self._feature.attributes[self._schema.priority] = value
         elif isinstance(value, str):
             reduced_str = value.lower().replace(" ", "").replace("_", "")
@@ -466,14 +546,13 @@ class Assignment(FeatureModel):
         else:
             raise ValidationError("Invalid priority", self)
 
-
     @property
     def status(self):
         """
-            Gets/Sets the :class:`String` status of the assignment
+         Gets/Sets the :class:`String` status of the assignment
 
-           `unassigned`, `assigned`, `in_progress`, `completed`, `declined`,
-           `paused`, `canceled`
+        `unassigned`, `assigned`, `in_progress`, `completed`, `declined`,
+        `paused`, `canceled`
         """
         lut = {
             0: "unassigned",
@@ -482,7 +561,7 @@ class Assignment(FeatureModel):
             3: "completed",
             4: "declined",
             5: "paused",
-            6: "canceled"
+            6: "canceled",
         }
         if self._feature.attributes[self._schema.status] is not None:
             try:
@@ -530,11 +609,21 @@ class Assignment(FeatureModel):
     def web_app_link(self):
         """Returns a link to the assignment in the Workforce web app"""
         if self.project.gis.properties["isPortal"]:
-            portal_url = self.project.gis.properties['portalHostname']
-            return "https://" + portal_url + "/apps/workforce/#/projects/" + self.project.id + "/dispatch/assignments/" + str(self.object_id)
-        else:
-            return "https://workforce.arcgis.com/projects/" + self.project.id + "/dispatch/assignments/" + str(self.object_id)
-        
+            portal_url = self.project.gis.properties["portalHostname"]
+            # AGOL and Enterprise 10.9+ no longer use hash routing
+            if self.project.gis.version >= [8, 4]:
+                projects_route = "/apps/workforce/projects/"
+            else:
+                projects_route = "/apps/workforce/#/projects/"
+            return (
+                "https://"
+                + portal_url
+                + projects_route
+                + self.project.id
+                + "/dispatch/assignments/"
+                + str(self.object_id)
+            )
+
     @property
     def work_order_id(self):
         """Gets/Sets the work order id of the assignment"""
@@ -548,7 +637,9 @@ class Assignment(FeatureModel):
     def worker_id(self):
         """Gets the worker id of the assignment"""
         if self.project._is_v2_project and self.worker is not None:
-            return self._feature.attributes.get(self._schema.worker_id).upper()
+            value = self._feature.attributes.get(self._schema.worker_id)
+            if value:
+                return value.upper()
         else:
             return self._feature.attributes.get(self._schema.worker_id)
 
@@ -595,17 +686,29 @@ class Assignment(FeatureModel):
     def _validate_assignment_type(self):
         errors = []
         if self.assignment_type is None:
-            errors.append(ValidationError("An assignment must have an assignment type", self))
+            errors.append(
+                ValidationError("An assignment must have an assignment type", self)
+            )
         else:
             if self.assignment_type.name is None:
-                errors.append(ValidationError("Invalid assignment type name: cannot be None", self))
+                errors.append(
+                    ValidationError(
+                        "Invalid assignment type name: cannot be None", self
+                    )
+                )
             if self.assignment_type.code is None:
-                errors.append(ValidationError("Invalid assignment type code: cannot be None", self))
+                errors.append(
+                    ValidationError(
+                        "Invalid assignment type code: cannot be None", self
+                    )
+                )
         return errors
 
     def _validate_assignment_type_on_server(self):
         errors = []
-        assignment_type = self.project._cached_assignment_types.get(self.assignment_type_code, None)
+        assignment_type = self.project._cached_assignment_types.get(
+            self.assignment_type_code, None
+        )
         if assignment_type is None:
             errors.append(ValidationError("Unrecognized assignment type", self))
         return errors
@@ -613,7 +716,9 @@ class Assignment(FeatureModel):
     def _validate_location(self):
         errors = []
         if not self.location or self.location.isspace():
-            errors.append(ValidationError("Assignment cannot have an empty location", self))
+            errors.append(
+                ValidationError("Assignment cannot have an empty location", self)
+            )
         return errors
 
     def _validate_geometry(self):
@@ -631,7 +736,7 @@ class Assignment(FeatureModel):
     def _validate_worker_on_server(self):
         errors = []
         if self.worker is not None:
-            worker = self.project._cached_workers.get(self.worker_id, None)
+            worker = self.project._cached_workers.get(self.worker_id)
             if not worker:
                 errors.append(ValidationError("Unrecognized worker object_id", self))
         return errors
@@ -647,9 +752,11 @@ class Assignment(FeatureModel):
     def _validate_dispatcher_on_server(self):
         errors = []
         if self.dispatcher is not None:
-            dispatcher = self.project._cached_dispatchers.get(self.dispatcher_id, None)
+            dispatcher = self.project._cached_dispatchers.get(self.dispatcher_id)
             if not dispatcher:
-                errors.append(ValidationError("Unrecognized dispatcher object_id", self))
+                errors.append(
+                    ValidationError("Unrecognized dispatcher object_id", self)
+                )
         return errors
 
     def _validate_status(self):
