@@ -5,24 +5,29 @@ chosen areas.
 enrich_layer retrieves information about the people, places, and businesses in a specific area, or within a selected
 travel time or distance from a location.
 """
+from arcgis.auth.tools import LazyLoader
 
-import arcgis as _arcgis
-import arcgis.network as network
-from .._impl.common._utils import inspect_function_inputs
-#--------------------------------------------------------------------------
-def enrich_layer(input_layer,
-                 data_collections=[],
-                 analysis_variables=[],
-                 country=None,
-                 buffer_type=None,
-                 distance=None,
-                 units=None,
-                 output_name=None,
-                 context=None,
-                 gis=None,
-                 estimate=False,
-                 return_boundaries=False,
-                 future=False):
+_util = LazyLoader("arcgis._impl.common._utils")
+_logging = LazyLoader("logging")
+_arcgis = LazyLoader("arcgis")
+network = LazyLoader("arcgis.network")
+
+# --------------------------------------------------------------------------
+def enrich_layer(
+    input_layer,
+    data_collections=[],
+    analysis_variables=[],
+    country=None,
+    buffer_type=None,
+    distance=None,
+    units=None,
+    output_name=None,
+    context=None,
+    gis=None,
+    estimate=False,
+    return_boundaries=False,
+    future=False,
+):
     """
     .. image:: _static/images/enrich_layer/enrich_layer.png
 
@@ -68,17 +73,32 @@ def enrich_layer(input_layer,
 
                                                                               Choice list: ['Meters', 'Kilometers', 'Feet', 'Yards', 'Miles', 'Seconds', 'Minutes'. 'Hours']
     ---------------------------------------------------------------------     --------------------------------------------------------------------
-    output_name                                                               Optional string. If provided, the task will create a feature service of the results.
-                                                                              You define the name of the service. If output_name is not supplied, the task will return a feature collection.
+    output_name                                                               Optional string or :class:`~arcgis.features.FeatureLayer`. Existing
+                                                                              feature layer will cause the new layer to be appended to the Feature Service.
+                                                                              If overwrite is True in context, new layer will overwrite existing layer.
+                                                                              If output_name not indicated then new :class:`~arcgis.features.FeatureCollection` created.
     ---------------------------------------------------------------------     --------------------------------------------------------------------
-    context                                                                   Optional string. Context contains additional settings that affect task execution. For ``enrich_layer`` method, there are two settings.
+    context                                                                   Optional dict. Additional settings such as processing extent and output spatial reference.
+                                                                              For enrich_layer, there are three settings.
+
+                                                                              - ``extent`` - a bounding box that defines the analysis area. Only those features in the input_layer that intersect the bounding box will be analyzed.
+                                                                              - ``outSR`` - the output features will be projected into the output spatial reference referred to by the `wkid`.
+                                                                              - ``overwrite`` - if True, then the feature layer in output_name will be overwritten with new feature layer.
+
+
+                                                                              .. code-block:: python
+                                                                                # Example Usage
+
+                                                                                                          "ymin": -9187921.892449,
+                                                                                                          "xmax": 3174104.927313,
+                                                                                                          "ymax": -9175500.875353,
+                                                                                                          "spatialReference":{"wkid":102100,"latestWkid":3857}},
+                                                                                                  "outSR": {"wkid": 3857},
+                                                                                                  "overwrite": True}
 
-                                                                              #. Extent (extent)-a bounding box that defines the analysis area. Only those features in the input_layer that intersect the bounding box will be analyzed.
-                                                                              #. Output Spatial Reference (outSR) the output features will be projected into the output spatial reference.
     ---------------------------------------------------------------------     --------------------------------------------------------------------
     gis                                                                       Optional, the GIS on which this tool runs. If not specified, the active GIS is used.
     ---------------------------------------------------------------------     --------------------------------------------------------------------
-    estimate                                                                  Optional Boolean. If True, the number of credits to run the operation will be returned.
     ---------------------------------------------------------------------     --------------------------------------------------------------------
     return_boundaries                                                         Optional boolean. Applies only for point and line input features. If True, a result layer of areas is returned.
                                                                               The returned areas are defined by the specified buffer_type. For example, if using a buffer_type of StraightLine with
@@ -91,7 +111,7 @@ def enrich_layer(input_layer,
     future                                                                    Optional boolean. If True, the result will be a GPJob object and results will be returned asynchronously.
     =====================================================================     ====================================================================
 
-    :returns result_layer : feature layer Item if output_name is specified, else Feature Collection.
+    :returns :class:`~arcgis.features.FeatureLayer` if output_name is specified, else :class:`~arcgis.features.FeatureCollection`.
 
     .. code-block:: python
 
@@ -103,33 +123,39 @@ def enrich_layer(input_layer,
 
     """
 
-
     gis = _arcgis.env.active_gis if gis is None else gis
     kwargs = {
-        "input_layer" : input_layer,
-        "data_collections" : data_collections,
-        "analysis_variables" : analysis_variables,
-        "country" : country,
-        "buffer_type" : buffer_type,
-        "distance" : distance,
-        "units" : units,
-        "output_name" : output_name,
-        "context" : context,
-        "gis" : gis,
-        "estimate" : estimate,
-        "return_boundaries" : return_boundaries,
-        "future" : future
+        "input_layer": input_layer,
+        "data_collections": data_collections,
+        "analysis_variables": analysis_variables,
+        "country": country,
+        "buffer_type": buffer_type,
+        "distance": distance,
+        "units": units,
+        "output_name": output_name,
+        "context": context,
+        "gis": gis,
+        "estimate": estimate,
+        "return_boundaries": return_boundaries,
+        "future": future,
     }
 
-    params = inspect_function_inputs(fn=gis._tools.featureanalysis._tbx.enrich_layer,
-                                     **kwargs)
+    params = _util.inspect_function_inputs(
+        fn=gis._tools.featureanalysis._tbx.enrich_layer, **kwargs
+    )
 
     if isinstance(buffer_type, str):
-        if buffer_type != 'StraightLine':
-            route_service = network.RouteLayer(gis.properties.helperServices.route.url, gis=gis)
-            buffer_type = [i for i in route_service.retrieve_travel_modes()['supportedTravelModes'] if i['name'] == buffer_type][0]
-            if 'buffer_type' in params:
-                params['buffer_type'] = buffer_type
+        if buffer_type != "StraightLine":
+            route_service = network.RouteLayer(
+                gis.properties.helperServices.route.url, gis=gis
+            )
+            buffer_type = [
+                i
+                for i in route_service.retrieve_travel_modes()["supportedTravelModes"]
+                if i["name"] == buffer_type
+            ][0]
+            if "buffer_type" in params:
+                params["buffer_type"] = buffer_type
         else:
             pass
 
