@@ -6,7 +6,7 @@ http://code.google.com/p/pyrtree/
 
 """
 
-__all__ = ['RTree', 'Rect', 'Rtree', 'RTreeError']
+__all__ = ["RTree", "Rect", "Rtree", "RTreeError"]
 
 MAXCHILDREN = 10
 MAX_KMEANS = 5
@@ -16,7 +16,10 @@ import random
 import time
 import array
 
-class RTreeError(Exception): pass
+
+class RTreeError(Exception):
+    pass
+
 
 class Rect(object):
     """
@@ -34,8 +37,8 @@ class Rect(object):
         self.x, self.y, self.xx, self.yy, self.swapped_x, self.swapped_y = state
 
     def __init__(self, minx, miny, maxx, maxy):
-        self.swapped_x = (maxx < minx)
-        self.swapped_y = (maxy < miny)
+        self.swapped_x = maxx < minx
+        self.swapped_y = maxy < miny
         self.x = minx
         self.y = miny
         self.xx = maxx
@@ -57,10 +60,10 @@ class Rect(object):
         toarray[idx + 1] = self.y
         toarray[idx + 2] = self.xx
         toarray[idx + 3] = self.yy
-        if (self.swapped_x):
+        if self.swapped_x:
             toarray[idx] = self.xx
             toarray[idx + 2] = self.x
-        if (self.swapped_y):
+        if self.swapped_y:
             toarray[idx + 1] = self.yy
             toarray[idx + 3] = self.y
 
@@ -94,14 +97,16 @@ class Rect(object):
         return Rect(nx, ny, nx2, ny2)
 
     def does_contain(self, o):
-        return self.does_containpoint((o.x, o.y)) and self.does_containpoint((o.xx, o.yy))
+        return self.does_containpoint((o.x, o.y)) and self.does_containpoint(
+            (o.xx, o.yy)
+        )
 
     def does_intersect(self, o):
-        return (self.intersect(o).area() > 0)
+        return self.intersect(o).area() > 0
 
     def does_containpoint(self, p):
         x, y = p
-        return (x >= self.x and x <= self.xx and y >= self.y and y <= self.yy)
+        return x >= self.x and x <= self.xx and y >= self.y and y <= self.yy
 
     def union(self, o):
         if o is NullRect:
@@ -141,6 +146,7 @@ class Rect(object):
     def diagonal(self):
         return math.sqrt(self.diagonal_sq())
 
+
 NullRect = Rect(0.0, 0.0, 0.0, 0.0)
 NullRect.swapped_x = False
 NullRect.swapped_y = False
@@ -150,7 +156,7 @@ def union_all(kids):
     cur = NullRect
     for k in kids:
         cur = cur.union(k.rect)
-    assert(False == cur.swapped_x)
+    assert False == cur.swapped_x
     return cur
 
 
@@ -168,7 +174,7 @@ class RTree(object):
             "longest_kmeans": 0.0,
             "sum_kmeans_iter_f": 0,
             "count_kmeans_iter_f": 0,
-            "avg_kmeans_iter_f": 0.0
+            "avg_kmeans_iter_f": 0.0,
         }
 
         # This round: not using objects directly -- they
@@ -179,8 +185,8 @@ class RTree(object):
         # Instead, it uses pools of arrays:
         self.count = 0
         self.leaf_count = 0
-        self.rect_pool = array.array('d')
-        self.node_pool = array.array('L')
+        self.rect_pool = array.array("d")
+        self.node_pool = array.array("L")
         self.leaf_pool = []  # leaf objects.
 
         self.cursor = _NodeCursor.create(self, NullRect)
@@ -195,7 +201,7 @@ class RTree(object):
 
     def insert(self, o, orect):
         self.cursor.insert(o, orect)
-        assert(self.cursor.index == 0)
+        assert self.cursor.index == 0
 
     def query_rect(self, r):
         for x in self.cursor.query_rect(r):
@@ -254,8 +260,8 @@ class _NodeCursor(object):
         rooto.count += 1
 
         rooto._ensure_pool(idx + 1)
-        #rooto.node_pool.extend([0,0])
-        #rooto.rect_pool.extend([0,0,0,0])
+        # rooto.node_pool.extend([0,0])
+        # rooto.rect_pool.extend([0,0,0,0])
 
         retv = _NodeCursor(rooto, idx, rect, 0, 0)
 
@@ -266,10 +272,10 @@ class _NodeCursor(object):
     def create_with_children(cls, children, rooto):
         rect = union_all([c for c in children])
         nr = Rect(rect.x, rect.y, rect.xx, rect.yy)
-        assert(not rect.swapped_x)
+        assert not rect.swapped_x
         nc = _NodeCursor.create(rooto, rect)
         nc._set_children(children)
-        assert(not nc.is_leaf())
+        assert not nc.is_leaf()
         return nc
 
     @classmethod
@@ -284,17 +290,40 @@ class _NodeCursor(object):
         rooto.leaf_pool.append(leaf_obj)
         res._save_back()
         res._become(idx)
-        assert(res.is_leaf())
+        assert res.is_leaf()
         return res
 
-    __slots__ = ("root", "npool", "rpool", "index", "rect",
-                 "next_sibling", "first_child")
+    __slots__ = (
+        "root",
+        "npool",
+        "rpool",
+        "index",
+        "rect",
+        "next_sibling",
+        "first_child",
+    )
 
     def __getstate__(self):
-        return (self.root, self.npool, self.rpool, self.index, self.rect, self.next_sibling, self.first_child)
+        return (
+            self.root,
+            self.npool,
+            self.rpool,
+            self.index,
+            self.rect,
+            self.next_sibling,
+            self.first_child,
+        )
 
     def __setstate__(self, state):
-        self.root, self.npool, self.rpool, self.index, self.rect, self.next_sibling, self.first_child = state
+        (
+            self.root,
+            self.npool,
+            self.rpool,
+            self.index,
+            self.rect,
+            self.next_sibling,
+            self.first_child,
+        ) = state
 
     def __init__(self, rooto, index, rect, first_child, next_sibling):
         self.root = rooto
@@ -307,7 +336,7 @@ class _NodeCursor(object):
         self.first_child = first_child
 
     def walk(self, predicate):
-        if (predicate(self, self.leaf_obj())):
+        if predicate(self, self.leaf_obj()):
             yield self
             if not self.is_leaf():
                 for c in self.children():
@@ -315,14 +344,17 @@ class _NodeCursor(object):
                         yield cr
 
     def query_rect(self, r):
-        """ Return things that intersect with 'r'. """
+        """Return things that intersect with 'r'."""
+
         def p(o, x):
             return r.does_intersect(o.rect)
+
         for rr in self.walk(p):
             yield rr
 
     def query_point(self, point):
-        """ Query by a point """
+        """Query by a point"""
+
         def p(o, x):
             return o.rect.does_containpoint(point)
 
@@ -330,11 +362,9 @@ class _NodeCursor(object):
             yield rr
 
     def lift(self):
-        return _NodeCursor(self.root,
-                           self.index,
-                           self.rect,
-                           self.first_child,
-                           self.next_sibling)
+        return _NodeCursor(
+            self.root, self.index, self.rect, self.first_child, self.next_sibling
+        )
 
     def _become(self, index):
         recti = index * 4
@@ -345,7 +375,7 @@ class _NodeCursor(object):
         xx = rp[recti + 2]
         yy = rp[recti + 3]
 
-        if (x == 0.0 and y == 0.0 and xx == 0.0 and yy == 0.0):
+        if x == 0.0 and y == 0.0 and xx == 0.0 and yy == 0.0:
             self.rect = NullRect
         else:
             self.rect = Rect(x, y, xx, yy)
@@ -408,8 +438,7 @@ class _NodeCursor(object):
         while True:
             if self.holds_leaves():
                 self.rect = self.rect.union(leafrect)
-                self._insert_child(_NodeCursor.create_leaf(
-                    self.root, leafo, leafrect))
+                self._insert_child(_NodeCursor.create_leaf(self.root, leafo, leafrect))
 
                 self._balance()
 
@@ -442,7 +471,7 @@ class _NodeCursor(object):
                 self._become(child)  # recurse.
 
     def _balance(self):
-        if (self.nchildren() <= MAXCHILDREN):
+        if self.nchildren() <= MAXCHILDREN:
             return
 
         t = time.clock()
@@ -453,24 +482,27 @@ class _NodeCursor(object):
 
         memo = {}
 
-        clusterings = [k_means_cluster(
-            self.root, k, s_children) for k in range(2, MAX_KMEANS)]
-        score, bestcluster = max(
-            [(silhouette_coeff(c, memo), c) for c in clusterings])
+        clusterings = [
+            k_means_cluster(self.root, k, s_children) for k in range(2, MAX_KMEANS)
+        ]
+        score, bestcluster = max([(silhouette_coeff(c, memo), c) for c in clusterings])
 
-        nodes = [_NodeCursor.create_with_children(
-            c, self.root) for c in bestcluster if len(c) > 0]
+        nodes = [
+            _NodeCursor.create_with_children(c, self.root)
+            for c in bestcluster
+            if len(c) > 0
+        ]
 
         self._set_children(nodes)
 
-        dur = (time.clock() - t)
+        dur = time.clock() - t
         c = float(self.root.stats["overflow_f"])
         oa = self.root.stats["avg_overflow_t_f"]
-        self.root.stats["avg_overflow_t_f"] = (
-            dur / (c + 1.0)) + (c * oa / (c + 1.0))
+        self.root.stats["avg_overflow_t_f"] = (dur / (c + 1.0)) + (c * oa / (c + 1.0))
         self.root.stats["overflow_f"] += 1
         self.root.stats["longest_overflow"] = max(
-            self.root.stats["longest_overflow"], dur)
+            self.root.stats["longest_overflow"], dur
+        )
 
     def _set_children(self, cs):
         self.first_child = 0
@@ -497,7 +529,7 @@ class _NodeCursor(object):
         self._save_back()
 
     def children(self):
-        if (0 == self.first_child):
+        if 0 == self.first_child:
             return
 
         idx = self.index
@@ -514,7 +546,7 @@ class _NodeCursor(object):
                 self._become(self.next_sibling)
 
         # Go back to becoming the same node we were.
-        #self._become(idx)
+        # self._become(idx)
         self.index = idx
         self.first_child = fc
         self.next_sibling = ns
@@ -549,15 +581,17 @@ def silhouette_w(node, cluster, next_closest_cluster, memo):
 
 def silhouette_coeff(clustering, memo_tab):
     # special case for a clustering of 1.0
-    if (len(clustering) == 1):
+    if len(clustering) == 1:
         return 1.0
 
     coeffs = []
     for cluster in clustering:
         others = [c for c in clustering if c is not cluster]
         others_cntr = [center_of_gravity(c) for c in others]
-        ws = [silhouette_w(node, cluster, others[closest(
-            others_cntr, node)], memo_tab) for node in cluster]
+        ws = [
+            silhouette_w(node, cluster, others[closest(others_cntr, node)], memo_tab)
+            for node in cluster
+        ]
         cluster_coeff = sum(ws) / len(ws)
         coeffs.append(cluster_coeff)
     return sum(coeffs) / len(coeffs)
@@ -598,7 +632,7 @@ def k_means_cluster(root, k, nodes):
     root.stats["count_kmeans_iter_f"] += 1
 
     # Initialize: take n random nodes.
-    #random.shuffle(ns)
+    # random.shuffle(ns)
 
     cluster_starts = ns[:k]
     cluster_centers = [center_of_gravity([n]) for n in ns[:k]]
@@ -615,21 +649,23 @@ def k_means_cluster(root, k, nodes):
         clusters = [c for c in clusters if len(c) > 0]
 
         for c in clusters:
-            if (len(c) == 0):
+            if len(c) == 0:
                 print("Errorrr....")
-                print("Nodes: %d, centers: %s" % (len(ns),
-                                                  repr(cluster_centers)))
+                print("Nodes: %d, centers: %s" % (len(ns), repr(cluster_centers)))
 
-            assert(len(c) > 0)
+            assert len(c) > 0
 
         rest = ns
         first = False
 
         new_cluster_centers = [center_of_gravity(c) for c in clusters]
         if new_cluster_centers == cluster_centers:
-            root.stats["avg_kmeans_iter_f"] = float(root.stats["sum_kmeans_iter_f"] / root.stats["count_kmeans_iter_f"])
+            root.stats["avg_kmeans_iter_f"] = float(
+                root.stats["sum_kmeans_iter_f"] / root.stats["count_kmeans_iter_f"]
+            )
             root.stats["longest_kmeans"] = max(
-                root.stats["longest_kmeans"], (time.clock() - t))
+                root.stats["longest_kmeans"], (time.clock() - t)
+            )
             return clusters
         else:
-            cluster_centers = new_cluster_centers
\ No newline at end of file
+            cluster_centers = new_cluster_centers
