@@ -9,8 +9,10 @@ from pandas import DataFrame, Series
 
 try:
     # Import register decorators from pandas >= 0.23
-    from pandas.api.extensions import (register_dataframe_accessor,
-                                       register_series_accessor)
+    from pandas.api.extensions import (
+        register_dataframe_accessor,
+        register_series_accessor,
+    )
 except ImportError:
     try:
         from pandas.core.accessor import AccessorProperty
@@ -20,6 +22,7 @@ except ImportError:
     # Define register decorators for pandas < 0.23
     class register_dataframe_accessor(object):
         """Register custom accessor on DataFrame."""
+
         def __init__(self, name):
             self.name = name
 
@@ -29,9 +32,10 @@ except ImportError:
 
     class register_series_accessor(object):
         """Register custom accessor on Series."""
+
         def __init__(self, name):
             self.name = name
 
         def __call__(self, accessor):
             setattr(Series, self.name, AccessorProperty(accessor, accessor))
-            return accessor
\ No newline at end of file
+            return accessor
