@@ -5,6 +5,14 @@ from ._ops import disjoint, equals
 from ._ops import overlaps, touches
 from ._ops import within
 
-__all__ = ['overlay', 'select','contains', 'crosses',
-           'disjoint', 'equals','overlaps', 'touches',
-           'within']
\ No newline at end of file
+__all__ = [
+    "overlay",
+    "select",
+    "contains",
+    "crosses",
+    "disjoint",
+    "equals",
+    "overlaps",
+    "touches",
+    "within",
+]
