@@ -9,48 +9,59 @@ from arcgis.features import FeatureCollection
 from arcgis.geoprocessing._job import GPJob
 
 
-def _prevent_bds_item(item):
+def _prevent_bds_item(item: Item) -> object:
     """checks if the input is a valid input for the GeoAnalytics Tool"""
     if isinstance(item, Item):
         raise ValueError(f"The {item.title} is an Item. Please pass the layer instead.")
     return item
 
-def _id_generator(size=6, chars=string.ascii_uppercase + string.digits):
-    return ''.join(random.choice(chars) for _ in range(size))
+
+def _id_generator(size: int = 6, chars: str = None, prefix: str = None) -> str:
+    """generates a random id of a given length"""
+    if chars is None:
+        chars = string.ascii_uppercase + string.digits
+
+    if prefix:
+        return str(prefix) + "".join(random.choice(chars) for _ in range(size))
+    return "".join(random.choice(chars) for _ in range(size))
 
 
 def _feature_input(self, input_layer):
     input_layer_url = ""
     if isinstance(input_layer, arcgis.gis.Item):
-        if input_layer.type.lower() == 'feature service':
-            input_param =  {'url': input_layer.layers[0].url}
-        elif input_layer.type.lower() == 'feature collection':
+        if input_layer.type.lower() == "feature service":
+            input_param = {"url": input_layer.layers[0].url}
+        elif input_layer.type.lower() == "feature collection":
             fcdict = input_layer.get_data()
-            fc = FeatureCollection(fcdict['layers'][0])
-            input_param =  fc.layer
+            fc = FeatureCollection(fcdict["layers"][0])
+            input_param = fc.layer
         else:
             raise TypeError("item type must be feature service or feature collection")
 
     elif isinstance(input_layer, arcgis.features.FeatureLayerCollection):
         input_layer_url = input_layer.layers[0].url
-        input_param =  {"url": input_layer_url }
+        input_param = {"url": input_layer_url}
 
     elif isinstance(input_layer, FeatureCollection):
-        input_param =  input_layer.properties
+        input_param = input_layer.properties
 
     elif isinstance(input_layer, Layer):
         input_layer_url = input_layer.url
-        input_param =  {"url": input_layer_url }
+        input_param = {"url": input_layer_url}
 
     elif isinstance(input_layer, dict):
-        input_param =  input_layer
+        input_param = input_layer
 
     elif isinstance(input_layer, str):
         input_layer_url = input_layer
-        input_param =  {"url": input_layer_url }
+        input_param = {"url": input_layer_url}
 
     else:
-        raise Exception("Invalid format of input layer. url string, feature service Item, feature service instance or dict supported")
+        raise Exception(
+            "Invalid format of input layer. The following formats "
+            "are supported: URL string, feature service item, feat"
+            "ure service instance or dictionary."
+        )
 
     return input_param
 
@@ -65,31 +76,44 @@ def _set_context(params):
     context = {}
     set_context = False
 
-    if default_aggregation_styles is not None and\
-       isinstance(default_aggregation_styles, bool):
-        context['defaultAggregationStyles'] = default_aggregation_styles
+    if default_aggregation_styles is not None and isinstance(
+        default_aggregation_styles, bool
+    ):
+        context["defaultAggregationStyles"] = default_aggregation_styles
         set_context = True
     if out_sr is not None:
-        context['outSR'] = {'wkid': int(out_sr)}
+        context["outSR"] = {"wkid": int(out_sr)}
         set_context = True
     if out_extent is not None:
-        context['extent'] = out_extent
+        context["extent"] = out_extent
         set_context = True
     if process_sr is not None:
-        context['processSR'] = {'wkid': int(process_sr)}
-        set_context = True
+        if isinstance(process_sr, str) and str(process_sr).isdigit() == False:
+            context["processSR"] = {"wkt": process_sr}
+            set_context = True
+        else:
+            context["processSR"] = {"wkid": int(process_sr)}
+            set_context = True
     if output_datastore is not None:
-        context['dataStore'] = output_datastore
+        context["dataStore"] = output_datastore
         set_context = True
 
     if set_context:
         params["context"] = json.dumps(context)
 
 
-def _create_output_service(gis, output_name, output_service_name='Analysis feature service', task='GeoAnalytics', output_datastore=None):
-    ok = gis.content.is_service_name_available(output_name, 'Feature Service')
+def _create_output_service(
+    gis,
+    output_name,
+    output_service_name="Analysis feature service",
+    task="GeoAnalytics",
+    output_datastore=None,
+):
+    ok = gis.content.is_service_name_available(output_name, "Feature Service")
     if not ok:
-        raise RuntimeError("A Feature Service by this name already exists: " + output_name)
+        raise RuntimeError(
+            "A feature service by this name already exists: " + output_name
+        )
     if output_datastore is None:
         if arcgis.env.output_datastore is not None:
             output_datastore = arcgis.env.output_datastore
@@ -117,32 +141,31 @@ def _create_output_service(gis, output_name, output_service_name='Analysis featu
             "enableEditorTracking": False,
             "enableOwnershipAccessControl": False,
             "allowOthersToUpdate": True,
-            "allowOthersToDelete": True
-            },
+            "allowOthersToDelete": True,
+        },
         "xssPreventionInfo": {
             "xssPreventionEnabled": True,
             "xssPreventionRule": "InputOnly",
-            "xssInputRule": "rejectInvalid"
-            },
+            "xssInputRule": "rejectInvalid",
+        },
         "tables": [],
-        "name": output_service_name.replace(' ', '_'),
-        "options": {
-            "dataSourceType": output_datastore
-        }
+        "name": output_service_name.replace(" ", "_"),
+        "options": {"dataSourceType": output_datastore},
     }
 
-    output_service = gis.content.create_service(output_name, create_params=createParameters, service_type="featureService")
-    description = "Feature Service generated from running the " + task + " tool."
+    output_service = gis.content.create_service(
+        output_name, create_params=createParameters, service_type="featureService"
+    )
+    description = "Feature service generated from running the " + task + " tool."
     item_properties = {
-            "description" : description,
-            "tags" : "Analysis Result, " + task,
-            "snippet": output_service_name
-            }
+        "description": description,
+        "tags": "Analysis Result, " + task,
+        "snippet": output_service_name,
+    }
     output_service.update(item_properties)
     return output_service
 
 
-
 class GAJob(object):
     """
     Represents a Single GeoAnalytics Job.  The `GAJob` class allows for the asynchronous operation
@@ -162,10 +185,11 @@ class GAJob(object):
     ================  ===============================================================
 
     """
+
     _gpjob = None
     _return_service = None
     _add_messages = None
-    #----------------------------------------------------------------------
+    # ----------------------------------------------------------------------
     def __init__(self, gpjob, return_service=None, add_messages=False):
         """
         initializer
@@ -174,45 +198,54 @@ class GAJob(object):
         self._gpjob = gpjob
         self._return_service = return_service
         self._add_messages = add_messages
-    #----------------------------------------------------------------------
+
+    # ----------------------------------------------------------------------
     def __str__(self):
         return "<%s GA Job: %s>" % (self.task, self._gpjob._jobid)
-    #----------------------------------------------------------------------
+
+    # ----------------------------------------------------------------------
     def __repr__(self):
         return "<%s GA Job: %s>" % (self.task, self._gpjob._jobid)
-    #----------------------------------------------------------------------
+
+    # ----------------------------------------------------------------------
     @property
     def task(self):
-        """Returns the task name.
-        :returns: string
+        """
+        Returns the task name.
+
+        :return: string
         """
         return self._gpjob.task
-    #----------------------------------------------------------------------
+
+    # ----------------------------------------------------------------------
     @property
     def ellapse_time(self):
         """
         Returns the Ellapse Time for the Job
         """
         return self._gpjob.ellapse_time
-    #----------------------------------------------------------------------
+
+    # ----------------------------------------------------------------------
     @property
     def messages(self):
         """
-        returns the GP messages
+        Returns the GP messages
 
-        :returns: List
+        :return: List
         """
         return self._gpjob.messages
-    #----------------------------------------------------------------------
+
+    # ----------------------------------------------------------------------
     @property
     def status(self):
         """
-        returns the GP status
+        Returns the GP status
 
-        :returns: String
+        :return: String
         """
         return self._gpjob.status
-    #----------------------------------------------------------------------
+
+    # ----------------------------------------------------------------------
     def cancel(self):
         """
         Attempt to cancel the call. If the call is currently being executed
@@ -220,66 +253,69 @@ class GAJob(object):
         return False, otherwise the call will be cancelled and the method
         will return True.
 
-        :returns: boolean
+        :return: Boolean
         """
         cancel = self._gpjob.cancel()
         if self._return_service:
             self._return_service.delete()
         return cancel
 
-    #----------------------------------------------------------------------
+    # ----------------------------------------------------------------------
     def cancelled(self):
         """
         Return True if the call was successfully cancelled.
 
-        :returns: boolean
+        :return: Boolean
         """
         return self._gpjob.cancelled()
-    #----------------------------------------------------------------------
+
+    # ----------------------------------------------------------------------
     def running(self):
         """
         Return True if the call is currently being executed and cannot be cancelled.
 
-        :returns: boolean
+        :return: Boolean
         """
         return self._gpjob.running()
-    #----------------------------------------------------------------------
+
+    # ----------------------------------------------------------------------
     def done(self):
         """
         Return True if the call was successfully cancelled or finished running.
 
-        :returns: boolean
+        :return: Boolean
         """
         return self._gpjob.done()
-    #----------------------------------------------------------------------
+
+    # ----------------------------------------------------------------------
     def process_info(self):
         """
         Returns the Processing Information for a GeoAnalytics job.
 
-        :returns: List or None if process_info does not exist.
+        :return: List or None if process_info does not exist.
 
         """
         processing_info = None
         if self.result():
             res = self.result()
-            if hasattr(res, "_asdict") and \
-               'process_info' in res._asdict().keys():
+            if hasattr(res, "_asdict") and "process_info" in res._asdict().keys():
                 return getattr(res, "process_info")
             else:
                 url = f"{self._gpjob._url}/jobs/{self._gpjob._jobid}"
-                params = {'f' : 'json'}
+                params = {"f": "json"}
                 res = self._gpjob._gis._con.get(url, params)
-                if "results" in res and 'processInfo' in res['results']:
+                if "results" in res and "processInfo" in res["results"]:
                     url = f"{self._gpjob._url}/jobs/{self._gpjob._jobid}/{res['results']['processInfo']['paramUrl']}"
-                    return self._gpjob._gis._con.get(url, params)['value']
+                    return self._gpjob._gis._con.get(url, params)["value"]
         return None
-    #----------------------------------------------------------------------
+
+    # ----------------------------------------------------------------------
     def result(self):
         """
         Return the value returned by the call. If the call hasn't yet completed
         then this method will wait.
 
-        :returns: object
+        :return: object
         """
         try:
 
