@@ -1,7 +1,7 @@
 """
 These tools help answer one of the most common questions posed in spatial analysis: What is near what?
 
-create_buffers() creates areas of a specified distance from features.
+```create_buffers()``` creates areas of a specified distance from features.
 """
 import json as _json
 
@@ -9,7 +9,14 @@ import logging as _logging
 import arcgis as _arcgis
 from arcgis.features import FeatureSet as _FeatureSet, FeatureCollection
 from arcgis.geoprocessing._support import _execute_gp_tool
-from ._util import _id_generator, _feature_input, _set_context, _create_output_service, GAJob, _prevent_bds_item
+from ._util import (
+    _id_generator,
+    _feature_input,
+    _set_context,
+    _create_output_service,
+    GAJob,
+    _prevent_bds_item,
+)
 from arcgis._impl.common._utils import inspect_function_inputs
 from arcgis.geoprocessing import import_toolbox
 
@@ -17,22 +24,167 @@ _log = _logging.getLogger(__name__)
 
 _use_async = True
 
-def trace_proximity_events(input_points,
-                           spatial_search_distance,
-                           spatial_search_distance_unit,
-                           temporal_search_distance,
-                           temporal_search_distance_unit,
-                           entity_id_field=None,
-                           entities_of_interest_ids=None,
-                           entities_of_interest_layer=None,
-                           distance_method="Planar",
-                           include_tracks_layer=False,
-                           max_trace_depth=None,
-                           attribute_match_criteria=None,
-                           output_name=None,
-                           context=None,
-                           gis=None,
-                           future=False):
+
+def group_by_proximity(
+    input_layer,
+    spatial_relationship,
+    spatial_near_distance=None,
+    spatial_near_distance_unit=None,
+    temporal_relationship=None,
+    temporal_near_distance=None,
+    temporal_near_distance_unit=None,
+    output_name=None,
+    context=None,
+    gis=None,
+    future=False,
+):
+    """
+    The Group By Proximity tool groups features that are within spatial
+    proximity or spatiotemporal proximity of each other.
+
+    ===================================================================    =============================================================================
+    **Argument**                                                           **Description**
+    -------------------------------------------------------------------    -----------------------------------------------------------------------------
+    input_layer                                                            Required layer. The point, line, or polygon features to be grouped.
+                                                                           See :ref:`Feature Input<gaxFeatureInput>`.
+    -------------------------------------------------------------------    -----------------------------------------------------------------------------
+    spatial_relationship                                                   Required String. The type of relationship to group by.
+
+                                                                           Values: ```Intersects, Touches, NearGeodesic, NearPlanar```
+    -------------------------------------------------------------------    -----------------------------------------------------------------------------
+    spatial_near_distance                                                  Optional Float. A float value used for the search distance to determine if
+                                                                           features are near one another. This is only applied if `NewGeodesic` or
+                                                                           `NewPlanar` is the selected `spatial_relationship`.
+    -------------------------------------------------------------------    -----------------------------------------------------------------------------
+    spatial_near_distance_unit                                             Optional String. The linear unit to be used with the distance value specified
+                                                                           in `spatial_near_distance`. The default value is Meters.
+
+                                                                           Values: Meters | Kilometers | Feet | Miles | NauticalMiles | Yards
+    -------------------------------------------------------------------    -----------------------------------------------------------------------------
+    temporal_relationship                                                  Optional String. The type of temporal relationship to group by.
+
+                                                                           Values: Intersects | Near
+    -------------------------------------------------------------------    -----------------------------------------------------------------------------
+    temporal_near_distance                                                 Optional Float. A float value used for the temporal search distance to determine if features are near one another.
+
+    -------------------------------------------------------------------    -----------------------------------------------------------------------------
+    temporal_near_distance_unit                                            Optional String. The temporal unit to be used with the distance value specified in `temporal_near_distance`.
+
+                                                                           Values: Milliseconds | Seconds | Minutes | Hours | Days | Weeks| Months | Years
+    -------------------------------------------------------------------    -----------------------------------------------------------------------------
+    output_name                                                            Optional string. The task will create a feature service of the results. You define the name of the service.
+    -------------------------------------------------------------------    -----------------------------------------------------------------------------
+    gis                                                                    Optional GIS. The GIS object where the analysis will take place.
+    -------------------------------------------------------------------    -----------------------------------------------------------------------------
+    context                                                                Optional string. The context parameter contains additional settings that affect task execution. For this task, there are four settings:
+
+                                                                           #. Extent (``extent``) - a bounding box that defines the analysis area. Only those features that intersect the bounding box will be analyzed.
+                                                                           #. Processing spatial reference (``processSR``) The features will be projected into this coordinate system for analysis.
+                                                                           #. Output Spatial Reference (``outSR``) - the features will be projected into this coordinate system after the analysis to be saved. The output spatial reference for the spatiotemporal big data store is always WGS84.
+                                                                           #. Data store (``dataStore``) Results will be saved to the specified data store. For ArcGIS Enterprise, the default is the spatiotemporal big data store.
+    -------------------------------------------------------------------    -----------------------------------------------------------------------------
+    future                                                                 optional Boolean. If True, a GAJob is returned instead of results. The GAJob can be queried on the status of the execution.
+    ===================================================================    =============================================================================
+
+    :return: :class:`~arcgis.gis.Item` when Future=False or GAJob when Future=True
+
+    """
+    input_features = _prevent_bds_item(input_layer)
+
+    gis = _arcgis.env.active_gis if gis is None else gis
+    url = gis.properties.helperServices.geoanalytics.url
+    tbx = import_toolbox(url, gis=gis)
+
+    if output_name is None:
+        output_service_name = _id_generator(prefix="Group_By_Proximity_")
+        output_name = output_service_name.replace(" ", "_")
+    else:
+        output_service_name = output_name.replace(" ", "_")
+    if context is not None:
+        output_datastore = context.get("dataStore", None)
+    else:
+        output_datastore = None
+    output_service = _create_output_service(
+        gis,
+        output_name,
+        output_service_name,
+        "Group By Proximity",
+        output_datastore=output_datastore,
+    )
+
+    params = {
+        "input_features": input_features,
+        "spatial_relationship": spatial_relationship,
+        "spatial_near_distance": spatial_near_distance,
+        "spatial_near_distance_unit": spatial_near_distance_unit,
+        "temporal_relationship": temporal_relationship,
+        "temporal_near_distance": temporal_near_distance,
+        "temporal_near_distance_unit": temporal_near_distance_unit,
+        "output_name": output_name,
+        "context": context,
+        "gis": gis,
+        "future": True,
+    }
+
+    if output_service:
+        params["output_name"] = _json.dumps(
+            {
+                "serviceProperties": {
+                    "name": output_name,
+                    "serviceUrl": output_service.url,
+                },
+                "itemProperties": {"itemId": output_service.itemid},
+            }
+        )
+    else:
+        params["output_name"] = output_name
+        output_service = f"Results were written to: '{params['context']['dataStore']}' with the name: '{output_name}'"
+
+    if context is not None:
+        params["context"] = context
+    else:
+        _set_context(params)
+
+    kwargs = {}
+    for key, value in params.items():
+        if key != "field":
+            if value is not None:
+                kwargs[key] = value
+        elif key == "field" and value:
+            kwargs[key] = value
+    params = inspect_function_inputs(tbx.group_by_proximity, **kwargs)
+    params["future"] = True
+
+    try:
+        gpjob = tbx.group_by_proximity(**params)
+        if future:
+            return GAJob(gpjob=gpjob, return_service=output_service)
+        gpjob.result()
+        return output_service
+    except Exception as e:
+        output_service.delete()
+        raise
+    return
+
+
+def trace_proximity_events(
+    input_points,
+    spatial_search_distance,
+    spatial_search_distance_unit,
+    temporal_search_distance,
+    temporal_search_distance_unit,
+    entity_id_field,
+    entities_of_interest_ids=None,
+    entities_of_interest_layer=None,
+    distance_method="Planar",
+    include_tracks_layer=False,
+    max_trace_depth=None,
+    attribute_match_criteria=None,
+    output_name=None,
+    context=None,
+    gis=None,
+    future=False,
+):
     """
     The Trace Proximity Events task analyzes time-enabled point features representing moving entities.
     The task will follow entities of interest in space (location) and time to see which other entities
@@ -60,7 +212,7 @@ def trace_proximity_events(input_points,
     temporal_search_distance_unit                                          Required String. The unit of `temporal_search_distance`.
                                                                            Values: Milliseconds | Seconds | Minutes | Hours | Days | Weeks| Months | Years
     -------------------------------------------------------------------    -----------------------------------------------------------------------------
-    entity_id_field                                                        Optional String. The field used to identify distinct entities.
+    entity_id_field                                                        Required String. The field used to identify distinct entities.
     -------------------------------------------------------------------    -----------------------------------------------------------------------------
     entities_of_interest_ids                                               Optional List. JSON used to specify one or more entities that you are
                                                                            interested in tracing from. You can optionally include a time to start tracing
@@ -98,79 +250,98 @@ def trace_proximity_events(input_points,
     future                                                                 optional Boolean. If True, a GPJob is returned instead of results. The GPJob can be queried on the status of the execution.
     ===================================================================    =============================================================================
 
-    :returns: Item when Future=False or GAJob when Future=True
+    :return: :class:`~arcgis.gis.Item` when Future=False or GAJob when Future=True
 
     """
     input_points = _prevent_bds_item(input_points)
 
-    if isinstance(input_points, FeatureCollection) and \
-       'layers' in input_points.properties and \
-       len(input_points.properties.layers) > 0:
+    if (
+        isinstance(input_points, FeatureCollection)
+        and "layers" in input_points.properties
+        and len(input_points.properties.layers) > 0
+    ):
         input_points = _FeatureSet.from_dict(
-            featureset_dict=input_points._lazy_properties.layers[0].featureSet)
-    if isinstance(entities_of_interest_layer, FeatureCollection ) and \
-       'layers' in entities_of_interest_layer.properties and \
-       len(entities_of_interest_layer.layers) > 0:
+            featureset_dict=input_points._lazy_properties.layers[0].featureSet
+        )
+    if (
+        isinstance(entities_of_interest_layer, FeatureCollection)
+        and "layers" in entities_of_interest_layer.properties
+        and len(entities_of_interest_layer.layers) > 0
+    ):
         entities_of_interest_layer = _FeatureSet.from_dict(
-            featureset_dict=entities_of_interest_layer._lazy_properties.layers[0].featureSet)
+            featureset_dict=entities_of_interest_layer._lazy_properties.layers[
+                0
+            ].featureSet
+        )
 
     gis = _arcgis.env.active_gis if gis is None else gis
     url = gis.properties.helperServices.geoanalytics.url
     tbx = import_toolbox(url_or_item=url, gis=gis)
 
     if output_name is None:
-        output_service_name = 'TraceProximityEvents_' + _id_generator()
-        output_name = output_service_name.replace(' ', '_')
+        output_service_name = _id_generator(prefix="Trace Proximity Events_")
+        output_name = output_service_name.replace(" ", "_")
     else:
-        output_service_name = output_name.replace(' ', '_')
+        output_service_name = output_name.replace(" ", "_")
     if context is not None:
-        output_datastore = context.get('dataStore', None)
+        output_datastore = context.get("dataStore", None)
     else:
         output_datastore = None
-    output_service = _create_output_service(gis, output_name, output_service_name, 'Trace Proximity Events',
-                                            output_datastore=output_datastore)
+    output_service = _create_output_service(
+        gis,
+        output_name,
+        output_service_name,
+        "Trace Proximity Events",
+        output_datastore=output_datastore,
+    )
 
     params = {
-        "input_points" : input_points,
-        "entity_id_field" : entity_id_field,
-        "entities_of_interest_ids" : entities_of_interest_ids,
-        "entities_of_interest_layer" : entities_of_interest_layer or "",
-        'distance_method' : distance_method or "Planar",
-        "spatial_search_distance" : spatial_search_distance,
-        "spatial_search_distance_unit" : spatial_search_distance_unit,
+        "input_points": input_points,
+        "entity_id_field": entity_id_field,
+        "entities_of_interest_ids": entities_of_interest_ids,
+        "entities_of_interest_layer": entities_of_interest_layer or "",
+        "distance_method": distance_method or "Planar",
+        "spatial_search_distance": spatial_search_distance,
+        "spatial_search_distance_unit": spatial_search_distance_unit,
         "temporal_search_distance": temporal_search_distance,
-        "temporal_search_distance_unit" : temporal_search_distance_unit,
-        "include_tracks_layer" :include_tracks_layer,
-        "max_trace_depth" : max_trace_depth,
-        "attribute_match_criteria" :attribute_match_criteria,
-        "output_name" : output_name,
-        "context":context,
-        "gis" : gis,
+        "temporal_search_distance_unit": temporal_search_distance_unit,
+        "include_tracks_layer": include_tracks_layer,
+        "max_trace_depth": max_trace_depth,
+        "attribute_match_criteria": attribute_match_criteria,
+        "output_name": output_name,
+        "context": context,
+        "gis": gis,
         "future": True,
     }
 
     if output_service:
-        params['output_name'] = _json.dumps({
-            "serviceProperties": {"name" : output_name, "serviceUrl" : output_service.url},
-            "itemProperties": {"itemId" : output_service.itemid}})
+        params["output_name"] = _json.dumps(
+            {
+                "serviceProperties": {
+                    "name": output_name,
+                    "serviceUrl": output_service.url,
+                },
+                "itemProperties": {"itemId": output_service.itemid},
+            }
+        )
     else:
-        params['output_name'] = output_name
+        params["output_name"] = output_name
         output_service = f"Results were written to: '{params['context']['dataStore']}' with the name: '{output_name}'"
 
     if context is not None:
         params["context"] = context
     else:
-        _set_context(params )
+        _set_context(params)
 
     kwargs = {}
     for key, value in params.items():
-        if key != 'field':
+        if key != "field":
             if value is not None:
                 kwargs[key] = value
-        elif key == 'field' and value:
+        elif key == "field" and value:
             kwargs[key] = value
     params = inspect_function_inputs(tbx.trace_proximity_events, **kwargs)
-    params['future'] = True
+    params["future"] = True
 
     try:
         gpjob = tbx.trace_proximity_events(**params)
@@ -182,22 +353,24 @@ def trace_proximity_events(input_points,
         output_service.delete()
         raise
 
-
     return
 
-def create_buffers(input_layer,
-                   distance=1,
-                   distance_unit="Miles",
-                   field=None,
-                   method="Planar",
-                   dissolve_option="None",
-                   dissolve_fields=None,
-                   summary_fields=None,
-                   multipart=False,
-                   output_name=None,
-                   context=None,
-                   gis=None,
-                   future=False):
+
+def create_buffers(
+    input_layer,
+    distance=None,
+    distance_unit=None,
+    field=None,
+    method="Planar",
+    dissolve_option="None",
+    dissolve_fields=None,
+    summary_fields=None,
+    multipart=False,
+    output_name=None,
+    context=None,
+    gis=None,
+    future=False,
+):
     """
 
     .. image:: _static/images/create_buffers_geo/create_buffers_geo.png
@@ -220,13 +393,13 @@ def create_buffers(input_layer,
                                                         You can only enter a single distance value. The units of the
                                                         distance value are supplied by the ``distance_unit`` parameter.
 
-                                                        The default value is 1.
+                                                        The default value is 1 when `field` is None.
     ------------------------------------------------    ---------------------------------------------------------
     distance_unit (Required if distance is used)        Optional string. The linear unit to be used with the value specified in distance.
 
                                                         Choice list:['Feet', 'Yards', 'Miles', 'Meters', 'Kilometers', 'NauticalMiles']
 
-                                                        The default value is "Miles"
+                                                        The default value is "Miles" when `field` is None.
     ------------------------------------------------    ---------------------------------------------------------
     field (Required if distance not provided)           Optional string. A field on the ``input_layer`` containing a buffer distance or a field expression.
                                                         A buffer expression must begin with an equal sign (=). To learn more about buffer expressions
@@ -303,7 +476,7 @@ def create_buffers(input_layer,
                                                         The default value is 'False'
     ================================================    =========================================================
 
-    :returns: Output Features as a feature layer collection item
+    :return: Output Features as a :class:`~arcgis.features.FeatureLayerCollection`
 
     .. code-block:: python
 
@@ -315,71 +488,93 @@ def create_buffers(input_layer,
                                     dissolve_option='All',
                                     dissolve_fields='Date')
     """
+    if field and distance:
+        raise ValueError(
+            "Both a distance and a field cannot be specified.  Choose one and resubmit."
+        )
+    if field is None and distance is None:
+        distance = 1
+    if distance and distance_unit is None:
+        distance_unit = "Miles"
 
     input_layer = _prevent_bds_item(input_layer)
 
     gis = _arcgis.env.active_gis if gis is None else gis
     url = gis.properties.helperServices.geoanalytics.url
 
-    if isinstance(input_layer, FeatureCollection) and \
-       'layers' in input_layer.properties and \
-       len(input_layer.properties.layers) > 0:
+    if (
+        isinstance(input_layer, FeatureCollection)
+        and "layers" in input_layer.properties
+        and len(input_layer.properties.layers) > 0
+    ):
         input_layer = _FeatureSet.from_dict(
-            featureset_dict=input_layer._lazy_properties.layers[0].featureSet)
+            featureset_dict=input_layer._lazy_properties.layers[0].featureSet
+        )
     kwargs = {
-        "input_layer" : input_layer,
-        "distance" : distance,
-        "distance_unit" : distance_unit,
-        "field" : field,
-        "method" : method,
-        "dissolve_option" : dissolve_option,
-        "dissolve_fields" : dissolve_fields,
-        "summary_fields" : summary_fields,
-        "multipart" : multipart,
-        "output_name" : output_name,
-        "context" : context,
-        "gis" : gis,
-        "future" : True
+        "input_layer": input_layer,
+        "distance": distance,
+        "distance_unit": distance_unit,
+        "field": field,
+        "method": method,
+        "dissolve_option": dissolve_option,
+        "dissolve_fields": dissolve_fields,
+        "summary_fields": summary_fields,
+        "multipart": multipart,
+        "output_name": output_name,
+        "context": context,
+        "gis": gis,
+        "future": True,
     }
     params = {}
     for key, value in kwargs.items():
-        if key != 'field':
+        if key != "field":
             if value is not None:
                 params[key] = value
-        elif key == 'field' and value:
+        elif key == "field" and value:
             params[key] = value
     if distance is None:
-        params['distance'] = None
+        params["distance"] = None
     if distance_unit is None:
-        params['distance_unit'] = None
+        params["distance_unit"] = None
     if output_name is None:
-        output_service_name = 'Create Buffers Analysis_' + _id_generator()
-        output_name = output_service_name.replace(' ', '_')
+        output_service_name = _id_generator(prefix="Create Buffers_")
+        output_name = output_service_name.replace(" ", "_")
     else:
-        output_service_name = output_name.replace(' ', '_')
+        output_service_name = output_name.replace(" ", "_")
     if context is not None:
-        output_datastore = context.get('dataStore', None)
+        output_datastore = context.get("dataStore", None)
     else:
         output_datastore = None
-    output_service = _create_output_service(gis, output_name, output_service_name, 'Create Buffers',
-                                            output_datastore=output_datastore)
+    output_service = _create_output_service(
+        gis,
+        output_name,
+        output_service_name,
+        "Create Buffers",
+        output_datastore=output_datastore,
+    )
 
     if output_service:
-        params['output_name'] = _json.dumps({
-            "serviceProperties": {"name" : output_name, "serviceUrl" : output_service.url},
-            "itemProperties": {"itemId" : output_service.itemid}})
+        params["output_name"] = _json.dumps(
+            {
+                "serviceProperties": {
+                    "name": output_name,
+                    "serviceUrl": output_service.url,
+                },
+                "itemProperties": {"itemId": output_service.itemid},
+            }
+        )
     else:
-        params['output_name'] = output_name
+        params["output_name"] = output_name
         output_service = f"Results were written to: '{params['context']['dataStore']}' with the name: '{output_name}'"
 
     if context is not None:
         params["context"] = context
     else:
-        _set_context(params )
+        _set_context(params)
 
     tbx = import_toolbox(url_or_item=url, gis=gis)
     params = inspect_function_inputs(tbx.create_buffers, **params)
-    params['future'] = True
+    params["future"] = True
 
     try:
         gpjob = tbx.create_buffers(**params)
@@ -390,4 +585,3 @@ def create_buffers(input_layer,
     except:
         output_service.delete()
         raise
-
