@@ -7,3 +7,14 @@ The module also provides simple types that can be used as parameters for these t
 from arcgis.geoprocessing._types import LinearUnit, DataFile, RasterData
 from arcgis.geoprocessing._tool import import_toolbox
 from arcgis.geoprocessing._job import GPJob
+from arcgis.geoprocessing._service import GPService, GPTask
+
+__all__ = [
+    "LinearUnit",
+    "DataFile",
+    "RasterData",
+    "import_toolbox",
+    "GPJob",
+    "GPService",
+    "GPTask",
+]
