@@ -7,11 +7,12 @@ from arcgis import env as _env
 from arcgis.gis._impl._jb import StatusJob
 from arcgis.gis import Item
 import concurrent.futures
+
 ###########################################################################
 class PortalDataStore(object):
     """
 
-    The datastores resource page provides access to operations that allow you to do the
+    The ``PortalDatastore`` provides access to operations that allow you to do the
     following:
 
     - Validate a data store against your server.
@@ -20,34 +21,35 @@ class PortalDataStore(object):
     - Refresh your data store registration information in the server.
     - Unregister a data store from your server.
 
-    Additionally, users also have the ability to bulk publish layers from a data store,
-    retrieve a list of previously published layers, and delete bulk-published layers.
+    .. note::
+        Users also have the ability to bulk publish layers from a :class:`~arcgis.gis.Datastore` object,
+        retrieve a list of previously published layers, and delete bulk-published layers.
 
     """
+
     _con = None
     _gis = None
     _url = None
     _properties = None
-    #----------------------------------------------------------------------
+    # ----------------------------------------------------------------------
     def __init__(self, url, gis):
         """Constructor"""
         self._url = url
         self._gis = gis
         self._con = gis._con
-    #----------------------------------------------------------------------
+
+    # ----------------------------------------------------------------------
     def __str__(self):
         return "< PortalDataStore @ {url} >".format(url=self._url)
-    #----------------------------------------------------------------------
+
+    # ----------------------------------------------------------------------
     def __repr__(self):
         return "< PortalDataStore @ {url} >".format(url=self._url)
-    #----------------------------------------------------------------------
-    def describe(self,
-                 item,
-                 server_id,
-                 path,
-                 store_type="datastore"):
+
+    # ----------------------------------------------------------------------
+    def describe(self, item, server_id, path, store_type="datastore"):
         """
-        Describe data store is used to list the contents of a data store. A
+        The ``describe`` method is used to list the contents of a data store. A
         client can use it multiple times to discover the contents of the
         data store incrementally. For example, the client can request a
         description of the root, and then request sub-folders.
@@ -56,9 +58,9 @@ class PortalDataStore(object):
         ==================     ====================================================================
         **Argument**           **Description**
         ------------------     --------------------------------------------------------------------
-        item                   Required Item. The Data Store `Item` to describe.
+        item                   Required Item. The :class:`~arcgis.gis.Item` to describe.
         ------------------     --------------------------------------------------------------------
-        server_id              Optiona String. The unique id of the registered server.
+        server_id              Optional String. The unique id of the registered server.
         ------------------     --------------------------------------------------------------------
         path                   Optional String. The path to examine the data in.
         ------------------     --------------------------------------------------------------------
@@ -66,82 +68,103 @@ class PortalDataStore(object):
                                and for sub-folders, the object type should be listed as `folder`.
         ==================     ====================================================================
 
-        :returns: StatusJob
+        .. code-block:: python
+
+            # Usage Example
+
+            >>> datastore_manager.describe(item= datastore_object1, server_id="server_name", path = "path_name")
+
+        :return:
+            A :class:`~arcgis.gis._impl..StatusJob` object
 
         """
         if isinstance(item, Item):
             item = item.id
         params = {
-            "datastoreId" : item,
-            "serverId" : server_id,
-            "path" : path,
-            "type" : store_type,
-            "f" : "json"
+            "datastoreId": item,
+            "serverId": server_id,
+            "path": path,
+            "type": store_type,
+            "f": "json",
         }
         url = f"{self._url}/describe"
         res = self._con.get(url, params)
-        _time.sleep(.5)
-        executor =  concurrent.futures.ThreadPoolExecutor(1)
-        futureobj = executor.submit(self._status, **{"job_id" : res['jobId'], "key": res['key']})
+        _time.sleep(0.5)
+        executor = concurrent.futures.ThreadPoolExecutor(1)
+        futureobj = executor.submit(
+            self._status, **{"job_id": res["jobId"], "key": res["key"]}
+        )
         executor.shutdown(False)
-        return StatusJob(future=futureobj,
-                         op='Describe DataStore',
-                         jobid=res['jobId'],
-                         gis=self._gis,
-                         notify=_env.verbose,
-                         extra_marker="")
-    #----------------------------------------------------------------------
+        return StatusJob(
+            future=futureobj,
+            op="Describe DataStore",
+            jobid=res["jobId"],
+            gis=self._gis,
+            notify=_env.verbose,
+            extra_marker="",
+        )
+
+    # ----------------------------------------------------------------------
     def _status(self, job_id, key=None):
         """
         Checks the status of an export job
 
-        :returns: dict
+        :return: dict
         """
         params = {}
         if job_id:
             url = f"{self._gis._portal.resturl}portals/self/jobs/{job_id}"
-            params['f'] = 'json'
+            params["f"] = "json"
             res = self._con.post(url, params)
             while res["status"] not in ["completed", "complete", "succeeded"]:
                 res = self._con.post(url, params)
-                if res['status'] == "failed":
+                if res["status"] == "failed":
                     raise Exception(res)
             count = 0
-            while res['status'] in ["completed", "complete", "succeeded"] and \
-                  not 'result' in res and \
-                  count < 10:
+            while (
+                res["status"] in ["completed", "complete", "succeeded"]
+                and not "result" in res
+                and count < 10
+            ):
                 res = self._con.post(url, params)
-                if 'result' in res:
+                if "result" in res:
                     return res
                 count += 1
             return res
         else:
             raise Exception(res)
-    #----------------------------------------------------------------------
+
+    # ----------------------------------------------------------------------
     @property
     def properties(self):
-        """returns the properties of the datastore"""
+        """
+        The ``properties`` property retrieves the properties of the current :class:`~arcgis.gis.DataStore` object
+
+        :return:
+           A list of the :class:`~arcgis.gis.DataStore` object properties
+        """
         if self._properties is None:
-            params = {'f' : 'json'}
+            params = {"f": "json"}
             res = self._con.get(self._url, params)
             self._properties = PropertyMap(res)
         return self._properties
-    #----------------------------------------------------------------------
-    def register(self,
-                 item,
-                 server_id,
-                 bind=False):
+
+    # ----------------------------------------------------------------------
+    def register(self, item, server_id, bind=False):
         """
 
-        The `register` method allows for Data Store type Items to be added to an ArcGIS Server instance.
-        Before registering a data store, it is recommended that you validate your data store with the
-        given server.
+        The ``register`` method allows for :class:`~arcgis.gis.Datastore` objects to be added to an ArcGIS Server
+        instance.
+
+        .. note::
+            Before registering a :class:`~arcgis.gis.Datastore`, it is recommended that you validate your
+            :class:`~arcgis.gis.Datastore` with the given server.
 
         ==================     ====================================================================
         **Argument**           **Description**
         ------------------     --------------------------------------------------------------------
-        item                   Required Item/Item Id (as string). The item Id or Item of the data
-                               store to register with the server. Note that a data store can be
+        item                   Required :class:`~arcgis.gis.Item` object or Item Id (as string). The item Id or Item of
+                               the data store to register with the server. Note that a data store can be
                                registered on multiple servers.
         ------------------     --------------------------------------------------------------------
         server_id              Required String. The unique id of the server you want to register
@@ -149,13 +172,23 @@ class PortalDataStore(object):
         ------------------     --------------------------------------------------------------------
         bind                   Optional Boolean. Specifies whether to bind the data store item to
                                the federated server. For more information about binding a data
-                               store to additional federated servers, see `Create a data store item
-                               for an existing registered data store`_. The default value is false.
+                               store to additional federated servers, see the note below.
+                               The default value is `False`.
         ==================     ====================================================================
 
-        :returns: Boolean
+        .. code-block:: python
 
-        .. _Create a data store item for an existing registered data store: https://enterprise.arcgis.com/en/portal/latest/administer/windows/create-item-from-existing-data-store.htm#ESRI_SECTION1_58D081604CF841AC80D527D34A67660C
+            # Usage Example
+
+            >>> datastore_manager.register(datastore_object1, "server_name")
+
+        :return:
+            A boolean indicating success (True), or failure (False)
+
+        .. note::
+            To create a :class:`~arcgis.gis.Datastore` object from an existing :class:`~arcgis.gis.dataStore`,
+            see the `Create a data store item from an existing data store <https://enterprise.arcgis.com/en/portal/latest/administer/windows/create-item-from-existing-data-store.htm#ESRI_SECTION1_58D081604CF841AC80D527D34A67660C>`_
+            page in the ArcGIS Online resources.
 
         """
         if isinstance(item, Item):
@@ -163,18 +196,15 @@ class PortalDataStore(object):
         elif isinstance(item, str):
             item_id = item
         url = "{base}/addToServer".format(base=self._url)
-        params = {
-            'f' : 'json',
-            'datastoreId' : item_id,
-            'serverId' : server_id
-        }
+        params = {"f": "json", "datastoreId": item_id, "serverId": server_id}
         if not bind is None:
-            params['bindToServerDatastore'] = bind
+            params["bindToServerDatastore"] = bind
         res = self._con.post(url, params)
-        if 'success' in res:
-            return res['success']
+        if "success" in res:
+            return res["success"]
         return res
-    #----------------------------------------------------------------------
+
+    # ----------------------------------------------------------------------
     @property
     def _all_datasets(self):
         """
@@ -184,32 +214,41 @@ class PortalDataStore(object):
         and remove all previously published layers in preparation for the
         deregistration of a data store.
 
-        :returns: Boolean
+        :return: Boolean
 
         """
         url = "{base}/allDatasets".format(base=self._url)
-        params = {
-            'f' : 'json'}
+        params = {"f": "json"}
         res = self._con.post(url, params)
-        if 'success' in res:
-            return res['success']
-        elif 'status' in res:
-            return res['status'] == 'success'
+        if "success" in res:
+            return res["success"]
+        elif "status" in res:
+            return res["status"] == "success"
         return res
-    #----------------------------------------------------------------------
+
+    # ----------------------------------------------------------------------
     def delete_layers(self, item):
         """
-        Before a data store can be unregistered from a server, all of its
-        bulk-published layers must be deleted. The delete_layers removes all
-        layers published from the data store.
+        The ``delete_layers`` method removes all layers published from the :class:`~arcgis.gis.Datastore` object.
+
+        .. note::
+            Before a :class:`~arcgis.gis.Datastore` object can be unregistered from a server, all of its
+            bulk-published layers must be deleted.
 
         ==================     ====================================================================
         **Argument**           **Description**
         ------------------     --------------------------------------------------------------------
-        item                   Required Item. The Data Store `Item` to delete all published layers.
+        item                   Required Item. The :class:`~arcgis.gis.Datastore` to delete all published layers.
         ==================     ====================================================================
 
-        :returns: Boolean
+        .. code-block:: python
+
+            # Usage Example
+
+            >>> datastore_manager.delete_layers(datastore_object)
+
+        :return:
+            A boolean indicating success (True), or failure (False)
 
         """
         if isinstance(item, Item):
@@ -217,29 +256,30 @@ class PortalDataStore(object):
         else:
             item_id = item
             item = self._gis.content.get(item_id)
-        params = {
-            'f' : 'json',
-            'datastoreId' : item_id
-        }
+        params = {"f": "json", "datastoreId": item_id}
         url = f"{self._url}/allDatasets/deleteLayers"
         res = self._con.post(url, params)
-        if res['success'] == True:
-            status = item.status(self, job_id=res['jobId'])
-            while status["status"].lower() != 'completed':
-                status = item.status(self, job_id=res['jobId'])
-                if status['status'].lower() == "failed":
+        if res["success"] == True:
+            status = item.status(self, job_id=res["jobId"])
+            while status["status"].lower() != "completed":
+                status = item.status(self, job_id=res["jobId"])
+                if status["status"].lower() == "failed":
                     return False
                 else:
                     _time.sleep(2)
             return True
         return False
-    #----------------------------------------------------------------------
+
+    # ----------------------------------------------------------------------
     def layers(self, item):
         """
-        The `layers operation returns a list of layers bulk published from a
-        data store with the `publish_layers` method. The `layers` method
-        returns an array of tuples, with each tuple containing two
-        objects: a layer and the dataset it was published from.
+        The ``layers`` operation returns a list of layers bulk published from a
+        :class:`~arcgis.gis.Datastore` with the
+        :attr:`~arcgis.gis._impl._datastores._ds.PortalDataStore.publish_layers` method.
+
+        .. note::
+            The ``layers`` method returns an list of tuples, with each tuple containing two
+            objects: a layer and the dataset it was published from.
 
         ==================     ====================================================================
         **Argument**           **Description**
@@ -248,7 +288,13 @@ class PortalDataStore(object):
                                and registered datasets.
         ==================     ====================================================================
 
-        :returns: List
+        .. code-block:: python
+
+            # Usage Example
+
+            >>> datastore_manager.layers(datastore_object1)
+
+        :return: A List of bulk-published :class:`~arcgis.gis.Layer` objects
 
         """
 
@@ -259,23 +305,19 @@ class PortalDataStore(object):
             item = self._gis.content.get(item_id)
 
         url = "{base}/allDatasets/getLayers".format(base=self._url)
-        params = {
-            'f' : 'json',
-            'datastoreId' : item_id
-        }
+        params = {"f": "json", "datastoreId": item_id}
         res = self._con.post(url, params)
         if "layerAndDatasets" in res:
             return res["layerAndDatasets"]
         return []
-    #----------------------------------------------------------------------
-    def publish(self,
-                config:dict,
-                server_id,
-                folder=None,
-                description=None,
-                tags:list=None):
+
+    # ----------------------------------------------------------------------
+    def publish(
+        self, config: dict, server_id, folder=None, description=None, tags: list = None
+    ):
         """
-        The publish operation is used to publish scene layers by reference to data in a data store.
+        The ``publish`` operation is used to publish scene layers by reference to data in a
+        :class:`~arcgis.gis.Datastore`.
 
         ==================     ====================================================================
         **Argument**           **Description**
@@ -296,7 +338,14 @@ class PortalDataStore(object):
         tags                   Optional list. An array of descriptive words that describes the newly published dataset.  This will be added to the `Item`
         ==================     ====================================================================
 
-        :returns: StatusJob
+        .. code-block:: python
+
+            # Usage Example
+
+            >>> datastore_manager.publish(config= {"required" : "dictionary"}, server_id= "server_name")
+
+        :return:
+            A :class:`~arcgis.gis._impl._jb.StatusJob` object
 
         """
         url = f"{self._url}/publish"
@@ -304,24 +353,32 @@ class PortalDataStore(object):
             tags = ",".join([str(t) for t in tags])
 
         params = {
-            "serviceConfiguration" : config,
-            "serverId" : server_id,
-            "serverFolder" : folder,
-            "description" : description,
-            "tags" : tags,
-            "f" : "json"
+            "serviceConfiguration": config,
+            "serverId": server_id,
+            "serverFolder": folder,
+            "description": description,
+            "tags": tags,
+            "f": "json",
         }
         res = self._con.post(url, params)
-        executor =  concurrent.futures.ThreadPoolExecutor(1)
-        futureobj = executor.submit(self._status, **{"job_id" : res['jobId'], "key": res['key']})
+        executor = concurrent.futures.ThreadPoolExecutor(1)
+        futureobj = executor.submit(
+            self._status, **{"job_id": res["jobId"], "key": res["key"]}
+        )
         executor.shutdown(False)
-        return StatusJob(future=futureobj, op='Publish',
-                         jobid=res['jobId'], gis=self._gis,
-                         notify=_env.verbose, extra_marker="")
-    #----------------------------------------------------------------------
+        return StatusJob(
+            future=futureobj,
+            op="Publish",
+            jobid=res["jobId"],
+            gis=self._gis,
+            notify=_env.verbose,
+            extra_marker="",
+        )
+
+    # ----------------------------------------------------------------------
     def servers(self, item):
         """
-        The `servers` property returns a list of your servers that a given
+        The ``servers`` property returns a list of your servers that a given
         data store has been registered to. This operation returns the
         serverId, the server name, both the server and admin URLs, and
         whether or not the server is hosted.
@@ -330,10 +387,17 @@ class PortalDataStore(object):
         ==================     ====================================================================
         **Argument**           **Description**
         ------------------     --------------------------------------------------------------------
-        item                   Required Item. The Data Store `Item` to list all registered servers.
+        item                   Required Item. The DataStore :class:`~arcgis.gis.Item` object to list all registered servers.
         ==================     ====================================================================
 
-        :returns: List
+        .. code-block:: python
+
+            # Usage Example
+
+            >>> datastore_manager.servers(datastore_object1)
+
+        :return:
+            A List with the serverID, server name, server URL, and admin URLs
 
         """
 
@@ -344,32 +408,34 @@ class PortalDataStore(object):
             item = self._gis.content.get(item_id)
 
         url = self._url + "/getServers"
-        params = {
-            'f' : 'json',
-            'datastoreId' : item_id
-        }
+        params = {"f": "json", "datastoreId": item_id}
         res = self._con.post(url, params)
-        if 'servers' in res:
-            return res['servers']
+        if "servers" in res:
+            return res["servers"]
         return res
-    #----------------------------------------------------------------------
-    def publish_layers(self,
-                       item,
-                       srv_config:dict,
-                       server_id,
-                       folder=None,
-                       server_folder=None,
-                       future=False):
-        """
-        The `publish_layers` operation publishes, or syncs, the datasets from a
-        data store onto your ArcGIS Server, resulting in at least one layer per
-        dataset.
 
-        When this operation is called for the first time, every parameter in
-        the operation must be passed through. On subsequent calls,
-        publish_layers will synchronize the datasets and created layers, which
-        includes both publishing layers from new datasets and removing layers
-        for datasets no longer found in the data store.
+    # ----------------------------------------------------------------------
+    def publish_layers(
+        self,
+        item,
+        srv_config: dict,
+        server_id,
+        folder=None,
+        server_folder=None,
+        future=False,
+    ):
+        """
+        The ``publish_layers`` operation publishes, or syncs, the datasets from a
+        :class:`~arcgis.gis.DataStore` object onto your ArcGIS Server, resulting in at least one
+        :attr:`~arcgis.gis.Layer` per dataset. It is quite similar to the
+        :attr:`~arcgis.gis._impl._datastores._ds.PortalDataStore.publish` method.
+
+        .. note::
+            When this operation is called for the first time, every parameter in
+            the operation must be passed through. On subsequent calls,
+            publish_layers will synchronize the datasets and created layers, which
+            includes both publishing layers from new datasets and removing layers
+            for datasets no longer found in the data store.
 
         ==================     ====================================================================
         **Argument**           **Description**
@@ -392,7 +458,14 @@ class PortalDataStore(object):
                                `StatusJob` is returned.
         ==================     ====================================================================
 
-        :returns: Boolean when `future=False` else a `StatusJob`
+        .. code-block:: python
+
+            # Usage Example
+
+            >>> datastore_manager.publish_layers(datastore_object1, srv_config= {"required" : "dictionary"})
+
+        :return:
+            Boolean when `future=False` else a :class:`~arcgis.gis._impl._jb.StatusJob` object
 
         """
         if server_folder is None:
@@ -401,10 +474,13 @@ class PortalDataStore(object):
                 base = item.title.lower().replace(" ", "")
             server_folder = f"{base}{uuid.uuid4().hex[:3]}"
         if folder is None:
-            from arcgis.gis  import UserManager, User, ContentManager
+            from arcgis.gis import UserManager, User, ContentManager
+
             isinstance(self._gis, GIS)
             cm = self._gis.content
-            folder = cm.create_folder(folder=f"srvc_folder_{uuid.uuid4().hex[:5]}")['id']
+            folder = cm.create_folder(folder=f"srvc_folder_{uuid.uuid4().hex[:5]}")[
+                "id"
+            ]
         if isinstance(item, Item):
             item_id = item.id
         else:
@@ -412,32 +488,38 @@ class PortalDataStore(object):
             item = self._gis.content.get(item_id)
         url = self._url + "/allDatasets/publishLayers"
         params = {
-            'f' : 'json',
-            'datastoreId' : item_id,
-            'templateSvcConfig' : srv_config,
-            'portalFolderId' : folder,
-            'serverId' : server_id,
-            'serverFolder' : server_folder,
+            "f": "json",
+            "datastoreId": item_id,
+            "templateSvcConfig": srv_config,
+            "portalFolderId": folder,
+            "serverId": server_id,
+            "serverFolder": server_folder,
         }
         res = self._con.post(url, params)
-        if res['success'] == True:
+        if res["success"] == True:
             status = item.status()
-            while status["status"].lower() != 'completed':
+            while status["status"].lower() != "completed":
                 status = item.status()
-                if status['status'].lower() == "failed":
+                if status["status"].lower() == "failed":
                     return False
                 else:
                     _time.sleep(2)
             return True
         return False
-    #----------------------------------------------------------------------
-    def unregister(self,
-                   item,
-                   server_id):
-        """
-        Removes the datastore association from a server.
 
-        :returns: Boolean
+    # ----------------------------------------------------------------------
+    def unregister(self, item, server_id):
+        """
+        The ``unregister`` method removes the :class:`~arcgis.gis.Datastore` association from a server.
+
+        .. code-block:: python
+
+            # Usage Example
+
+            >>> datastore_manager.unregister(datastore_object1, "server_id")
+
+        :return:
+            A boolean indicating success (True), or failure (False)
 
         """
         if isinstance(item, Item):
@@ -445,28 +527,27 @@ class PortalDataStore(object):
         elif isinstance(item, str):
             item_id = item
         url = self._url + "/removeFromServer"
-        params = {
-            'f' : 'json',
-            'datastoreId' : item_id,
-            'serverId' : server_id
-        }
+        params = {"f": "json", "datastoreId": item_id, "serverId": server_id}
         res = self._con.post(url, params)
-        if 'success' in res:
-            return res['success']
+        if "success" in res:
+            return res["success"]
         return res
-    #----------------------------------------------------------------------
-    def refresh_server(self,
-                       item,
-                       server_id):
+
+    # ----------------------------------------------------------------------
+    def refresh_server(self, item, server_id):
         """
-        After a data store has been registered, there may be times in which
-        the data store's registration information may be changed. When
-        changes like these occur, the server will need to be updated with
-        the newly configured information so that your users will still be
-        able to access the data store items without interruption. The
-        `refresh_server` can be called to propagate these changes to your
-        ArcGIS Server. This operation can only be performed after the data
-        store information has been updated.
+        Th ``refresh_server`` method updates the server with newly configured inforamtion.
+
+        .. note::
+            After a :class:`~arcgis.gis.Datastore` has been registered, there may be times in which
+            the :class:`~arcgis.gis.Datastore` object's registration information may be changed. When
+            changes like these occur, the server will need to be updated with
+            the newly configured information so that your users will still be
+            able to access the data store items without interruption.
+
+        .. warning::
+            This operation can only be performed after the data
+            store information has been updated.
 
         ==================     ====================================================================
         **Argument**           **Description**
@@ -479,7 +560,14 @@ class PortalDataStore(object):
                                the datastore with.
         ==================     ====================================================================
 
-        :returns: Boolean
+        .. code-block:: python
+
+            # Usage Example
+
+            >>> datastore_manager.refresh_server(datastore_object1, "server_id")
+
+        :return:
+            A boolean indicating success (True), or failure (False)
 
         """
         if isinstance(item, Item):
@@ -488,28 +576,24 @@ class PortalDataStore(object):
             item_id = item
 
         url = self._url + "/refreshServer"
-        params = {
-            'f' : 'json',
-            'datastoreId' : item_id,
-            'serverId' : server_id
-        }
+        params = {"f": "json", "datastoreId": item_id, "serverId": server_id}
         res = self._con.post(url, params)
-        if 'success' in res:
-            return res['success']
+        if "success" in res:
+            return res["success"]
         return res
-    #----------------------------------------------------------------------
-    def validate(self,
-                 server_id,
-                 item=None,
-                 config=None,
-                 future=False):
+
+    # ----------------------------------------------------------------------
+    def validate(self, server_id, item=None, config=None, future=False):
         """
-        The `validate` ensures that your ArcGIS Server can connect and use
-        the datasets stored within a given data store. While this operation
-        can be called before or after the data store has been registered
-        with your server, it is recommended that the validate operation is
-        performed beforehand. A data store can be validated by using either
-        its datastoreId or the JSON for an unregistered data store.
+        The ``validate`` method ensures that your ArcGIS Server can connect and use
+        the datasets stored within a given data store. A :class:`~arcgis.gis.Datastore` can be validated by using either
+        its datastoreId or the JSON for an unregistered :class:`~arcgis.gis.Datastore`.
+
+        .. note::
+            While this operation
+            can be called before or after the :class:`~arcgis.gis.Datastore` has been registered
+            with your server, it is recommended that the validate operation is
+            performed beforehand.
 
         ==================     ====================================================================
         **Argument**           **Description**
@@ -524,7 +608,14 @@ class PortalDataStore(object):
         config                 Optional dict. The connection information for a new datastore.
         ==================     ====================================================================
 
-        :returns: Boolean
+        .. code-block:: python
+
+            # Usage Example
+
+            >>> datastore_manager.validate("server_name")
+
+        :return:
+            A boolean indicating success (True), or failure (False)
 
         """
 
@@ -536,20 +627,18 @@ class PortalDataStore(object):
             item_id = None
 
         url = self._url + "/validate"
-        params = {
-                'f' : 'json',
-                'serverId' : server_id
-            }
+        params = {"f": "json", "serverId": server_id}
         if item:
-            params['datastoreId'] = item_id
+            params["datastoreId"] = item_id
         elif config:
             import json
-            params['datastore'] = json.dumps(config)
+
+            params["datastore"] = json.dumps(config)
         else:
             raise ValueError("Invalid parameters, an item or config is required.")
         res = self._con.post(url, params)
-        if 'status' in res:
-            if res['status'] == 'success':
+        if "status" in res:
+            if res["status"] == "success":
                 return True
-            return res['status']
+            return res["status"]
         return res
