@@ -16,7 +16,7 @@ from ._machines import Machines, Machine
 from ._security import EnterpriseGroups, EnterpriseUsers, OAuth
 from ._security import Security, SSLCertificate, SSLCertificates
 from ._site import Site
-from ._system import Directory, Licenses, System
+from ._system import Directory, Licenses, System, Indexer, EmailManager
 from ._system import PortalLicense
 from ._system import WebAdaptor, WebAdaptors
 from ._collaboration import Collaboration, CollaborationManager
@@ -28,4 +28,5 @@ from ._catagoryschema import CategoryManager
 from ._idp import IdentityProviderManager
 from ._wh import WebhookManager, Webhook
 from ._usage import AGOLUsageReports
-__all__ = ['PortalAdminManager', 'AGOLAdminManager']
\ No newline at end of file
+
+__all__ = ["PortalAdminManager", "AGOLAdminManager"]
