@@ -3,43 +3,52 @@ import json
 from arcgis._impl.common._mixins import PropertyMap
 from arcgis.gis import GIS
 
+
 class WebhookManager(object):
     """
-    Creates and manages ArcGIS Portal WebHooks.  Webhooks allow you to be
+    Creates and manages ArcGIS Enterprise webhooks.  Webhooks allow you to be
     automatically notified when events associated with items, groups, and
     users occur. Once a webhook has been triggered, an HTTP request is
     made to a user-defined URL to provide information regarding the event.
     """
+
     _con = None
     _gis = None
     _url = None
     _properties = None
+
     def __init__(self, url, gis):
         self._url = url
         self._gis = gis
         isinstance(self._gis, GIS)
         self._con = self._gis._con
-    #----------------------------------------------------------------------
+
+    # ----------------------------------------------------------------------
     def _init(self):
         """initializer"""
-        params = {'f':'json'}
+        params = {"f": "json"}
         res = self._con.get(self._url, params)
         self._properties = PropertyMap(res)
-    #----------------------------------------------------------------------
+
+    # ----------------------------------------------------------------------
     def __str__(self):
         from urllib.parse import urlparse
+
         return "<WebhookManager @ {id}>".format(id=urlparse(self._url).netloc)
-    #----------------------------------------------------------------------
+
+    # ----------------------------------------------------------------------
     def __repr__(self):
         return self.__str__()
-    #----------------------------------------------------------------------
+
+    # ----------------------------------------------------------------------
     @property
     def properties(self):
         """returns the Webhook properties"""
         if self._properties is None:
             self._init()
         return self._properties
-    #----------------------------------------------------------------------
+
+    # ----------------------------------------------------------------------
     @property
     def settings(self):
         """
@@ -70,50 +79,29 @@ class WebhookManager(object):
 
         """
         url = "%s/settings" % self._url
-        params = {'f':'json'}
+        params = {"f": "json"}
         return self._con.get(url, params)
-    #----------------------------------------------------------------------
+
+    # ----------------------------------------------------------------------
     @settings.setter
     def settings(self, value):
         """
-        There are several advanced parameters that can be used to configure
-        the connection behavior of your webhook. These parameters will be
-        applied to all of the configured webhooks in your Portal. Use the
-        Update operation to modify any of the parameters.
-
-
-
-        ** Dictionary Key/Values **
-
-        =================================  ===============================================================================
-        **Argument**                       **Description**
-        ---------------------------------  -------------------------------------------------------------------------------
-        notificationAttempts               Required Integer. This will determine how many attempts will be made to deliver
-                                           a payload.
-        ---------------------------------  -------------------------------------------------------------------------------
-        otificationTimeOutInSeconds        Required Integer. The length of time (in seconds) that Portal will wait to
-                                           receive a response. The max response is 60.
-        ---------------------------------  -------------------------------------------------------------------------------
-        notificationElapsedTimeInSeconds   Required Integer. The amount of time between each payload delivery attempt. By
-                                           default, this is set to 30 seconds and can be set to a maximum of 100 seconds
-                                           and a minimum of one second.
-        =================================  ===============================================================================
-
-
+        See main ``settings`` property docstring
         """
         url = "%s/settings/update" % self._url
-        params = {'f':'json'}
+        params = {"f": "json"}
 
         params = {
             "notificationAttempts": 4,
             "notificationTimeOutInSeconds": 11,
             "notificationElapsedTimeInSeconds": 6,
-            "f": "json"
+            "f": "json",
         }
-        for k,v in value.items():
+        for k, v in value.items():
             params[k] = v
         self._con.post(url, params)
-    #----------------------------------------------------------------------
+
+    # ----------------------------------------------------------------------
     def get(self, name):
         """finds a single instance of a webhook by name"""
         for wh in self.list():
@@ -121,14 +109,18 @@ class WebhookManager(object):
                 return wh
             del wh
         return
-    #----------------------------------------------------------------------
-    def create(self,
-               name,
-               url,
-               events="ALL",
-               number_of_failures=5,
-               days_in_past=5,
-               secret=None):
+
+    # ----------------------------------------------------------------------
+    def create(
+        self,
+        name,
+        url,
+        events="ALL",
+        number_of_failures=5,
+        days_in_past=5,
+        secret=None,
+        properties=None,
+    ):
         """
         Creates a WebHook to monitor REST endpoints and report activities
 
@@ -223,6 +215,8 @@ class WebhookManager(object):
         ---------------------------------  -------------------------------------------------------------------------------
         secret                             Optional String. Add a Secret to your payload that can be used to authenticate
                                            the message on your receiver.
+        ---------------------------------  -------------------------------------------------------------------------------
+        properties                         Optional Dict. At 10.9.1+ users can provide additional configuration properties.
         =================================  ===============================================================================
 
         :returns a :class:`WebHook<arcgis.gis.admin.Webhook>` instance
@@ -240,90 +234,104 @@ class WebhookManager(object):
                                url="https://hooks.zapier.com/hooks/catch/6694048/odqj9o3/",
                                events=["/items/981e98b949d9432ebf26433f40948cec/move",
                                        "/items/981e98b949d9432ebf26433f40948cec/update"]
-        
+
         See `Webhook Blog Post <https://www.esri.com/arcgis-blog/products/arcgis-enterprise/administration/webhooks-dev-summit-2019/>`_ for a detailed explanation.
 
         """
         if secret is None:
             secret = ""
         purl = "%s/createWebhook" % self._url
-        params = {
-            'f': 'json',
-            'name' : name,
-            'url' : url,
-            'secret' : secret,
-            'configuration' : {
-                "numberOfFailures" : number_of_failures,
-                "daysInPast" : days_in_past
+        config = {
+            "deactivationPolicy": {
+                "numberOfFailures": number_of_failures,
+                "daysInPast": days_in_past,
             }
         }
+        if properties:
+            config["properties"] = properties
+        params = {
+            "f": "json",
+            "name": name,
+            "url": url,
+            "secret": secret,
+            "config": config,
+        }
         if str(events).lower() == "all":
-            params['changes'] = "allChanges"
+            params["changes"] = "allChanges"
             res = self._con.post(purl, params)
 
         elif isinstance(events, list):
-            params['changes'] =  'manualChanges'
-            params['events'] = ",".join(events)
+            params["changes"] = "manualChanges"
+            params["events"] = ",".join(events)
             res = self._con.post(purl, params)
-        if 'success' in res and \
-           res['success']:
+        if "success" in res and res["success"]:
             return self.get(name=name)
         return None
-    #----------------------------------------------------------------------
+
+    # ----------------------------------------------------------------------
     def list(self):
         """Returns a list of WebHook objects"""
         hooks = []
         self._properties = None
         for wh in self.properties.webhooks:
             try:
-                url = "%s/%s" % (self._url, wh['id'])
+                url = "%s/%s" % (self._url, wh["id"])
                 hooks.append(Webhook(url=url, gis=self._gis))
-            except: pass
+            except:
+                pass
         return hooks
+
+
 ########################################################################
 class Webhook(object):
     """a single webhook"""
+
     _con = None
     _gis = None
-    _url =  None
+    _url = None
     _properties = None
-    #----------------------------------------------------------------------
+    # ----------------------------------------------------------------------
     def __init__(self, url, gis):
         """Constructor"""
         self._url = url
         self._gis = gis
         self._con = gis._con
-    #----------------------------------------------------------------------
+
+    # ----------------------------------------------------------------------
     def __str__(self):
         return "<WebHook @ {name}>".format(name=self.properties.name)
-    #----------------------------------------------------------------------
+
+    # ----------------------------------------------------------------------
     def __repr__(self):
         return self.__str__()
-    #----------------------------------------------------------------------
+
+    # ----------------------------------------------------------------------
     def _init(self):
         """Constructor"""
         if self._properties is None:
-            self._properties = PropertyMap(
-                self._con.get(self._url, {'f': 'json'}))
-    #----------------------------------------------------------------------
+            self._properties = PropertyMap(self._con.get(self._url, {"f": "json"}))
+
+    # ----------------------------------------------------------------------
     @property
     def properties(self):
-        """"""
+        """ """
         if self._properties is None:
             self._init()
         return self._properties
-    #----------------------------------------------------------------------
+
+    # ----------------------------------------------------------------------
     def delete(self):
         """
         Removes the current webhook from the system.
 
-        :returns: Boolean
+        :return: Boolean
 
         """
         url = self._url + "/delete"
-        params = {'f' : 'json'}
-        return self._con.post(url, params)['success']
-    #----------------------------------------------------------------------
+        params = {"f": "json"}
+        return self._con.post(url, params)["success"]
+
+    # ----------------------------------------------------------------------
     @property
     def notifications(self):
         """
@@ -333,60 +341,61 @@ class Webhook(object):
         payloads such as the time the webhook was triggered, the response
         received from the payload URL, and the delivered payload data.
 
-        :returns: List
+        :return: List
 
         """
         url = "%s/notificationStatus" % self._url
         messages = []
-        params = {
-            "num" : 100,
-            "start" : 1,
-            'f' : 'json'
-        }
+        params = {"num": 100, "start": 1, "f": "json"}
         res = self._con.post(url, params)
-        if len(res['WebhookStatus']) > 0:
-            messages += res['WebhookStatus']
-        while res['nextStart'] != -1:
-            params['start'] = res['nextStart']
+        if len(res["WebhookStatus"]) > 0:
+            messages += res["WebhookStatus"]
+        while res["nextStart"] != -1:
+            params["start"] = res["nextStart"]
             res = self._con.post(url, params)
             messages += res["WebhookStatus"]
-            if res['nextStart'] == -1:
+            if res["nextStart"] == -1:
                 return messages
         return messages
-    #----------------------------------------------------------------------
+
+    # ----------------------------------------------------------------------
     def deactivate(self):
         """
         Temporarily pause the webhook. This will stop the webhook from
         delivering payloads when it is invoked. The webhook will be
         automatically deactivated when the deactivation policy is met.
 
-        :returns: boolean
+        :return: boolean
         """
         url = self._url + "/deactivate"
-        params = {'f': 'json'}
+        params = {"f": "json"}
         res = self._con.post(url, params)
         self._properties = None
-        if 'success' in res:
-            return res['success']
+        if "success" in res:
+            return res["success"]
         return False
-    #----------------------------------------------------------------------
+
+    # ----------------------------------------------------------------------
     def activate(self):
-        """"""
+        """ """
         url = self._url + "/activate"
-        params = {'f': 'json'}
+        params = {"f": "json"}
         res = self._con.post(url, params)
         self._properties = None
-        if 'success' in res:
-            return res['success']
+        if "success" in res:
+            return res["success"]
         return False
-    #----------------------------------------------------------------------
-    def update(self,
-               name=None,
-               url=None,
-               events=None,
-               number_of_failures=None,
-               days_in_past=None,
-               secret=None):
+
+    # ----------------------------------------------------------------------
+    def update(
+        self,
+        name=None,
+        url=None,
+        events=None,
+        number_of_failures=None,
+        days_in_past=None,
+        secret=None,
+    ):
         """
         The Update Webhook operation allows administrators to update any of
         the parameters of their webhook.
@@ -473,14 +482,19 @@ class Webhook(object):
                                             | Enable a specific user's account                   | /users/<username>/enable  |
                                             +----------------------------------------------------+---------------------------+
 
-                                           Example Syntax: ['/users', '/groups/abcd1234....']
+                                           .. code-block:: python
+
+                                               #Example Usage:
+
+                                               >>> events = ['/users', '/groups/abcd1234....']
 
         ---------------------------------  -------------------------------------------------------------------------------
-        number_of_failures                 Optional Integer. The number of failures to allow before the service
+        number_of_failures                 Optional Integer. The number of failures to allow before the webhook is
+                                           deactivated.
         ---------------------------------  -------------------------------------------------------------------------------
         days_in_past                       Option Integer. The number of days to report back on.
         ---------------------------------  -------------------------------------------------------------------------------
-        secret                             Optional String. Add a Secret to your payload that can be used to authenticate
+        secret                             Optional String. Add a secret to your payload that can be used to authenticate
                                            the message on your receiver.
         =================================  ===============================================================================
 
@@ -494,7 +508,9 @@ class Webhook(object):
         if url is None:
             url = self.properties.payloadUrl
         if number_of_failures is None:
-            number_of_failures = self.properties.config.deactivationPolicy.numberOfFailures
+            number_of_failures = (
+                self.properties.config.deactivationPolicy.numberOfFailures
+            )
         if days_in_past is None:
             days_in_past = self.properties.config.deactivationPolicy.daysInPast
         if events is None:
@@ -502,57 +518,21 @@ class Webhook(object):
         purl = self._url + "/update"
         self._properties = None
         params = {
-            'f': 'json',
-            'name' : name,
-            'url' : url,
-            'secret' : secret,
-            'configuration' : {
-                "numberOfFailures" : number_of_failures,
-                "daysInPast" : days_in_past
-            }
+            "f": "json",
+            "name": name,
+            "url": url,
+            "secret": secret,
+            "config": {
+                "deactivationPolicy": {
+                    "numberOfFailures": number_of_failures,
+                    "daysInPast": days_in_past,
+                }
+            },
         }
 
-        params['events'] = events
+        params["events"] = events
         res = self._con.post(purl, params)
 
-        if 'success' in res:
-            return res['success']
+        if "success" in res:
+            return res["success"]
         return False
-
-#if __name__ == "__main__":
-    #from arcgis.gis import GIS
-    #gis = GIS(url="https://dev0005215.esri.com/portal")
-    #print(gis)
-    #url = "{baseurl}portals/self/webhooks".format(baseurl=gis._portal.resturl)
-    #whm = WebhookManager(url=url, gis=gis)
-    #hook1 = whm.create(name="update_test", url="https://JJEYARAJAH.esri.com:8001", number_of_failures=5)
-    #hook1.update(name='thisisupdated')
-    #hook1.update(events='/users')
-    #hook1.delete()
-    #print(hook1.properties.name)
-
-    #hook2 = whm.get("update_test")
-    #isinstance(hook, Webhook)
-    #assert hook.update(name="update_test", url="https://JJEYARAJAH.esri.com:8001", events="/users")
-    #print(hook.properties)
-    #print(hook.delete())
-    ##assert isinstance(whm, WebhookManager)
-    ##assert isinstance(whm.list(), list)
-    ##assert isinstance(whm.properties, PropertyMap)
-
-    ##wh = whm.get("Users Webhook")
-    ##assert isinstance(wh, Webhook)
-    ##assert isinstance(wh.notifications, list)
-    ###assert whm.create(name="AmazingWebHook", url="https://JJEYARAJAH.esri.com:8001", number_of_failures=5)
-    ##wh = whm.get("AmazingWebHook")
-    ##assert wh.deactivate()
-    ##assert wh.properties.isActive == False
-    ##assert wh.activate()
-    ##assert wh.properties.isActive == True
-
-    ##assert wh.delete()
-
-
-
-    ##print(whm.properties)
-    #print()
