@@ -1,5 +1,5 @@
 """
-A collection of classes for administering an ArcGIS Enterprise server.
+A collection of classes for administering ArcGIS Server sites.
 """
 from .catalog import ServicesDirectory
 from ._service import Service
@@ -16,8 +16,15 @@ from .admin._logs import LogManager
 from .admin._machines import Machine, MachineManager
 from .admin._mode import Mode
 from .admin._security import Role, RoleManager, User, UserManager
-from .admin._services import Service, ServiceManager, ItemInformationManager
+from .admin._services import (
+    Service,
+    ServiceManager,
+    ItemInformationManager,
+    JobManager,
+    Job,
+)
 from .admin._system import ConfigurationStore, DirectoryManager, Jobs
 from .admin._system import ServerDirectory, ServerProperties, SystemManager
 from .admin._uploads import Uploads
 from .admin._usagereports import Report, ReportManager
+from .admin._mode import Mode
