@@ -4,5 +4,5 @@ Classes for ArcGIS Services
 from __future__ import absolute_import
 from ._layerfactory import Service
 
-__version__ = "1.8.5"
-__all__ = ['Service']
+__version__ = "2.0.0"
+__all__ = ["Service"]
