@@ -1,12 +1,14 @@
 try:
     from ._data import _raise_fastai_import_error
-    from ._tracking.track_processor import TrackProcessor
+    from ._tracking.track_processor import TrackProcessor, get_default_tracker_options
     import traceback
+
     HAS_FASTAI = True
 except ImportError:
     import_exception = traceback.format_exc()
     HAS_FASTAI = False
 
+
 class ObjectTracker:
     """
     Creates ObjectTracker Object.
@@ -24,9 +26,15 @@ class ObjectTracker:
                             keys as parameter names and values as
                             parameter values.
 
+                            "enable_post_processing" refers to
+                            the flag which enables/disables post_processing
+                            of tracks internal to ObjectTracker module.
+                            For DeepSort, it's recommended to keep this
+                            flag as False. Default - True
+
                             "detection_interval" refers to
                             the interval in frames at which the detector
-                            is invoked.
+                            is invoked. It should be >= 1
 
                             "detection_threshold" refers to
                             the lower threshold for selecting the
@@ -79,54 +87,87 @@ class ObjectTracker:
                             "recover_conf_threshold" refers
                             to the minimum confidence value over which
                             recovery logic is enabled.
-                            
+
                             "recover_iou_threshold" refers to the minimum
                             overlap between template and detection for
                             successful recovery.
     =====================   ===========================================
 
-    :returns: `ObjectTracker` Object
+    :return: `ObjectTracker` Object
     """
-    def __init__(self, tracker, detector=None, tracker_options={"detection_interval": 5,
-                                                                "detection_threshold": 0.3,
-                                                                "detect_track_failure": True,
-                                                                "recover_track": True,
-                                                                "stab_period": 6,
-                                                                "detect_fail_interval": 5,
-                                                                "min_obj_size": 10,
-                                                                "template_history": 25,
-                                                                "status_history": 60,
-                                                                "status_fail_threshold": 0.6,
-                                                                "search_period": 60,
-                                                                "knn_distance_ratio": 0.75,
-                                                                "recover_conf_threshold": 0.1,
-                                                                "recover_iou_threshold": 0.1}):
+
+    def __init__(
+        self,
+        tracker,
+        detector=None,
+        tracker_options={
+            "enable_post_processing": True,
+            "detection_interval": 5,
+            "detection_threshold": 0.3,
+            "detect_track_failure": True,
+            "recover_track": True,
+            "stab_period": 6,
+            "detect_fail_interval": 5,
+            "min_obj_size": 10,
+            "template_history": 25,
+            "status_history": 60,
+            "status_fail_threshold": 0.6,
+            "search_period": 60,
+            "knn_distance_ratio": 0.75,
+            "recover_conf_threshold": 0.1,
+            "recover_iou_threshold": 0.1,
+        },
+    ):
         if not HAS_FASTAI:
             _raise_fastai_import_error(import_exception=import_exception)
 
+        if tracker_options is None:
+            tracker_options = get_default_tracker_options()
+
         self.tracker = tracker
         self.detector = detector
         self.tracks = []
-        self.detect_interval = tracker_options.get('detection_interval', 10)
-        self.detect_threshold = tracker_options.get('detection_threshold', 0.3)
+        self.detect_interval = tracker_options.get("detection_interval", 10)
+        self.detect_threshold = tracker_options.get("detection_threshold", 0.3)
+        self.enable_post_processing = tracker_options.get(
+            "enable_post_processing", True
+        )
         processor_options = {}
-        processor_options["detect_track_failure"] = tracker_options.get('detect_track_failure', True)
-        processor_options["recover_track"] = tracker_options.get('recover_track', True)
-        processor_options["stab_period"] = tracker_options.get('stab_period', 6)
-        processor_options["detect_fail_interval"] = tracker_options.get('detect_fail_interval', 5)
-        processor_options["min_obj_size"] = tracker_options.get('min_obj_size', 10)
-        processor_options["template_history"] = tracker_options.get('template_history', 25)
-        processor_options["status_history"] = tracker_options.get('status_history', 60)
-        processor_options["status_fail_threshold"] = tracker_options.get('status_fail_threshold', 0.6)
-        processor_options["search_period"] = tracker_options.get('search_period', 60)
-        processor_options["knn_distance_ratio"] = tracker_options.get('knn_distance_ratio', 0.75)
-        processor_options["recover_conf_threshold"] = tracker_options.get('recover_conf_threshold', 0.1)
-        processor_options["recover_iou_threshold"] = tracker_options.get('recover_iou_threshold', 0.1)
+        processor_options["detect_track_failure"] = tracker_options.get(
+            "detect_track_failure", True
+        )
+        processor_options["recover_track"] = tracker_options.get("recover_track", True)
+        processor_options["stab_period"] = tracker_options.get("stab_period", 6)
+        processor_options["detect_fail_interval"] = tracker_options.get(
+            "detect_fail_interval", 5
+        )
+        processor_options["min_obj_size"] = tracker_options.get("min_obj_size", 10)
+        processor_options["template_history"] = tracker_options.get(
+            "template_history", 25
+        )
+        processor_options["status_history"] = tracker_options.get("status_history", 60)
+        processor_options["status_fail_threshold"] = tracker_options.get(
+            "status_fail_threshold", 0.6
+        )
+        processor_options["search_period"] = tracker_options.get("search_period", 60)
+        processor_options["knn_distance_ratio"] = tracker_options.get(
+            "knn_distance_ratio", 0.75
+        )
+        processor_options["recover_conf_threshold"] = tracker_options.get(
+            "recover_conf_threshold", 0.1
+        )
+        processor_options["recover_iou_threshold"] = tracker_options.get(
+            "recover_iou_threshold", 0.1
+        )
 
-        self.processor = TrackProcessor(processor_options)
+        self.processor = None
+        if self.enable_post_processing:
+            TrackProcessor(processor_options)
         self.frames_processed = 0
 
-    def init(self, frame, detections=None, labels=None, reset=True):
+    def init(
+        self, frame, detections=None, labels=None, reset=True
+    ):  # TODO: pass scores
         """
         Initializes tracks based on the detections returned by detector/
         manually fed to the function.
@@ -148,29 +189,45 @@ class ObjectTracker:
                                 before initialization.
         =====================   ===========================================
 
-        :returns: list of active track objects
+        :return: list of active track objects
         """
         if detections is None:
             if self.detector is not None:
-                predictions, labels, scores = self.detector.predict(frame, return_scores=True)
-                detections = self._convert_to_dets(predictions, labels, scores)
+                predictions, labels, scores = self.detector.predict(
+                    frame, return_scores=True
+                )
+                detections, labels, scores = self._filter_dets(
+                    predictions, labels, scores
+                )
         else:
+            # TODO: see deepsort get_corrected_labels_scores
             if labels is None or len(labels) != len(detections):
-                labels = ['Object'] * len(detections)
+                labels = ["Object"] * len(detections)
 
-        if detections is not None:
-            if len(detections) > 0:
-                self.tracks = self.tracker.init(frame, detections, labels, reset)
-                track_list = self._convert_tracks_to_list(self.tracks)
-                self.processor.init(frame, track_list, reset)
+        self.tracks = self.tracker.init(
+            frame,
+            detections=detections,
+            labels=labels,
+            reset=reset,
+            scores=scores,
+            update_interval=self.detect_interval,
+        )
+
+        if (
+            self.enable_post_processing is True
+            and self.processor is not None
+            and not detections is None
+        ):
+            track_list = self._convert_tracks_to_list(self.tracks)
+            self.processor.init(frame, track_list, reset)
 
         if reset:
             self.frames_processed = 0
-        return self._check_status(self.tracks)
+        return self._get_active_tracks(self.tracks)
 
     def update(self, frame):
         """
-        Tracks the position of the object in the frame/Image
+        Tracks the position of the object in the frame/Image.
 
         =====================   ===========================================
         **Argument**            **Description**
@@ -179,30 +236,35 @@ class ObjectTracker:
                                 frame to be used to track the objects.
         =====================   ===========================================
 
-        :returns: list of active track objects
+        :return: list of active track objects
         """
-        if self.frames_processed % self.detect_interval == 0 and self.detector is not None:
-            self.tracks = self.tracker.update(frame)
-            predictions, labels, scores = self.detector.predict(frame, return_scores=True)
-            dets = self._convert_to_dets(predictions, labels, scores)
-            if len(dets) > 0:
-                self.tracks = self.tracker.init(frame, dets)
+        if (
+            self.frames_processed % self.detect_interval == 0
+            and self.detector is not None
+        ):
+            predictions, labels, scores = self.detector.predict(
+                frame, return_scores=True
+            )
+            detections, labels, scores = self._filter_dets(predictions, labels, scores)
+            self.tracks = self.tracker.update(
+                frame, detections=detections, labels=labels, scores=scores
+            )
+            if self.enable_post_processing is True and self.processor is not None:
                 tracks_list = self._convert_tracks_to_list(self.tracks)
                 self.processor.init(frame, tracks_list, False)
         else:
-            if len(self.tracks) > 0:
-                self.tracks = self.tracker.update(frame)
+            self.tracks = self.tracker.update(frame)
+            if self.enable_post_processing is True and self.processor is not None:
                 tracks_list = self._convert_tracks_to_list(self.tracks)
                 tracks_list = self.processor.update(frame, tracks_list)
                 self.tracks = self._update_processed_tracks(tracks_list)
 
         self.frames_processed = self.frames_processed + 1
-
-        return self._check_status(self.tracks)
+        return self._get_active_tracks(self.tracks)
 
     def remove(self, tracks_ids):
         """
-        Removes the tracks corresponding to track_ids parameter
+        Removes the tracks corresponding to track_ids parameter.
 
         =====================   ===========================================
         **Argument**            **Description**
@@ -213,14 +275,15 @@ class ObjectTracker:
 
         """
         self.tracks = self.tracker.remove(tracks_ids)
-        self.processor.remove(tracks_ids)
+        if self.enable_post_processing is True and self.processor is not None:
+            self.processor.remove(tracks_ids)
 
         return self.tracks
 
-    def _convert_to_dets(self, predictions, labels, scores):
+    def _filter_dets(self, predictions, labels, scores):
         """
-        Converts the predictions to 1D list which can be used by
-        TrackProcessor
+        Filters the predictions and then converts it to 1D list which can be used by
+        TrackProcessor.
 
         =====================   ===========================================
         **Argument**            **Description**
@@ -235,29 +298,37 @@ class ObjectTracker:
                                 the predictions.
         =====================   ===========================================
 
-        :returns: 1D list with predictions
+        :return: 1D lists with predictions, labels, scores
         """
-        if len(predictions) == 0:
-            return []
+        if predictions is None or len(predictions) == 0:
+            return [], [], []
 
-        dets = []
+        tdets = []
+        tlabels = []
+        tscores = []
         offset = 0
         max_objects = 1000
 
         for index in range(offset, min(max_objects, len(predictions) - offset)):
             prediction = predictions[index]
-            label = labels[index]
-            score = scores[index]
+            label = "Object"
+            score = 0.0
+            if labels is not None and index < len(labels):
+                label = labels[index]
+            if scores is not None and index < len(scores):
+                score = scores[index]
 
             if score > self.detect_threshold:
-                dets.append(prediction)
+                tdets.append(prediction)
+                tlabels.append(label)
+                tscores.append(score)
 
-        return dets
+        return tdets, tlabels, tscores
 
     def _convert_tracks_to_list(self, tracks):
         """
         Converts tracks list to 1D list which can be used by
-        TrackProcessor
+        TrackProcessor.
 
         =====================   ===========================================
         **Argument**            **Description**
@@ -266,16 +337,19 @@ class ObjectTracker:
                                 TrackProcessor.
         =====================   ===========================================
 
-        :returns: 1D list with values of track members used by
+        :return: 1D list with values of track members used by
                   TrackProcessor
         """
-        tracks_list=[]
+        tracks_list = []
+        if tracks is None:
+            return tracks_list
+
         tracks_list.append(len(tracks))
         tracks_list.append(10)
 
         for track in tracks:
             tracks_list.append(int(track.id))
-            tracks_list.append(str(track.label)) #int(track.class_id)
+            tracks_list.append(str(track.label))  # int(track.class_id)
             tracks_list.append(int(track.age))
             tracks_list.append(float(track.score))
             tracks_list.append(int(track.bbox[0]))
@@ -291,7 +365,7 @@ class ObjectTracker:
 
     def _update_processed_tracks(self, tracks_list):
         """
-        Uses 1D list returned by TrackProcessor to update list of tracks
+        Uses 1D list returned by TrackProcessor to update list of tracks.
 
         =====================   ===========================================
         **Argument**            **Description**
@@ -300,9 +374,9 @@ class ObjectTracker:
                                 members.
         =====================   ===========================================
 
-        :returns: list of track objects
+        :return: list of track objects
         """
-        if tracks_list is None:
+        if tracks_list is None or len(tracks_list) == 0:
             return []
 
         num_dets = tracks_list[0]
@@ -324,9 +398,9 @@ class ObjectTracker:
 
         return self.tracks
 
-    def _check_status(self, tracks):
+    def _get_active_tracks(self, tracks):
         """
-        Filters the active tracks using the argument tracks
+        Filters the active tracks using the argument tracks.
 
         =====================   ===========================================
         **Argument**            **Description**
@@ -335,7 +409,11 @@ class ObjectTracker:
                                 filtered.
         =====================   ===========================================
 
-        :returns: list of active track objects
+        :return: list of active track objects
         """
-        active_tracks = filter(lambda track: track.status == 16, tracks)
+        # TODO: 16
+        active_tracks = list(filter(lambda track: track.status == 16, tracks))
         return active_tracks
+
+
+# TODO: Implement predict_video
