@@ -2,7 +2,6 @@ from .._utils.env import _LAMBDA_TEXT_CLASSIFICATION
 
 if not _LAMBDA_TEXT_CLASSIFICATION:
     from ._ssd import SingleShotDetector
-    from ._inferencing import _DynamicSSD
     from ._unet import UnetClassifier
     from ._retinanet import RetinaNet
     from ._classifier import FeatureClassifier
@@ -24,5 +23,14 @@ if not _LAMBDA_TEXT_CLASSIFICATION:
     from ._multi_task_road_extractor import MultiTaskRoadExtractor
     from ._change_detector import ChangeDetector
     from ._pix2pix import Pix2Pix
-    from ._siammask import SiamMask, Track
+    from ._mmdetection import MMDetection
+    from ._embeddings import Embeddings
     from ._connect_net import ConnectNet
+    from ._mmsegmentation import MMSegmentation
+    from ._siammask import SiamMask, Track
+    from ._deepsort import DeepSort
+    from ._auto_ml import AutoML
+    from ._pix2pix_hd import Pix2PixHD
+    from ._auto_dl import AutoDL, ImageryModel
+    from ._max_deeplab import MaXDeepLab
+    from ._wnet_cgan import WNet_cGAN
