@@ -5,7 +5,13 @@ try:
     import torch.nn as nn
     import math
     from torch import tensor
-    from .util import variable_tile_size_check
+    from .util import (
+        variable_tile_size_check,
+        fold_tensor,
+        unfold_tensor,
+        dihedral_transform,
+        create_interpolation_mask,
+    )
 
     HAS_TORCH = True
 except Exception:
@@ -31,7 +37,9 @@ def split_tensor(tensor, tile_size, stride):
     mask_p = unfold(softmax_mask)
     patches = unfold(tensor)
 
-    patches = patches.reshape(number_of_bands, tile_size, tile_size, -1).permute(3, 0, 1, 2)
+    patches = patches.reshape(number_of_bands, tile_size, tile_size, -1).permute(
+        3, 0, 1, 2
+    )
     if tensor.is_cuda:
         patches_base = torch.zeros(patches.size(), device=tensor.get_device())
     else:
@@ -41,8 +49,14 @@ def split_tensor(tensor, tile_size, stride):
 
 
 def rebuild_tensor(input_tensor, mask_t, t_size, tile_size, stride):
-    input_tensor_permuted = input_tensor.permute(1, 2, 3, 0).reshape(-1, input_tensor.size(0)).unsqueeze(0)
-    fold = nn.Fold(output_size=(t_size[0], t_size[1]), kernel_size=(tile_size, tile_size), stride=stride)
+    input_tensor_permuted = (
+        input_tensor.permute(1, 2, 3, 0).reshape(-1, input_tensor.size(0)).unsqueeze(0)
+    )
+    fold = nn.Fold(
+        output_size=(t_size[0], t_size[1]),
+        kernel_size=(tile_size, tile_size),
+        stride=stride,
+    )
     output_tensor = fold(input_tensor_permuted) / fold(mask_t)
     return output_tensor
 
@@ -94,7 +108,7 @@ def get_tile_size(model_height, model_width, padding, batch_height, batch_width)
 
 
 def tile_to_batch(
-        pixel_block, model_height, model_width, padding, fixed_tile_size=True, **kwargs
+    pixel_block, model_height, model_width, padding, fixed_tile_size=True, **kwargs
 ):
     inner_width = model_width - 2 * padding
     inner_height = model_height - 2 * padding
@@ -119,13 +133,13 @@ def tile_to_batch(
 
         # pixel block might not be the shape (band_count, model_height, model_width)
         sub_pixel_block = pixel_block[
-                          :,
-                          y * inner_height: y * inner_height + model_height,
-                          x * inner_width: x * inner_width + model_width,
-                          ]
+            :,
+            y * inner_height : y * inner_height + model_height,
+            x * inner_width : x * inner_width + model_width,
+        ]
         sub_pixel_block_shape = sub_pixel_block.shape
         batch[
-        b, :, : sub_pixel_block_shape[1], : sub_pixel_block_shape[2]
+            b, :, : sub_pixel_block_shape[1], : sub_pixel_block_shape[2]
         ] = sub_pixel_block
 
     return batch, batch_height, batch_width
@@ -143,9 +157,9 @@ def batch_to_tile(batch, batch_height, batch_width):
         x = int(b % batch_width)
 
         tile[
-        :,
-        y * inner_height: (y + 1) * inner_height,
-        x * inner_width: (x + 1) * inner_width,
+            :,
+            y * inner_height : (y + 1) * inner_height,
+            x * inner_width : (x + 1) * inner_width,
         ] = batch[b]
 
     return tile
@@ -202,11 +216,13 @@ class ChildImageClassifier:
                     "value": 4,
                     "displayName": "Batch Size",
                     "description": "Batch Size",
-                }
+                },
             ]
-
         )
-        if self.json_info.get('ArcGISLearnVersion', False) and self.json_info["ArcGISLearnVersion"] > "1.8.4":
+        if (
+            self.json_info.get("ArcGISLearnVersion", False)
+            and self.json_info["ArcGISLearnVersion"] > "1.8.4"
+        ):
             required_parameters.extend(
                 [
                     {
@@ -218,35 +234,92 @@ class ChildImageClassifier:
                         "description": "If True, will return the probability surface of the result.",
                     },
                     {
-                        'name': 'threshold',
-                        'dataType': 'numeric',
-                        'value': 0.5,
-                        'required': False,
-                        'displayName': 'Confidence Score Threshold [0.0, 1.0]',
-                        'description': 'Confidence score threshold value [0.0, 1.0]'
-                    }
-                ])
-        required_parameters = variable_tile_size_check(self.json_info, required_parameters)
+                        "name": "threshold",
+                        "dataType": "numeric",
+                        "value": 0.5,
+                        "required": False,
+                        "displayName": "Confidence Score Threshold [0.0, 1.0]",
+                        "description": "Confidence score threshold value [0.0, 1.0]",
+                    },
+                ]
+            )
+
+        required_parameters.extend(
+            [
+                {
+                    "name": "test_time_augmentation",
+                    "dataType": "string",
+                    "required": False,
+                    "value": "False"
+                    if "test_time_augmentation" not in self.json_info
+                    else str(self.json_info["test_time_augmentation"]),
+                    "displayName": "Perform test time augmentation while predicting",
+                    "description": "If True, will merge predictions from flipped and rotated images.",
+                },
+                {
+                    "name": "merge_policy",
+                    "dataType": "string",
+                    "required": False,
+                    "value": "max",
+                    "displayName": "Policy for merging augmented predictions",
+                    "description": "Policy for merging predictions('mean', 'max' or 'min'). Applicable when test_time_augmentation is True.",
+                },
+            ]
+        )
+
+        required_parameters = variable_tile_size_check(
+            self.json_info, required_parameters
+        )
         return required_parameters
 
     def getConfiguration(self, **scalars):
 
-        self.tytx = int(scalars.get('tile_size', self.json_info['ImageHeight']))
+        self.tytx = int(scalars.get("tile_size", self.json_info["ImageHeight"]))
         self.padding = int(
-            scalars.get("padding", self.json_info["ImageHeight"] // 4))  # Default padding Imageheight//4.
-        self.batch_size = int(math.sqrt(int(scalars.get("batch_size", 4)))) ** 2  # Default 4 batch_size
-        self.predict_background = True  # Default value True
-        self.probability_raster = scalars.get("return_probability_raster", "false").lower() in [
+            scalars.get("padding", self.tytx // 4)
+        )  ## Default padding Imageheight//4.
+        self.batch_size = (
+            int(math.sqrt(int(scalars.get("batch_size", 4)))) ** 2
+        )  # Default 4 batch_size
+        self.predict_background = scalars.get("predict_background", "true").lower() in [
             "true",
             "1",
             "t",
             "y",
-            "yes"
+            "yes",
+        ]  # Default value True
+        self.probability_raster = scalars.get(
+            "return_probability_raster", "false"
+        ).lower() in [
+            "true",
+            "1",
+            "t",
+            "y",
+            "yes",
         ]  # Default value False
-        self.rectangle_height, self.rectangle_width = calculate_rectangle_size_from_batch_size(self.batch_size)
-        ty, tx = get_tile_size(self.tytx, self.tytx, self.padding, self.rectangle_height, self.rectangle_width)
+        (
+            self.rectangle_height,
+            self.rectangle_width,
+        ) = calculate_rectangle_size_from_batch_size(self.batch_size)
+        ty, tx = get_tile_size(
+            self.tytx,
+            self.tytx,
+            self.padding,
+            self.rectangle_height,
+            self.rectangle_width,
+        )
 
-        self.thres = float(scalars.get('threshold', 0.5))  ## Default 0.5 threshold.
+        self.thres = float(scalars.get("threshold", 0.5))  ## Default 0.5 threshold.
+
+        self.use_tta = scalars.get("test_time_augmentation", "false").lower() in [
+            "true",
+            "1",
+            "t",
+            "y",
+            "yes",
+        ]  # Default value True
+
+        self.merge_policy = scalars.get("merge_policy", "mean").lower()
 
         return {
             "extractBands": tuple(self.json_info["ExtractBands"]),
@@ -255,20 +328,19 @@ class ChildImageClassifier:
             "ty": ty,
             "threshold": self.thres,
             "fixedTileSize": 1,
+            "test_time_augmentation": self.use_tta,
+            "merge_policy": self.merge_policy,
         }
 
-    def pixel_classify_image(self, model, tiles, device, classes, predict_bg, model_info):
+    def pixel_classify_image(
+        self, model, tiles, device, classes, predict_bg, model_info
+    ):
         model = model.to(device).eval()
         # logger.info("Tiles length is ", len(tiles))
         normed_batch_tensor = tensor(tiles).to(device).float()
         # logger.info("Normed length is ", len(normed_batch_tensor))
         with torch.no_grad():
             output, _ = model(normed_batch_tensor)
-            # logger.debug("Output  is ", output)
-            # logger.debug("Output  shape is ", output.shape)
-        # _, road_pixels = torch.max(output, axis=1)
-        # if not predict_bg:
-        #    road_pixels[road_pixels == 0] = -1
 
         if predict_bg:
             return output.max(dim=1)[1]
@@ -305,29 +377,91 @@ class ChildImageClassifier:
 
         return semantic_predictions
 
-    def detectRoads(self, tlc, shape, props, **pixelBlocks):
+    def split_predict_interpolate(self, normalized_image_tensor):
+        model_arch = self.json_info["ModelParameters"]["mtl_model"]
+        kernel_size = self.tytx
+        stride = kernel_size - (2 * self.padding)
 
-        input_image = pixelBlocks["raster_pixels"].astype(np.float32)
-        input_image_tensor = tensor(input_image).to(self.device).float()
-
-        kernel_size = self.tytx  # json_info["ImageHeight"]
-        stride = 2 * self.padding
+        if model_arch == "hourglass":
+            kernel_size = (math.ceil(kernel_size / 2)) * 2
 
         # Split image into overlapping tiles
-        mask_t, base_tensor, t_size, patches = split_tensor(input_image_tensor.unsqueeze(0), kernel_size, stride)
+        masks, t_size, patches = unfold_tensor(
+            normalized_image_tensor, kernel_size, stride
+        )
 
-        # predict
         with torch.no_grad():
             output, _ = self.model(patches)
 
-        # get probablity of roads - class 1
-        softmax_output = output.softmax(dim=1)[:, [1], :, :]  # probability of road (class 1)
+        interpolation_mask = create_interpolation_mask(
+            kernel_size, 0, self.device, "hann"
+        )
+        output = output * interpolation_mask
+        masks = masks * interpolation_mask
 
         # merge predictions from overlapping chips
-        softmax_surface = rebuild_tensor(softmax_output, mask_t, t_size, kernel_size, stride)
-        if self.probability_raster:
-            predictions = (softmax_surface * tensor(1.))
+        int_surface = fold_tensor(output, masks, t_size, kernel_size, stride)
+
+        return int_surface
+
+    def tta_predict(self, normalized_image_tensor, test_time_aug=True):
+        all_activations = []
+
+        transforms = [0]
+        if test_time_aug:
+            if self.json_info["ImageSpaceUsed"] == "MAP_SPACE":
+                transforms = list(range(8))
+            else:
+                transforms = [
+                    0,
+                    2,
+                ]  # no vertical flips for pixel space (oriented imagery)
+
+        for k in transforms:
+            flipped_image_tensor = dihedral_transform(normalized_image_tensor[0], k)
+            int_surface = self.split_predict_interpolate(
+                flipped_image_tensor.unsqueeze(0)
+            )
+            corrected_activation = dihedral_transform(int_surface[0], k)
+
+            if k in [5, 6]:
+                corrected_activation = dihedral_transform(int_surface[0], k).rot90(
+                    2, [1, 2]
+                )
+
+            all_activations.append(corrected_activation)
+
+        all_activations = torch.stack(all_activations)
+
+        return all_activations
+
+    def detectRoads(self, tlc, shape, props, **pixelBlocks):  # 8 x 224 x 224 x 3
+        input_image = pixelBlocks["raster_pixels"].astype(np.float32)
+        input_image_tensor = torch.tensor(input_image).to(self.device).float()
+
+        normalized_image_tensor = input_image_tensor.unsqueeze(0)
+
+        all_activations = self.tta_predict(
+            normalized_image_tensor, test_time_aug=self.use_tta
+        )
+
+        softmax_surface = all_activations.softmax(dim=1)[:, [1], :, :]
+        if self.merge_policy == "max":
+            activations = softmax_surface.max(dim=0)[0]
+        elif self.merge_policy == "min":
+            activations = softmax_surface.min(dim=0)[0]
         else:
-            predictions = (softmax_surface.gt(self.thres) * tensor(1.))
+            activations = softmax_surface.mean(dim=0)
+
+        if self.probability_raster:
+            predictions = activations
+        else:
+            predictions = activations.gt(self.thres) * torch.tensor(1)
+
         pad = self.padding
-        return predictions[0][0][pad:-pad, pad:-pad].unsqueeze(0).cpu().numpy()
+        return (
+            predictions[0][pad : -pad or None, pad : -pad or None]
+            .unsqueeze(0)
+            .cpu()
+            .numpy()
+        )
