@@ -5,12 +5,11 @@ from ._codetemplate import code
 import random
 import statistics
 import warnings
-from .._data import _raise_fastai_import_error  
+from .._data import _raise_fastai_import_error
 import traceback
 
 HAS_OPENCV = True
 HAS_FASTAI = True
-HAS_ARCPY = True
 
 # Try to import the necessary modules
 # Exception will turn the HAS_FASTAI flag to false so that relevant exception can be raised
@@ -25,19 +24,35 @@ try:
     from fastai.vision.image import open_image, bb2hw, image2np, Image, pil2tensor
     from fastai.core import ifnone
     from torchvision import models
-    from .._utils.pascal_voc_rectangles import ObjectDetectionCategoryList, show_results_multispectral
-    from ._retinanet_utils import RetinaNetModel, RetinaNetFocalLoss, compute_class_AP, get_predictions
+    from .._utils.pascal_voc_rectangles import (
+        ObjectDetectionCategoryList,
+        show_results_multispectral,
+    )
+    from ._retinanet_utils import (
+        RetinaNetModel,
+        RetinaNetFocalLoss,
+        compute_class_AP,
+        get_predictions,
+        AveragePrecision,
+    )
     from fastai.callbacks import EarlyStoppingCallback
     from fastai.basic_train import Learner
     from ._arcgis_model import SaveModelCallback, _resnet_family
-    from .._image_utils import _get_image_chips, _get_transformed_predictions, _draw_predictions, _exclude_detection
+    from .._image_utils import (
+        _get_image_chips,
+        _get_transformed_predictions,
+        _draw_predictions,
+        _exclude_detection,
+    )
     from .._video_utils import VideoUtils
     from .._utils.common import get_multispectral_data_params_from_emd, _get_emd_path
     from fastprogress.fastprogress import progress_bar
     from .._utils.env import _IS_ARCGISPRONOTEBOOK
     import matplotlib.pyplot as plt
 except Exception as e:
-    import_exception = "\n".join(traceback.format_exception(type(e), e, e.__traceback__))
+    import_exception = "\n".join(
+        traceback.format_exception(type(e), e, e.__traceback__)
+    )
     HAS_FASTAI = False
 
 try:
@@ -45,15 +60,11 @@ try:
 except:
     HAS_OPENCV = False
 
-try:
-    import arcpy
-except:
-    HAS_ARCPY = False
 
 class RetinaNet(ArcGISModel):
     """
     Creates a RetinaNet Object Detector with the specified zoom scales
-    and aspect ratios. 
+    and aspect ratios.
     Based on the Fast.ai notebook at https://github.com/fastai/fastai_dev/blob/master/dev_nb/102a_coco.ipynb
 
     =====================   ===========================================
@@ -69,67 +80,99 @@ class RetinaNet(ArcGISModel):
     ---------------------   -------------------------------------------
     backbone                Optional function. Backbone CNN model to be used for
                             creating the base of the `RetinaNet`, which
-                            is `resnet50` by default. 
+                            is `resnet50` by default.
                             Compatible backbones: 'resnet18', 'resnet34', 'resnet50', 'resnet101', 'resnet152'
     ---------------------   -------------------------------------------
     pretrained_path         Optional string. Path where pre-trained model is
                             saved.
     =====================   ===========================================
-    
-    :returns: `RetinaNet` Object
+
+    :return: `RetinaNet` Object
     """
 
-    def __init__(self, data, scales=None, ratios=None, backbone=None, pretrained_path=None, *args, **kwargs):
+    def __init__(
+        self,
+        data,
+        scales=None,
+        ratios=None,
+        backbone=None,
+        pretrained_path=None,
+        *args,
+        **kwargs,
+    ):
+
+        if pretrained_path is not None:
+            backbone_pretrained = False
+        else:
+            backbone_pretrained = True
 
         # Set default backbone to be 'resnet50'
-        if backbone is None: 
+        if backbone is None:
             backbone = models.resnet50
 
         self._check_dataset_support(data)
         if not (self._check_backbone_support(backbone)):
-            raise Exception (f"Enter only compatible backbones from {', '.join(self.supported_backbones)}")
+            raise Exception(
+                f"Enter only compatible backbones from {', '.join(self.supported_backbones)}"
+            )
 
         super().__init__(data, backbone, **kwargs)
 
-        
-        n_bands = len(getattr(self._data, '_extract_bands', [0, 1, 2]))
+        n_bands = len(getattr(self._data, "_extract_bands", [0, 1, 2]))
         _backbone = self._backbone
-        if hasattr(self, '_orig_backbone'):
+        if hasattr(self, "_orig_backbone"):
             _backbone = self._orig_backbone
 
         self.name = "RetinaNet"
         self._code = code
 
-        self.scales = ifnone(scales, [1,2**(-1/3), 2**(-2/3)])
-        self.ratios = ifnone(ratios, [1/2,1,2])
+        self.scales = ifnone(scales, [1, 2 ** (-1 / 3), 2 ** (-2 / 3)])
+        self.ratios = ifnone(ratios, [1 / 2, 1, 2])
         self._n_anchors = len(self.scales) * len(self.ratios)
 
         self._data = data
-        self._chip_size = (data.chip_size,data.chip_size)
+        self._chip_size = (data.chip_size, data.chip_size)
 
         # Cut-off the backbone before the penultimate layer
-        self._encoder = create_body(self._backbone, -2)
+        self._encoder = create_body(self._backbone, backbone_pretrained)
 
-        # Initialize the model, loss function and the Learner object        
-        self._model = RetinaNetModel(self._encoder, n_classes=data.c-1, final_bias=-4, chip_size=self._chip_size, n_anchors=self._n_anchors, n_bands=n_bands)
-        self._loss_f = RetinaNetFocalLoss(sizes=self._model.sizes, scales=self.scales, ratios=self.ratios, device=self._device)
+        # Initialize the model, loss function and the Learner object
+        self._model = RetinaNetModel(
+            self._encoder,
+            n_classes=data.c - 1,
+            final_bias=-4,
+            chip_size=self._chip_size,
+            n_anchors=self._n_anchors,
+            n_bands=n_bands,
+        )
+        self._loss_f = RetinaNetFocalLoss(
+            sizes=self._model.sizes,
+            scales=self.scales,
+            ratios=self.ratios,
+            device=self._device,
+        )
         self.learn = Learner(data, self._model, loss_func=self._loss_f)
+        self.learn.metrics = [AveragePrecision(self, data.c - 1)]
         self.learn.split([self._model.encoder[6], self._model.c5top5])
         self.learn.freeze()
         if pretrained_path is not None:
             self.load(str(pretrained_path))
-        self._arcgis_init_callback() # make first conv weights learnable
+        self._arcgis_init_callback()  # make first conv weights learnable
 
     def __str__(self):
         return self.__repr__()
 
     def __repr__(self):
-        return '<%s>' % (type(self).__name__)
+        return "<%s>" % (type(self).__name__)
+
+    @staticmethod
+    def _available_metrics():
+        return ["valid_loss", "average_precision"]
 
     # Return a list of supported backbones names
     @property
     def supported_backbones(self):
-        """ Supported torchvision backbones for this model. """        
+        """Supported torchvision backbones for this model."""
         return RetinaNet._supported_backbones()
 
     @staticmethod
@@ -137,13 +180,13 @@ class RetinaNet(ArcGISModel):
         return [*_resnet_family]
 
     @property
-    def  supported_datasets(self):
-        """ Supported dataset types for this model. """
+    def supported_datasets(self):
+        """Supported dataset types for this model."""
         return RetinaNet._supported_datasets()
-    
+
     @staticmethod
     def _supported_datasets():
-        return ['PASCAL_VOC_rectangles', 'KITTI_rectangles']     
+        return ["PASCAL_VOC_rectangles", "KITTI_rectangles"]
 
     def _get_emd_params(self, save_inference_file):
         _emd_template = {}
@@ -151,32 +194,44 @@ class RetinaNet(ArcGISModel):
         if save_inference_file:
             _emd_template["InferenceFunction"] = "ArcGISObjectDetector.py"
         else:
-            _emd_template["InferenceFunction"] = "[Functions]System\\DeepLearning\\ArcGISLearn\\ArcGISObjectDetector.py"
+            _emd_template[
+                "InferenceFunction"
+            ] = "[Functions]System\\DeepLearning\\ArcGISLearn\\ArcGISObjectDetector.py"
         _emd_template["ModelConfiguration"] = "_RetinaNet_Inference"
         _emd_template["ModelType"] = "ObjectDetection"
         _emd_template["ExtractBands"] = [0, 1, 2]
-        _emd_template['ModelParameters'] = {}
-        _emd_template['ModelParameters']['scales'] = self._loss_f.scales #Scales and Ratios are attributes of RetinaNetFocalLoss object _loss_f
-        _emd_template['ModelParameters']['ratios'] = self._loss_f.ratios
-        _emd_template['Classes'] = []
+        _emd_template["ModelParameters"] = {}
+        _emd_template["ModelParameters"][
+            "scales"
+        ] = (
+            self._loss_f.scales
+        )  # Scales and Ratios are attributes of RetinaNetFocalLoss object _loss_f
+        _emd_template["ModelParameters"]["ratios"] = self._loss_f.ratios
+        _emd_template["Classes"] = []
 
         class_data = {}
-        for i, class_name in enumerate(self._data.classes[1:]): # 0th index is background
+        for i, class_name in enumerate(
+            self._data.classes[1:]
+        ):  # 0th index is background
             inverse_class_mapping = {v: k for k, v in self._data.class_mapping.items()}
             class_data["Value"] = inverse_class_mapping[class_name]
             class_data["Name"] = class_name
             color = [random.choice(range(256)) for i in range(3)]
             class_data["Color"] = color
-            _emd_template['Classes'].append(class_data.copy())
+            _emd_template["Classes"].append(class_data.copy())
 
         return _emd_template
 
     @property
     def _model_metrics(self):
-        return {'accuracy': self.average_precision_score(show_progress=True)}
+        return {"accuracy": self.average_precision_score(show_progress=True)}
 
-    def _analyze_pred(self, pred, thresh=0.5, nms_overlap=0.1, ret_scores=True, device=None):
-        return get_predictions(pred, crit=self._loss_f, detect_thresh=thresh, nms_overlap=nms_overlap)
+    def _analyze_pred(
+        self, pred, thresh=0.5, nms_overlap=0.1, ret_scores=True, device=None
+    ):
+        return get_predictions(
+            pred, crit=self._loss_f, detect_thresh=thresh, nms_overlap=nms_overlap
+        )
 
     @classmethod
     def from_model(cls, emd_path, data=None):
@@ -193,22 +248,22 @@ class RetinaNet(ArcGISModel):
                                 object from `prepare_data` function or None for
                                 inferencing.
         =====================   ===========================================
-        
-        :returns: `RetinaNet` Object
+
+        :return: `RetinaNet` Object
         """
         if not HAS_FASTAI:
             _raise_fastai_import_error(import_exception=import_exception)
-        emd_path = _get_emd_path(emd_path)    
+        emd_path = _get_emd_path(emd_path)
         emd = json.load(open(emd_path))
-        model_file = Path(emd['ModelFile'])
+        model_file = Path(emd["ModelFile"])
         chip_size = emd["ImageWidth"]
 
         if not model_file.is_absolute():
             model_file = emd_path.parent / model_file
 
-        class_mapping = {i['Value'] : i['Name'] for i in emd['Classes']}
-        
-        resize_to = emd.get('resize_to')
+        class_mapping = {i["Value"]: i["Name"] for i in emd["Classes"]}
+
+        resize_to = emd.get("resize_to")
         if isinstance(resize_to, list):
             resize_to = (resize_to[0], resize_to[1])
 
@@ -222,13 +277,22 @@ class RetinaNet(ArcGISModel):
 
             with warnings.catch_warnings():
                 warnings.simplefilter("ignore", UserWarning)
-                
+
                 sd = ImageList([], path=emd_path.parent.parent.parent).split_by_idx([])
-                data = sd.label_const(0, label_cls=ObjectDetectionCategoryList, classes=list(class_mapping.values())).transform(ds_tfms).databunch(device=_get_device()).normalize(imagenet_stats)
+                data = (
+                    sd.label_const(
+                        0,
+                        label_cls=ObjectDetectionCategoryList,
+                        classes=list(class_mapping.values()),
+                    )
+                    .transform(ds_tfms)
+                    .databunch(device=_get_device())
+                    .normalize(imagenet_stats)
+                )
 
             data.chip_size = chip_size
             data.class_mapping = class_mapping
-            data.classes = ['background'] + list(class_mapping.values())
+            data.classes = ["background"] + list(class_mapping.values())
             data = get_multispectral_data_params_from_emd(data, emd)
             # Add 1 for background class
             data.c += 1
@@ -237,12 +301,12 @@ class RetinaNet(ArcGISModel):
             data.emd = emd
 
         data.resize_to = resize_to
-        ret = cls(data, **emd['ModelParameters'], pretrained_path=model_file)
-        
+        ret = cls(data, **emd["ModelParameters"], pretrained_path=model_file)
+
         if not data_passed:
             ret.learn.data.single_ds.classes = ret._data.classes
             ret.learn.data.single_ds.y.classes = ret._data.classes
-        
+
         return ret
 
     def show_results(self, rows=5, thresh=0.5, nms_overlap=0.1):
@@ -259,36 +323,39 @@ class RetinaNet(ArcGISModel):
                                 a detection will be considered valid.
         ---------------------   -------------------------------------------
         nms_overlap             Optional float. The intersection over union
-                                threshold with other predicted bounding 
+                                threshold with other predicted bounding
                                 boxes, above which the box with the highest
                                 score will be considered a true positive.
         =====================   ===========================================
-        
+
         """
         self._check_requisites()
 
         if rows > len(self._data.valid_ds):
             rows = len(self._data.valid_ds)
 
-        self.learn.show_results(rows=rows, thresh=thresh, nms_overlap=nms_overlap, model=self)
+        self.learn.show_results(
+            rows=rows, thresh=thresh, nms_overlap=nms_overlap, model=self
+        )
         if _IS_ARCGISPRONOTEBOOK:
             plt.show()
 
-    def _show_results_multispectral(self, rows=5, thresh=0.3, nms_overlap=0.1, alpha=1, **kwargs):
-        return_fig = kwargs.get('return_fig', False)
+    def _show_results_multispectral(
+        self, rows=5, thresh=0.3, nms_overlap=0.1, alpha=1, **kwargs
+    ):
+        return_fig = kwargs.get("return_fig", False)
         ret_val = show_results_multispectral(
-            self, 
-            nrows=rows, 
-            thresh=thresh, 
-            nms_overlap=nms_overlap, 
-            alpha=alpha, 
-            **kwargs
+            self,
+            nrows=rows,
+            thresh=thresh,
+            nms_overlap=nms_overlap,
+            alpha=alpha,
+            **kwargs,
         )
         if return_fig:
             fig, ax = ret_val
             return fig
 
-
     def predict_video(
         self,
         input_video_path,
@@ -301,21 +368,22 @@ class RetinaNet(ArcGISModel):
         multiplex=False,
         multiplex_file_path=None,
         tracker_options={
-            'assignment_iou_thrd': 0.3,
-            'vanish_frames': 40,
-            'detect_frames': 10
+            "assignment_iou_thrd": 0.3,
+            "vanish_frames": 40,
+            "detect_frames": 10,
         },
         visual_options={
-            'show_scores': True,
-            'show_labels': True,
-            'thickness': 2,
-            'fontface': 0,
-            'color': (255, 255, 255)
+            "show_scores": True,
+            "show_labels": True,
+            "thickness": 2,
+            "fontface": 0,
+            "color": (255, 255, 255),
         },
-        resize=False
+        resize=False,
     ):
         """
         Runs prediction on a video and appends the output VMTI predictions in the metadata file.
+        This method is only supported for RGB images.
 
         =====================   ===========================================
         **Argument**            **Description**
@@ -368,16 +436,17 @@ class RetinaNet(ArcGISModel):
                                 0-255.
         ---------------------   -------------------------------------------
         resize                  Optional boolean. Resizes the video frames to the same size
-                                (chip_size parameter in prepare_data) that the model was trained on,
-                                before detecting objects.
-                                Note that if resize_to parameter was used in prepare_data,
+                                (chip_size parameter in prepare_data) that the model was
+                                trained on, before detecting objects. Note that if
+                                resize_to parameter was used in prepare_data,
                                 the video frames are resized to that size instead.
 
-                                By default, this parameter is false and the detections are run
-                                in a sliding window fashion by applying the model on cropped sections
-                                of the frame (of the same size as the model was trained on).
+                                By default, this parameter is false and the detections
+                                are run in a sliding window fashion by applying the
+                                model on cropped sections of the frame (of the same
+                                size as the model was trained on).
         =====================   ===========================================
-        
+
         """
 
         VideoUtils.predict_video(
@@ -386,18 +455,28 @@ class RetinaNet(ArcGISModel):
             metadata_file,
             threshold,
             nms_overlap,
-            track, visualize,
+            track,
+            visualize,
             output_file_path,
             multiplex,
             multiplex_file_path,
             tracker_options,
             visual_options,
-            resize
+            resize,
         )
 
-    def predict(self, image_path, threshold=0.5, nms_overlap=0.1, return_scores=True, visualize=False, resize=False):
+    def predict(
+        self,
+        image_path,
+        threshold=0.5,
+        nms_overlap=0.1,
+        return_scores=True,
+        visualize=False,
+        resize=False,
+    ):
         """
         Predicts and displays the results of a trained model on a single image.
+        This method is only supported for RGB images.
 
         =====================   ===========================================
         **Argument**            **Description**
@@ -409,33 +488,38 @@ class RetinaNet(ArcGISModel):
                                 a detection will be considered valid.
         ---------------------   -------------------------------------------
         nms_overlap             Optional float. The intersection over union
-                                threshold with other predicted bounding 
+                                threshold with other predicted bounding
                                 boxes, above which the box with the highest
                                 score will be considered a true positive.
         ---------------------   -------------------------------------------
         return_scores           Optional boolean.
-                                Will return the probability scores of the 
+                                Will return the probability scores of the
                                 bounding box predictions if True.
         ---------------------   -------------------------------------------
-        visualize               Optional boolean. Displays the image with 
+        visualize               Optional boolean. Displays the image with
                                 predicted bounding boxes if True.
         ---------------------   -------------------------------------------
-        resize                  Optional boolean. Resizes the image to the same size
-                                (chip_size parameter in prepare_data) that the model was trained on,
-                                before detecting objects.
-                                Note that if resize_to parameter was used in prepare_data,
-                                the image is resized to that size instead.
+        resize                  Optional boolean. Resizes the image to the
+                                same size (chip_size parameter in prepare_data)
+                                that the model was trained on, before detecting
+                                objects. Note that if resize_to parameter was
+                                used in prepare_data, the image is resized to
+                                that size instead.
 
-                                By default, this parameter is false and the detections are run
-                                in a sliding window fashion by applying the model on cropped sections
-                                of the image (of the same size as the model was trained on).
+                                By default, this parameter is false and the
+                                detections are run in a sliding window fashion
+                                by applying the model on cropped sections of
+                                the image (of the same size as the model was
+                                trained on).
         =====================   ===========================================
-        
-        :returns: 'List' of xmin, ymin, width, height of predicted bounding boxes on the given image
+
+        :return: 'List' of xmin, ymin, width, height of predicted bounding boxes on the given image
         """
 
         if not HAS_OPENCV:
-            raise Exception("This function requires opencv 4.0.1.24. Install it using pip install opencv-python==4.0.1.24")
+            raise Exception(
+                "This function requires opencv 4.0.1.24. Install it using pip install opencv-python==4.0.1.24"
+            )
 
         if isinstance(image_path, str):
             image = cv2.imread(image_path)
@@ -445,8 +529,7 @@ class RetinaNet(ArcGISModel):
         orig_height, orig_width, _ = image.shape
         orig_frame = image.copy()
 
-        if resize and self._data.resize_to is None\
-                and self._data.chip_size is not None:
+        if resize and self._data.resize_to is None and self._data.chip_size is not None:
             image = cv2.resize(image, (self._data.chip_size, self._data.chip_size))
 
         if self._data.resize_to is not None:
@@ -460,7 +543,16 @@ class RetinaNet(ArcGISModel):
         if self._data.chip_size is not None:
             chips = _get_image_chips(image, self._data.chip_size)
         else:
-            chips = [{'width': width, 'height': height, 'xmin': 0, 'ymin': 0, 'chip': image, 'predictions': []}]
+            chips = [
+                {
+                    "width": width,
+                    "height": height,
+                    "xmin": 0,
+                    "ymin": 0,
+                    "chip": image,
+                    "predictions": [],
+                }
+            ]
 
         valid_tfms = self._data.valid_ds.tfms
         self._data.valid_ds.tfms = []
@@ -469,46 +561,82 @@ class RetinaNet(ArcGISModel):
         if len(chips) == 1:
             include_pad_detections = True
 
-        for chip in chips:
-            frame = Image(pil2tensor(PIL.Image.fromarray(cv2.cvtColor(chip['chip'], cv2.COLOR_BGR2RGB)), dtype=np.float32).div_(255))
-            bbox = self.learn.predict(frame, thresh=threshold, nms_overlap=nms_overlap, ret_scores=True, model=self)[0]
-            if bbox:
-                scores = bbox.scores
-                bboxes, lbls = bbox._compute_boxes()
-                bboxes.add_(1).mul_(
-                    torch.tensor([chip['height'] / 2, chip['width'] / 2, chip['height'] / 2, chip['width'] / 2])).long()
-                for index, bbox in enumerate(bboxes):
-                    if lbls is not None:
-                        label = lbls[index]
-                    else:
-                        label = 'Default'
+        from .._utils.pascal_voc_rectangles import modified_getitem
+        from fastai.data_block import LabelList
 
-                    data = bb2hw(bbox)
-                    if include_pad_detections or not _exclude_detection((data[0], data[1], data[2], data[3]), chip['width'], chip['height']):
-                        chip['predictions'].append({
-                            'xmin': data[0],
-                            'ymin': data[1],
-                            'width': data[2],
-                            'height': data[3],
-                            'score': float(scores[index]),
-                            'label': label
-                        })
+        orig_getitem = LabelList.__getitem__
+        LabelList.__getitem__ = modified_getitem
+
+        try:
+            for chip in chips:
+                frame = Image(
+                    pil2tensor(
+                        PIL.Image.fromarray(
+                            cv2.cvtColor(chip["chip"], cv2.COLOR_BGR2RGB)
+                        ),
+                        dtype=np.float32,
+                    ).div_(255)
+                )
+                bbox = self.learn.predict(
+                    frame,
+                    thresh=threshold,
+                    nms_overlap=nms_overlap,
+                    ret_scores=True,
+                    model=self,
+                )[0]
+                if bbox:
+                    scores = bbox.scores
+                    bboxes, lbls = bbox._compute_boxes()
+                    bboxes.add_(1).mul_(
+                        torch.tensor(
+                            [
+                                chip["height"] / 2,
+                                chip["width"] / 2,
+                                chip["height"] / 2,
+                                chip["width"] / 2,
+                            ]
+                        )
+                    ).long()
+                    for index, bbox in enumerate(bboxes):
+                        if lbls is not None:
+                            label = lbls[index]
+                        else:
+                            label = "Default"
+
+                        data = bb2hw(bbox)
+                        if include_pad_detections or not _exclude_detection(
+                            (data[0], data[1], data[2], data[3]),
+                            chip["width"],
+                            chip["height"],
+                        ):
+                            chip["predictions"].append(
+                                {
+                                    "xmin": data[0],
+                                    "ymin": data[1],
+                                    "width": data[2],
+                                    "height": data[3],
+                                    "score": float(scores[index]),
+                                    "label": label,
+                                }
+                            )
+        finally:
+            LabelList.__getitem__ = orig_getitem
 
         self._data.valid_ds.tfms = valid_tfms
 
         predictions, labels, scores = _get_transformed_predictions(chips)
 
         # Scale the predictions to original image and clip the predictions to image dims
-        y_ratio = orig_height/height
-        x_ratio = orig_width/width
+        y_ratio = orig_height / height
+        x_ratio = orig_width / width
         for index, prediction in enumerate(predictions):
-            prediction[0] = prediction[0]*x_ratio
-            prediction[1] = prediction[1]*y_ratio
-            prediction[2] = prediction[2]*x_ratio
-            prediction[3] = prediction[3]*y_ratio
+            prediction[0] = prediction[0] * x_ratio
+            prediction[1] = prediction[1] * y_ratio
+            prediction[2] = prediction[2] * x_ratio
+            prediction[3] = prediction[3] * y_ratio
 
             # Clip xmin
-            if prediction[0] < 0: 
+            if prediction[0] < 0:
                 prediction[2] = prediction[2] + prediction[0]
                 prediction[0] = 1
 
@@ -529,12 +657,13 @@ class RetinaNet(ArcGISModel):
                 prediction[0],
                 prediction[1],
                 prediction[2],
-                prediction[3]
+                prediction[3],
             ]
 
         if visualize:
             image = _draw_predictions(orig_frame, predictions, labels)
             import matplotlib.pyplot as plt
+
             plt.xticks([])
             plt.yticks([])
             image = cv2.cvtColor(image, cv2.COLOR_BGR2RGB)
@@ -545,7 +674,9 @@ class RetinaNet(ArcGISModel):
         else:
             return predictions, labels
 
-    def average_precision_score(self, detect_thresh=0.5, iou_thresh=0.1, mean=False, show_progress=True):
+    def average_precision_score(
+        self, detect_thresh=0.5, iou_thresh=0.1, mean=False, show_progress=True
+    ):
         """
         Computes average precision on the validation set for each class.
 
@@ -563,14 +694,21 @@ class RetinaNet(ArcGISModel):
         ---------------------   -------------------------------------------
         mean                    Optional bool. If False returns class-wise
                                 average precision otherwise returns mean
-                                average precision.                        
+                                average precision.
         =====================   ===========================================
-        
-        :returns: `dict` if mean is False otherwise `float`
+
+        :return: `dict` if mean is False otherwise `float`
         """
         self._check_requisites()
-        aps = compute_class_AP(self, self._data.valid_dl, self._data.c - 1, show_progress, detect_thresh=detect_thresh, iou_thresh=iou_thresh)
+        aps = compute_class_AP(
+            self,
+            self._data.valid_dl,
+            self._data.c - 1,
+            show_progress,
+            detect_thresh=detect_thresh,
+            iou_thresh=iou_thresh,
+        )
         if mean:
             return statistics.mean(aps)
         else:
-            return dict(zip(self._data.classes[1:], aps))
\ No newline at end of file
+            return dict(zip(self._data.classes[1:], aps))
