@@ -11,20 +11,26 @@ class _TSLSTM(nn.Module):
 
         self.linear = nn.Linear(hidden_layer_size, output_size)
         self.device = device
-        self.hidden_cell = (torch.zeros(1, 1, self.hidden_layer_size).to(device),
-                            torch.zeros(1, 1, self.hidden_layer_size).to(device))
+        self.hidden_cell = (
+            torch.zeros(1, 1, self.hidden_layer_size).to(device),
+            torch.zeros(1, 1, self.hidden_layer_size).to(device),
+        )
 
     def forward(self, input_seq):
         outputs = []
         for batch in input_seq:
             batch = batch.squeeze()
-            self.hidden_cell = (torch.zeros(1, 1, self.hidden_layer_size).to(self.device),
-                                torch.zeros(1, 1, self.hidden_layer_size).to(self.device))
+            self.hidden_cell = (
+                torch.zeros(1, 1, self.hidden_layer_size).to(self.device),
+                torch.zeros(1, 1, self.hidden_layer_size).to(self.device),
+            )
 
-            lstm_out, self.hidden_cell = self.lstm(batch.view(len(batch), 1, -1).to(self.device), self.hidden_cell)
+            lstm_out, self.hidden_cell = self.lstm(
+                batch.view(len(batch), 1, -1).to(self.device), self.hidden_cell
+            )
             predictions = self.linear(lstm_out.view(len(batch), -1))
             outputs += [predictions[-1]]
 
         outputs = torch.stack(outputs, 1)
 
-        return outputs.to(self.device)
\ No newline at end of file
+        return outputs.to(self.device)
