@@ -1,3 +1,3 @@
 from ._lyrs import Object3DLayer, IntegratedMeshLayer, Point3DLayer
 from ._lyrs import PointCloudLayer, BuildingLayer
-from ._lyrs import SceneLayer
\ No newline at end of file
+from ._lyrs import SceneLayer, SceneLayerManager, EnterpriseSceneLayerManager
