@@ -1,3 +1,5 @@
-from arcgis.widgets._mapview._raster.engine.matplotlib_raster_engine import MatplotlibRasterEngine
+from arcgis.widgets._mapview._raster.engine.matplotlib_raster_engine import (
+    MatplotlibRasterEngine,
+)
 from arcgis.widgets._mapview._raster.engine.arcpy_raster_engine import ArcPyRasterEngine
 from arcgis.widgets._mapview._raster.engine.rasterio_engine import RasterIOEngine
